//
// C++ Implementation: kpgaggregate
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgaggregate.h"

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <ktextedit.h>

#include <qtable.h>

#include "../kpglinklabel.h"
#include "kpgaggregatesfolder.h"
#include "../PropertyPages/kpgaggregateproppage.h"

KPGAggregate::KPGAggregate(KPGAggregatesFolder *parent, const QString name)
  : KPGTreeItem(parent, name)
{
  setPixmap(0, *m_pIconAggregate);
}

KPGAggregate::KPGAggregate(KPGAggregatesFolder *parent, KPGAggregate *after, const QString name)
  : KPGTreeItem(parent, after, name)
{
  setPixmap(0, *m_pIconAggregate);
}

KPGAggregate::~KPGAggregate()
{
}

void KPGAggregate::setProperties(
		const QString strDescription,
		pqxx::oid oidAggregateFn,
		const QString strAggregateFn,
		const QString strOwner,
		pqxx::oid oidTransFn,
		const QString strTransFn,
		pqxx::oid oidFinalFn,
		const QString strFinalFn,
		pqxx::oid oidInputType,
		const QString strInputTypName,
		pqxx::oid oidTransType,
		const QString strTransTypName,
		pqxx::oid oidRetType,
		const QString strRetTypName,
		const QString strInitVal
	)
{
	m_strDescription = strDescription;
	m_oidAggregateFn = oidAggregateFn;
	m_strAggregateFn = strAggregateFn;
	m_strOwner = strOwner;
	m_oidTransFn = oidTransFn;
	m_strTransFn = strTransFn;
	m_oidFinalFn = oidFinalFn;
	m_strFinalFn = strFinalFn;
	m_oidInputType = oidInputType;
	m_strInputTypName = strInputTypName;
	m_oidTransType = oidTransType;
	m_strTransTypName = strTransTypName;
	m_oidRetType = oidRetType;
	m_strRetTypName = strRetTypName;
	m_strInitVal = strInitVal;
}

void KPGAggregate::displayProperties(QWidget *widgetProperies)
{
	// Display server version info
	KPGAggregatePropPage *pPropertyPage = static_cast <KPGAggregatePropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
  QHeader* hHeader = pTableProperties->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
  
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Aggregate name"));
	
	//--- Aggregate function
	pTableProperties->setText(iRow, 0, i18n("Aggregate function"));
  KPGLinkLabel *pLabel = new KPGLinkLabel( pTableProperties, "Aggregate function" );
	pLabel->setValues(m_oidAggregateFn, m_strAggregateFn);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Aggregate function name"));
	
	//--- Owner
	pTableProperties->setText(iRow, 0, i18n("Owner"));
  pTableProperties->setText(iRow, 1, m_strOwner);
	pTableProperties->setText(iRow++, 2, i18n("Owner of the aggregate"));
	
	//--- Description
	pTableProperties->setText(iRow, 0, i18n("Description"));
  pTableProperties->setText(iRow, 1, m_strDescription);
	pTableProperties->setText(iRow++, 2, i18n("Description of the aggregate function"));
		
	//--- Transition function
	pTableProperties->setText(iRow, 0, i18n("Transition function"));
  pLabel = new KPGLinkLabel( pTableProperties, "Transition function" );
	pLabel->setValues(m_oidTransFn, m_strTransFn);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Transition function name"));
	
	//--- Final function
	pTableProperties->setText(iRow, 0, i18n("Final function"));
  pLabel = new KPGLinkLabel( pTableProperties, "Final function" );
	pLabel->setValues(m_oidFinalFn, m_strFinalFn);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Final function name"));
	
	//--- Input type
	pTableProperties->setText(iRow, 0, i18n("Input type"));
  pLabel = new KPGLinkLabel( pTableProperties, "Input type" );
	pLabel->setValues(m_oidInputType, m_strInputTypName);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Aggregate function input type name"));
	
	//--- Transition type
	pTableProperties->setText(iRow, 0, i18n("Transition type"));
  pLabel = new KPGLinkLabel( pTableProperties, "Transition type" );
	pLabel->setValues(m_oidTransType, m_strTransTypName);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("The type of the aggregate function's internal transition (state) data"));
	
	//--- Return type
	pTableProperties->setText(iRow, 0, i18n("Return type"));
  pLabel = new KPGLinkLabel( pTableProperties, "Return type" );
	pLabel->setValues(m_oidRetType, m_strRetTypName);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Data type of the aggregate function return value"));
	
	//--- Initial value
	pTableProperties->setText(iRow, 0, i18n("Initial value"));
  pTableProperties->setText(iRow, 1, m_strInitVal);
	pTableProperties->setText(iRow++, 2, i18n("The initial value of the transition state. This is a text field containing the initial value in its external string representation. If the value is null, the transition state value starts out null."));
	
	for(int nCol = 0; nCol < 3; nCol++)
  {
    pTableProperties->adjustColumn(nCol);
  }
}

