//
// C++ Implementation: kpgaggregatesfolder
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgaggregatesfolder.h"

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <qtable.h>

#include "kpgconnection.h"
#include "kpgschema.h"
#include "../PropertyPages/kpgpropertypage.h"
#include "kpgaggregate.h"
#include "../kpgsqldialog.h"

KPGAggregatesFolder::KPGAggregatesFolder(KPGSchema *parent, pqxx::oid oidNameSpace)
 : KPGItemsFolder(parent, i18n("Aggregates"))
{
	setPixmap(0, *m_pIconFolderYellow);
	m_oidNameSpace = oidNameSpace;
}

KPGAggregatesFolder::~KPGAggregatesFolder()
{
}

bool KPGAggregatesFolder::refresh(pqxx::work &pqxxXaction)
{
  // delete all child items (databases)
  while (QListViewItem * pItem = firstChild())
    delete pItem;

	// obtain list of aggregates
  QString strQuery("SELECT pr.proname AS aggrname, aggfnoid::oid AS aggegfnoid, aggfnoid AS aggfnname, pg_get_userbyid(proowner) AS aggrowner, description, aggtransfn::oid AS aggtransfnoid, aggtransfn, aggfinalfn::oid AS aggfinalfnoid, aggfinalfn, "); // proargtypes[0] AS aggbasetype,
		
	strQuery.append("ti.oid AS inputtypoid, ti.typname AS inputtypname, ");
	strQuery.append("tt.oid AS transtypoid, tt.typname AS transtypname, ");
	strQuery.append("tf.oid AS returntypoid, tf.typname AS returntypname, ");
		
	strQuery.append("agginitval ");
	strQuery.append("FROM pg_aggregate ag ");
	strQuery.append("JOIN pg_proc pr ON pr.oid = ag.aggfnoid::oid ");
	strQuery.append("JOIN pg_type ti on ti.oid=pr.proargtypes[0] ");
	strQuery.append("JOIN pg_type tt on tt.oid=aggtranstype ");
	strQuery.append("JOIN pg_type tf on tf.oid=prorettype ");
	strQuery.append("LEFT OUTER JOIN pg_description des ON des.objoid=aggfnoid::oid ");
	strQuery.append("WHERE pr.pronamespace = " + QString("%1").arg(m_oidNameSpace) + " ");
	strQuery.append("ORDER BY aggrname");
			
  try
  {
    m_pqxxResultAggregates = pqxxXaction.Exec(strQuery.latin1());

		KPGAggregate *pgAggregate = 0;
		
    for (result::size_type i = 0; i != m_pqxxResultAggregates.size(); ++i)
    {
			if(pgAggregate == 0)
				pgAggregate = new KPGAggregate(this, m_pqxxResultAggregates[i]["aggrname"].c_str());
			else
				pgAggregate = new KPGAggregate(this, pgAggregate, m_pqxxResultAggregates[i]["aggrname"].c_str());
			
			pqxx::oid oidAggregateFn = 0;
			pqxx::oid oidTransFn = 0;
			pqxx::oid oidFinalFn = 0;
			pqxx::oid oidInputType = 0;
			pqxx::oid oidTransType = 0;
			pqxx::oid oidRetType = 0;
		
			m_pqxxResultAggregates[i]["aggegfnoid"].to(oidAggregateFn);
			m_pqxxResultAggregates[i]["aggtransfnoid"].to(oidTransFn);
			m_pqxxResultAggregates[i]["aggfinalfnoid"].to(oidFinalFn);
			m_pqxxResultAggregates[i]["inputtypoid"].to(oidInputType);
			m_pqxxResultAggregates[i]["transtypoid"].to(oidTransType);
			m_pqxxResultAggregates[i]["returntypoid"].to(oidRetType);
								
			pgAggregate->setProperties(
				m_pqxxResultAggregates[i]["description"].c_str(),
				oidAggregateFn,
				m_pqxxResultAggregates[i]["aggfnname"].c_str(),
				m_pqxxResultAggregates[i]["aggrowner"].c_str(),
				oidTransFn,
				m_pqxxResultAggregates[i]["aggtransfn"].c_str(),
				oidFinalFn,
				m_pqxxResultAggregates[i]["aggfinalfn"].c_str(),
				oidInputType,
				m_pqxxResultAggregates[i]["inputtypname"].c_str(),
				oidTransType,
				m_pqxxResultAggregates[i]["transtypname"].c_str(),
				oidRetType,
				m_pqxxResultAggregates[i]["returntypname"].c_str(),
				m_pqxxResultAggregates[i]["agginitval"].c_str()
				);
    }
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return false;
  } 
  
  return true;
}
void KPGAggregatesFolder::displayProperties(QWidget *widgetProperies)
{
	KPGPropertyPage *pPropertyPage = static_cast <KPGPropertyPage *> (widgetProperies);
  pPropertyPage->setLabelText(i18n("Agregates &list:"));
	
	//----------------------------------------------------------------------------
	//Display aggregates list
		
  QTable *pTableAggregates = pPropertyPage->m_pTableItems;
  
 	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pqxxResultAggregates, pTableAggregates);

  // field names
  QHeader* hHeader = pTableAggregates->horizontalHeader();

  int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
	hHeader->setLabel(nCol++, i18n("Aggregate function"));
  hHeader->setLabel(nCol++, i18n("Owner"));
	hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Transition function"));
  hHeader->setLabel(nCol++, i18n("Final function"));
	hHeader->setLabel(nCol++, i18n("Input type"));
	hHeader->setLabel(nCol++, i18n("Transition type"));
	hHeader->setLabel(nCol++, i18n("Return type"));
	hHeader->setLabel(nCol++, i18n("Initial value"));
	
  for(int nCol = 0; nCol < nTableCols; nCol++)
  {
    pTableAggregates->adjustColumn(nCol);
  }
}




