//
// C++ Implementation: kpgcast
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgcast.h"

// include files for KDE
#include <kiconloader.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <qtable.h>

#include "../kpglinklabel.h"
#include "kpgcastsfolder.h"
#include "../PropertyPages/kpgcastproppage.h"

KPGCast::KPGCast(KPGCastsFolder *parent, const QString name, pqxx::oid _oid)
  : KPGObject(parent, name, _oid)
{
  setPixmap(0, *m_pIconCast);
}

KPGCast::KPGCast(KPGCastsFolder *parent, KPGCast *after, const QString name, pqxx::oid _oid)
  : KPGObject(parent, after, name, _oid)
{
  setPixmap(0, *m_pIconCast);
}

KPGCast::~KPGCast()
{
}

void KPGCast::setProperties(
		const QString strContext,
		pqxx::oid oidSourceType,
		const QString strSourceType,
		pqxx::oid oidTargetType,
		const QString strTargetType,
		pqxx::oid oidCastFunc,
		const QString strCastFunc,
		pqxx::oid oidNameSpace,
		const QString strNameSpace
	)
{
	m_strContext = strContext;
	m_oidSourceType = oidSourceType;
	m_strSourceType = strSourceType;
	m_oidTargetType = oidTargetType;
	m_strTargetType = strTargetType;
	m_oidCastFunc = oidCastFunc;
	m_strCastFunc = strCastFunc;
	m_oidNameSpace = oidNameSpace;
	m_strNameSpace = strNameSpace;
}

void KPGCast::displayProperties(QWidget *widgetProperies)
{
	// Display cast info
	KPGCastPropPage *pPropertyPage = static_cast <KPGCastPropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
  QHeader* hHeader = pTableProperties->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
		  	
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
	
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Context"));
  pTableProperties->setText(iRow, 1, m_strContext);
	pTableProperties->setText(iRow++, 2, i18n("Indicates what contexts the cast may be invoked in. Posible values:  Explicit cast (using CAST, ::, or function-call syntax). Implicitly in assignment to a target column, as well as explicitly. Implicitly in expressions, as well as the other cases."));
	
	//--- Source type
	pTableProperties->setText(iRow, 0, i18n("Source type"));
  KPGLinkLabel *pLabel = new KPGLinkLabel( pTableProperties, "Source type" );
	pLabel->setValues(m_oidSourceType, m_strSourceType);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Source data type"));
	
	//--- Target type
	pTableProperties->setText(iRow, 0, i18n("Target type"));
  pLabel = new KPGLinkLabel( pTableProperties, "Source type" );
	pLabel->setValues(m_oidTargetType, m_strTargetType);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Target data type"));
	
	//--- Function
	pTableProperties->setText(iRow, 0, i18n("Function"));
  pLabel = new KPGLinkLabel( pTableProperties, "Function" );
	pLabel->setValues(m_oidCastFunc, m_strCastFunc);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Function to use to perform this cast. Nothing if the data types are binary compatible (that is, no run-time operation is needed to perform the cast). "));
	
	//--- Namespace
	pTableProperties->setText(iRow, 0, i18n("Namespace"));
  pLabel = new KPGLinkLabel( pTableProperties, "Namespace" );
	pLabel->setValues(m_oidNameSpace, m_strNameSpace);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Cast function namespace"));	
	
	for(int nCol = 0; nCol < 3; nCol++)
  {
    pTableProperties->adjustColumn(nCol);
  }
}


