//
// C++ Implementation: kpgcastsfolder
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgcastsfolder.h"

#include <qtable.h>

#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include "kpgconnection.h"
#include "kpgcast.h"
#include "kpgdatabase.h"
#include "../PropertyPages/kpgpropertypage.h"
#include "../kpgsqldialog.h"

KPGCastsFolder::KPGCastsFolder(KPGDatabase *parent)
 : KPGItemsFolder(parent, i18n("Casts"))
{
	setPixmap(0, *m_pIconFolderGreen);
}


KPGCastsFolder::~KPGCastsFolder()
{
}

bool KPGCastsFolder::refresh(pqxx::work &pqxxXaction)
{
  // delete all child items
  while (QListViewItem * pItem = firstChild())
    delete pItem;

  // obtain list of casts
  QString strQuery("SELECT ca.oid, st.typname || ' -> ' || tt.typname AS name, CASE WHEN ca.castcontext = 'e' THEN 'explicit' WHEN ca.castcontext = 'a' THEN 'implicitly in assignment' WHEN ca.castcontext = 'i' THEN 'implicitly in expressions' END AS castcontext, castsource, st.typname AS srctype, casttarget, tt.typname AS trgtype, castfunc, pr.proname, pr.pronamespace, nspname ");
	strQuery.append("FROM pg_cast ca ");
	strQuery.append("JOIN pg_type st ON st.oid=castsource ");
	strQuery.append("JOIN pg_type tt ON tt.oid=casttarget ");
	strQuery.append("LEFT JOIN pg_proc pr ON pr.oid=castfunc ");
	strQuery.append("LEFT JOIN pg_namespace na ON na.oid=pr.pronamespace ");
	strQuery.append("ORDER BY st.typname, tt.typname");
		
	try
  {
    m_pqxxResultCasts = pqxxXaction.Exec(strQuery.latin1());

		KPGCast *pgCast = 0;
		
    for (result::size_type i = 0; i != m_pqxxResultCasts.size(); ++i)
    {
			pqxx::oid oid;
			m_pqxxResultCasts[i][0].to(oid);
		
			if(pgCast == 0)
      	pgCast = new KPGCast(this, QString(m_pqxxResultCasts[i]["name"].c_str()), oid);
			else
				pgCast = new KPGCast(this, pgCast, QString(m_pqxxResultCasts[i]["name"].c_str()), oid);
					
			pqxx::oid oidSourceType = 0;
			pqxx::oid oidTargetType = 0;
			pqxx::oid oidCastFunc = 0;
			pqxx::oid oidNameSpace = 0;
			
			m_pqxxResultCasts[i]["castsource"].to(oidSourceType);
			m_pqxxResultCasts[i]["casttarget"].to(oidTargetType);
			m_pqxxResultCasts[i]["castfunc"].to(oidCastFunc);
			m_pqxxResultCasts[i]["pronamespace"].to(oidNameSpace);
						
				
			pgCast->setProperties(
				m_pqxxResultCasts[i]["castcontext"].c_str(), 
				oidSourceType,
				m_pqxxResultCasts[i]["srctype"].c_str(),
				oidTargetType,
				m_pqxxResultCasts[i]["trgtype"].c_str(),
				oidCastFunc,
				m_pqxxResultCasts[i]["proname"].c_str(),
				oidNameSpace,
				m_pqxxResultCasts[i]["nspname"].c_str()
				);
    }
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return false;
  } 
  
  return true;
}

void KPGCastsFolder::displayProperties(QWidget *widgetProperies)
{
	KPGPropertyPage *pPropertyPage = static_cast <KPGPropertyPage *> (widgetProperies);
  pPropertyPage->setLabelText(i18n("Casts &list:"));
	
	//----------------------------------------------------------------------------
	//Display casts list
		
	QTable *pTableCasts = pPropertyPage->m_pTableItems;
  
  // fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pqxxResultCasts, pTableCasts);

  // field names
  QHeader* hHeader = pTableCasts->horizontalHeader();

  int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  hHeader->setLabel(nCol++, i18n("Context"));
  hHeader->setLabel(nCol++, i18n("Source type"));
	hHeader->setLabel(nCol++, i18n("Target type"));
  hHeader->setLabel(nCol++, i18n("Function"));
	hHeader->setLabel(nCol++, i18n("Namespace"));
	
  for(int nCol = 0; nCol < nTableCols; nCol++)
  {
    pTableCasts->adjustColumn(nCol);
  }
}

