/***************************************************************************
                          kpgconnection.cpp  -  description
                             -------------------
    begin                : � led 13 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : dsacher@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for libpgxx 
#include "pqxx/nontransaction.h" 
#include "pqxx/robusttransaction.h" 
 
// include files for Qt
#include <qstring.h>
#include <qstringlist.h>
#include <qtextcodec.h>

// include files for KDE
#include <kdebug.h>
 
#include "kpgconnection.h"


void KPGNoticer::operator()(const char Msg[]) throw ()
{
  if(!m_pListOfNotifications)
	{
		kdDebug() << "KPGNoticer::operator(): m_pListOfNotifications is NULL !" << endl;
		return;
	}
	
	// Add notification to list
	*m_pListOfNotifications << Msg;
}

//--------------------------------------------------------------------------

KPGConnection::KPGConnection(const QString strConnectString)
  : connection(strConnectString.latin1())
{
  KPGNoticer *pMyNoticer = new KPGNoticer;
	pMyNoticer->m_pListOfNotifications = &m_listOfNotifications;
  
  // This is not a memory leak: C stores MyNoticer in an auto_ptr that will delete the object on destruction.
  set_noticer(PGSTD::auto_ptr<noticer> (pMyNoticer));
  
  m_pFileTrace = 0;
}


KPGConnection::~KPGConnection() throw ()
{
	if(is_open())
		disconnect();
    
  if(m_pFileTrace)
  {
    fclose(m_pFileTrace);
  }
}

// Run SQL query, expect only LATIN1 characters in string 
result KPGConnection::runQuery(const QString& strQuery, eTransType transType)
{
 	// run query
  if(transType == eTransNormal)
	{
		work T(*this, "runQuery");
		result R(T.exec(strQuery.latin1()));
		T.commit();
		return R;
	}
	else
	{
		if(transType ==  eTransNone)
			{
				pqxx::nontransaction T(*this, "runQuery");
				result R(T.exec(strQuery.latin1()));
				return R;
			}
	}	
		
	//	eTransRobust:
	pqxx::robusttransaction<> T(*this, "runQuery");
	result R(T.exec(strQuery.latin1()));
	T.commit();
	return R;
}

// Run SQL query, make conversion regard DB encoding
result KPGConnection::runQuery(const QString& strQuery, const QString& strDbEncoding, eTransType transType)
{
 	QTextCodec *pTextCodec = QTextCodec::codecForName(encoderName(strDbEncoding));
	if(!pTextCodec)
		pTextCodec = QTextCodec::codecForLocale(); // hope that correct codec is returned
		
	// run query
  if(transType == eTransNormal)
	{
		work T(*this, "runQuery");
		result R(T.exec(pTextCodec->fromUnicode(strQuery)));
		T.commit();
  	return R;
	}
	else
	{
		if(transType ==  eTransNone)
			{
				pqxx::nontransaction T(*this, "runQuery");
				result R(T.exec(pTextCodec->fromUnicode(strQuery)));
				return R;
			}
	}	
		
	//	eTransRobust:
	pqxx::robusttransaction<> T(*this, "runQuery");
	result R(T.exec(pTextCodec->fromUnicode(strQuery)));
	T.commit();
	return R;
}

// Return encoder name for QTextCodec regards to DB encoding
const QString KPGConnection::encoderName(const QString & strDbEncoding) const
{
	if(strDbEncoding == "LATIN1") return "iso-8859-1";
	if(strDbEncoding == "LATIN2") return "iso-8859-2";
	if(strDbEncoding == "LATIN3") return "iso-8859-3";
	if(strDbEncoding == "LATIN4") return "iso-8859-4";
	if(strDbEncoding == "LATIN5") return "iso-8859-5";	
	if(strDbEncoding == "LATIN6") return "iso-8859-6";
	if(strDbEncoding == "LATIN7") return "iso-8859-7";
	if(strDbEncoding == "LATIN8") return "iso-8859-8";
	if(strDbEncoding == "LATIN9") return "iso-8859-9";
	
	return QString(strDbEncoding);
}

// Enable tracing to file
void KPGConnection::setTrace()
{
  if(m_pFileTrace)
  {
    kdError() << "KPGConnection::setTrace : trace already set" << endl;
    return;
  }
  
  m_pFileTrace = fopen("/tmp/KPoGre.log","w");
  
  trace((FILE *) m_pFileTrace);
}

// Clear notifications
void KPGConnection::clearNotifications()
{
	m_listOfNotifications.clear();
}
