//
// C++ Implementation: kpgconversion
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgconversion.h"

// include files for KDE
#include <kiconloader.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <qtable.h>

#include "../kpglinklabel.h"
#include "kpgconversionsfolder.h"
#include "../PropertyPages/kpgconversionproppage.h"


KPGConversion::KPGConversion(KPGConversionsFolder *parent, const QString name, pqxx::oid _oid)
  : KPGObject(parent, name, _oid)
{
  setPixmap(0, *m_pIconConversion);
}

KPGConversion::KPGConversion(KPGConversionsFolder *parent, KPGConversion *after, const QString name, pqxx::oid _oid)
  : KPGObject(parent, after, name, _oid)
{
  setPixmap(0, *m_pIconConversion);
}

KPGConversion::~KPGConversion()
{
}

void KPGConversion::setProperties(
	bool bIsDefault,
  const QString strSourceEncoding,
	const QString strDestinationEncoding,
	pqxx::oid oidConversionProcedure,
 	const QString strConversionProcedure,
	const QString strOwner
	)
{
	m_bIsDefault = bIsDefault;
  m_strSourceEncoding = strSourceEncoding;
	m_strDestinationEncoding = strDestinationEncoding;
 	m_oidConversionProcedure = oidConversionProcedure;
	m_strConversionProcedure = strConversionProcedure;
	m_strOwner = strOwner;	
}

void KPGConversion::displayProperties(QWidget *widgetProperies)
{
	// Display conversion info
	KPGConversionPropPage *pPropertyPage = static_cast <KPGConversionPropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
  QHeader* hHeader = pTableProperties->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
  	
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
	
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Conversion name"));
	
	//--- Is Default
	pTableProperties->setText(iRow, 0, i18n("Is Default"));
  pTableProperties->setText(iRow, 1, (m_bIsDefault) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("True if this is the default conversion"));
	
	//--- Source encoding
	pTableProperties->setText(iRow, 0, i18n("Source encoding"));
  pTableProperties->setText(iRow, 1, m_strSourceEncoding);
	pTableProperties->setText(iRow++, 2, i18n("Source encoding"));

	//--- Destination encoding
	pTableProperties->setText(iRow, 0, i18n("Destination encoding"));
  pTableProperties->setText(iRow, 1, m_strDestinationEncoding);
	pTableProperties->setText(iRow++, 2, i18n("Destination encoding"));
	
	//--- Conversion procedure
	pTableProperties->setText(iRow, 0, i18n("Conversion procedure"));
  KPGLinkLabel *pLabel = new KPGLinkLabel( pTableProperties, "Conversion procedure" );
	pLabel->setValues(m_oidConversionProcedure, m_strConversionProcedure);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Conversion procedure name"));
	
	//--- Owner
	pTableProperties->setText(iRow, 0, i18n("Owner"));
  pTableProperties->setText(iRow, 1, m_strOwner);
	pTableProperties->setText(iRow++, 2, i18n("Owner of the conversion"));
	
		
	
	for(int nCol = 0; nCol < 3; nCol++)
  {
    pTableProperties->adjustColumn(nCol);
  }
}


