/***************************************************************************
                          kpgdatabase.cpp  -  description
                             -------------------
    begin                : � led 6 2004
    copyright            : (C) 2004 by Lummir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qtable.h>
#include <qtabwidget.h>

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kcursor.h>
#include <kcombobox.h> 

#include "kpgitemsfolder.h"
#include "kpgdatabase.h"
#include "kpgconnection.h"
#include "kpgserver.h"
#include "kpgschema.h"
#include "kpgcastsfolder.h"
#include "kpglanguagesfolder.h"

#include "../PropertyPages/kpgdatabaseproppage.h"
#include "../kpgsqldialog.h"
#include "../kpglinklabel.h"

KPGDatabase::KPGDatabase(
  KPGServer *parent,
  const QString name,
	pqxx::oid _oid
  )
  : KPGObject(parent, name, _oid)
{
  setPixmap(0, *m_pIconDatabase);
	m_pDbConnection = 0;
	
	m_pFolderCasts = 0;
	m_pFolderLanguages = 0;
  
  m_bIsSystemObject = false;	
}

KPGDatabase::KPGDatabase(
  KPGServer *parent,
  KPGDatabase *after,
  const QString name,
  pqxx::oid _oid
  )
  : KPGObject(parent, after, name, _oid)
{
  setPixmap(0, *m_pIconDatabase);
  m_pDbConnection = 0;
  
  m_pFolderCasts = 0;
  m_pFolderLanguages = 0; 
  
  m_bIsSystemObject = false;
}

KPGDatabase::~KPGDatabase()
{
  if(m_pDbConnection)
   delete m_pDbConnection;
}

void KPGDatabase::activate()
{
  if(!m_pDbConnection && m_bAllowConnections)
    { 
		  listView()->setCursor(KCursor::waitCursor());
			if(connectToDB())
			{
				if(m_bIsTemplate)
					{
            setPixmap(0, *m_pIconDatabaseSys);
            m_bIsSystemObject = true;
           }
				else
				  setPixmap(0, *m_pIconDatabaseUsr);
			}
			listView()->setCursor(KCursor::arrowCursor());
		}
}

bool KPGDatabase::connectToDB()
{

  if(m_pDbConnection)
    return true; // already connected

  try
  {
    m_pDbConnection = new KPGConnection(connectionString());
		//emit sigDbConnected(strConnectString);
  }
  catch (const std::exception &e)
  {
    kdError() << "Failed to open database " << endl;
    KMessageBox::sorry(0, e.what());
    return false;
  }  

  refresh();

  return true;
}

void KPGDatabase::disconnectFromDB()
{
	setPixmap(0, *m_pIconDatabase);
	
	// delete all child items
  while(QListViewItem * pItem = firstChild())
    delete pItem;
		
	if(!m_pDbConnection)
    return; // already disconnected
		
	delete m_pDbConnection;
	m_pDbConnection = 0;
}

const QString KPGDatabase::connectionString()
{
	KPGTreeItem *parentItem = static_cast <KPGTreeItem *> (parent());
  if(parentItem->type() != nodeServer)
  {
    kdError() << "KPGDatabase::connectToDB cannot find parent server." << endl;
		return "";
  }
	
	KPGServer *parentServer = static_cast <KPGServer *> (parentItem);
  KPGConnection *connServer = parentServer->connection();
  if(!connServer)
  {
    kdError() << "KPGDatabase::connectToDB parent server has no connection." << endl;
		return "";
  }
	
	
	QString strConnectString = makeConnectionString(
    connServer->hostname(),
    connServer->port(),
    text(0),
    connServer->username(),
    parentServer->password()
    );
		
	return strConnectString;
}

bool KPGDatabase::refresh()
{
	// delete all child items
  while(QListViewItem * pItem = firstChild())
    delete pItem;

	if(m_pDbConnection == 0)
		return true;	
			
	//--- Folders for casts and languages
	m_pFolderCasts = new KPGCastsFolder(this);
	m_pFolderLanguages = new KPGLanguagesFolder(this);	
  
  // Get pointer to server for version info
  KPGServer *pServer = static_cast <KPGServer *> (parent());
	
	// obtain list of schemas
  QString strQuery("SELECT nsp.oid, nsp.nspname, description, nspacl, has_schema_privilege(nsp.oid, 'CREATE') as cancreate, pg_get_userbyid(nspowner) AS owner ");
  
  if(pServer->versionMajor() >= 8)
    strQuery.append(", nsp.nsptablespace, ts.spcname ");
  
	strQuery.append("FROM pg_namespace nsp ");
	strQuery.append("LEFT OUTER JOIN pg_description des ON des.objoid=nsp.oid ");
  
  if(pServer->versionMajor() >= 8)
    strQuery.append("LEFT JOIN pg_tablespace ts ON nsp.nsptablespace=ts.oid ");
    
	strQuery.append("ORDER BY nspname");
			
  try
  {
	  work pqxxXaction(*m_pDbConnection, "refreshSchemasList");		
		
    m_pqxxResultSchemas = pqxxXaction.Exec(strQuery.latin1());

		KPGSchema *pgSchema = 0;
		
    for (result::size_type i = 0; i != m_pqxxResultSchemas.size(); ++i)
    {
			pqxx::oid oid;
			m_pqxxResultSchemas[i][0].to(oid);
		
      if(pgSchema == 0)
				pgSchema = new KPGSchema(this, m_pqxxResultSchemas[i]["nspname"].c_str(), oid);
			else
				pgSchema = new KPGSchema(this, pgSchema, m_pqxxResultSchemas[i]["nspname"].c_str(), oid);
			
			bool bCanCreate;
			m_pqxxResultSchemas[i]["cancreate"].to(bCanCreate);
			
			if(pServer->versionMajor() <= 7)
			{
				pgSchema->setProperties(
					m_pqxxResultSchemas[i]["owner"].c_str(),
					bCanCreate, 
					m_pqxxResultSchemas[i]["nspacl"].c_str(),
					m_pqxxResultSchemas[i]["description"].c_str()
					);
			}
			else
			{
				pqxx::oid oidTablespace;
				m_pqxxResultSchemas[i]["nsptablespace"].to(oidTablespace);
				
				pgSchema->setProperties(
					m_pqxxResultSchemas[i]["owner"].c_str(),
					bCanCreate, 
					m_pqxxResultSchemas[i]["nspacl"].c_str(),
					m_pqxxResultSchemas[i]["description"].c_str(),
					oidTablespace,
					m_pqxxResultSchemas[i]["spcname"].c_str()
					);
			}
				
			//kdDebug() << "------- START: " << m_pqxxResultSchemas[i]["nspname"].c_str() << endl;	
			pgSchema->refresh(pqxxXaction);
			//kdDebug() << "--------END: " << m_pqxxResultSchemas[i]["nspname"].c_str() << endl;
    }
		
		//--- Folders for casts and languages
		m_pFolderCasts->refresh(pqxxXaction);
		m_pFolderLanguages->refresh(pqxxXaction);
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return false;
  } 
	
	setOpen(true);
		
	return true;
}

// Refresh only database info, without childs objects
bool KPGDatabase::refreshItem()
{
  KPGServer *pServer = static_cast <KPGServer *> (parent());
  work pqxxXaction(*m_pDbConnection, "refreshItem");   
  
  QString strQuery("SELECT datname, pg_get_userbyid(datdba) AS datowner, datacl");
  
  if(pServer->versionMajor() <= 7)
    strQuery.append(", datpath "); // deprecated in PostgreSQL 8
  else
     strQuery.append(", dattablespace, spcname ");
     
  strQuery.append(", pg_encoding_to_char(encoding) AS serverencoding, ");
  strQuery.append("has_database_privilege(db.oid, 'CREATE') as cancreate, datconfig, datallowconn, datistemplate ");
      
  strQuery.append("FROM pg_database db ");
  
  if(pServer->versionMajor() >= 8)
    strQuery.append("LEFT JOIN pg_tablespace ts ON db.dattablespace=ts.oid ");

  strQuery.append("WHERE db.oid = " + QString("%1").arg(m_oid));  
    
  try
  {
    pqxx::result pqxxResultDatabases = pqxxXaction.Exec(strQuery.latin1());
    
    if(pqxxResultDatabases.size() != 1)
    {
      kdDebug() << "KPGDatabase::refreshItem(): expect one row in result" << endl;
      return false;
    }
    
    // Get refreshed properties
    bool bCanCreate;
    bool bAllowConnections;
    bool bIsTemplate;
            
    pqxxResultDatabases[0]["cancreate"].to(bCanCreate);
    pqxxResultDatabases[0]["datallowconn"].to(bAllowConnections);
    pqxxResultDatabases[0]["datistemplate"].to(bIsTemplate);
        
    if(pServer->versionMajor() <= 7)
    {             
      setProperties(
          pqxxResultDatabases[0]["datowner"].c_str(), 
          pqxxResultDatabases[0]["datacl"].c_str(), 
          pqxxResultDatabases[0]["datpath"].c_str(), 
          pqxxResultDatabases[0]["serverencoding"].c_str(), 
          bCanCreate,
          pqxxResultDatabases[0]["datconfig"].c_str(), 
          bAllowConnections,
          bIsTemplate
        );
        }
    else
    {  
      pqxx::oid oidTablespace;
      pqxxResultDatabases[0]["dattablespace"].to(oidTablespace);
                    
      setProperties(
          pqxxResultDatabases[0]["datowner"].c_str(), 
          pqxxResultDatabases[0]["datacl"].c_str(),
          oidTablespace, 
          pqxxResultDatabases[0]["spcname"].c_str(), 
          pqxxResultDatabases[0]["serverencoding"].c_str(), 
          bCanCreate,
          pqxxResultDatabases[0]["datconfig"].c_str(), 
          bAllowConnections,
          bIsTemplate
        );
    }
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
    dlg.exec();
    return false;
  } 
  
  return true;
}

pqxx::result KPGDatabase::runQuery(const QString strQuery, KPGConnection::eTransType transType)
{
 	if(m_pDbConnection == 0)
	{
		kdError() << "KPGDatabase::runQuery Database not connected to server !" << endl;
   	KMessageBox::sorry(0, "Database not connected to server !");
		pqxx::result R;
		return R;
	}
		
  return m_pDbConnection->runQuery(strQuery, m_strEncoding, transType);
}

// Set database properties - for PostgreSQL 7.x or older
void KPGDatabase::setProperties(
	const QString strOwner, 
	const QString strACL, 
	const QString strPath, 
	const QString strEncoding, 
	bool bCanCreate, 
	const QString strDefaultVariables, 
	bool bAllowConnections, 
	bool bIsTemplate
	)
{
	m_strOwner = strOwner;
	m_strACL = strACL;
	m_strPath = strPath;
	m_strEncoding = strEncoding;
	m_bCanCreate = bCanCreate;
	m_strDefaultVariables = strDefaultVariables;
  m_bAllowConnections = bAllowConnections;
	m_bIsTemplate = bIsTemplate; 
}

// Set database properties - for PostgreSQL 8 or newer
void KPGDatabase::setProperties(
  const QString strOwner, 
  const QString strACL, 
  pqxx::oid oidTablespace,
  const QString strTablespace,
  const QString strEncoding, 
  bool bCanCreate, 
  const QString strDefaultVariables, 
  bool bAllowConnections, 
  bool bIsTemplate
  )
{
  m_strOwner = strOwner;
  m_strACL = strACL;
  m_oidTablespace = oidTablespace;
  m_strTablespace = strTablespace;
  m_strEncoding = strEncoding;
  m_bCanCreate = bCanCreate;
  m_strDefaultVariables = strDefaultVariables;
  m_bAllowConnections = bAllowConnections;
  m_bIsTemplate = bIsTemplate; 
}

void KPGDatabase::displayProperties(QWidget *widgetProperies)
{
  // Get pointer to server for version info
	KPGServer *pServer = static_cast <KPGServer *> (parent());
    
  // Display database properties
	KPGDatabasePropPage *pPropertyPage = static_cast <KPGDatabasePropPage *> (widgetProperies);
	
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	if(m_pDbConnection == 0)
		{ 
			pTableProperties->setNumRows(0);
			
			KPGDatabasePropPage *pPropertyPage = static_cast <KPGDatabasePropPage *> (widgetProperies);
	
			QTable *pTableStatistics = pPropertyPage->m_pTableStatistics;
			pTableStatistics->setNumRows(0);
			
			pPropertyPage->m_pTabWidget->setEnabled(false);
		  return;
		}
	
	//--- Header
	// field names
  QHeader* hHeader = pTableProperties->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
  
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
		
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Database name"));
	
	//--- Owner
	pTableProperties->setText(iRow, 0, i18n("Owner"));
  pTableProperties->setText(iRow, 1, m_strOwner);
	pTableProperties->setText(iRow++, 2, i18n("Owner of the database, usually the user who created it"));
	
	//--- ACL
	pTableProperties->setText(iRow, 0, i18n("ACL"));
  pTableProperties->setText(iRow, 1, m_strACL);
	pTableProperties->setText(iRow++, 2, i18n("Access privileges"));
	
	//--- Path / Tablespace
  if(pServer->versionMajor() <= 7)
	{
    pTableProperties->setText(iRow, 0, i18n("Path"));
    pTableProperties->setText(iRow, 1, m_strPath);
	  pTableProperties->setText(iRow++, 2, i18n("If the database is stored at an alternative location then this records the location. It's either an environment variable name or an absolute path, depending how it was entered."));
	}
  else
  {
    pTableProperties->setText(iRow, 0, i18n("Tablespace"));
    KPGLinkLabel *pLabel = new KPGLinkLabel( pTableProperties, "Tablespace" );
    pLabel->setValues(m_oidTablespace, m_strTablespace);
    pTableProperties->setCellWidget(iRow, 1, pLabel);
    pTableProperties->setText(iRow++, 2, i18n("Tablespace name"));
  }
  
	//--- Encoding
	pTableProperties->setText(iRow, 0, i18n("Encoding"));
  pTableProperties->setText(iRow, 1, m_strEncoding);
	pTableProperties->setText(iRow++, 2, i18n("Character encoding for this database"));
	
	//--- Can create
	pTableProperties->setText(iRow, 0, i18n("Can create"));
  pTableProperties->setText(iRow, 1, (m_bCanCreate) ? i18n("Yes") : i18n("No"));
	pTableProperties->setText(iRow++, 2, i18n("Does current user have access to database ?"));
	
	//--- Default variables
	pTableProperties->setText(iRow, 0, i18n("Default variables"));
  pTableProperties->setText(iRow, 1, m_strDefaultVariables);
	pTableProperties->setText(iRow++, 2, i18n("Session defaults for run-time configuration variables"));
	
	//--- Allow connection
	pTableProperties->setText(iRow, 0, i18n("Allow connection"));
  pTableProperties->setText(iRow, 1, (m_bAllowConnections) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("If false then no one can connect to this database. This is used to protect the template0 database from being altered."));
	
	//--- Is template
	pTableProperties->setText(iRow, 0, i18n("Is template"));
  pTableProperties->setText(iRow, 1, (m_bIsTemplate) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("If true then this database can be used in the TEMPLATE clause of CREATE DATABASE to create a new database as a clone of this one."));
	
	//--- Adjust DataTable columns size
  for(int nCol = 0; nCol < 3; nCol++)
  {
    pTableProperties->adjustColumn(nCol);
  }
	
	//---
	displayStatistics(widgetProperies);
	displayCasts(widgetProperies);
	displayLanguages(widgetProperies);
	displaySchemas(widgetProperies);
}

void KPGDatabase::displayStatistics(QWidget *widgetProperies)
{
	KPGDatabasePropPage *pPropertyPage = static_cast <KPGDatabasePropPage *> (widgetProperies);
	
	QTable *pTableStatistics = pPropertyPage->m_pTableStatistics;
	
	//--- Header
	// field names
  QHeader* hHeader = pTableStatistics->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	if(!m_pDbConnection)
		return;
	
	// obtain list of statistics	
	QString strQuery(QString("SELECT numbackends, xact_commit, xact_rollback, blks_read, blks_hit FROM pg_stat_database WHERE datid = '%1'").arg(m_oid));
				
	try
  {
    work pqxxXaction(*m_pDbConnection, "statistics");
		
		result pqxxResult = pqxxXaction.Exec(strQuery.latin1());

    if(pqxxResult.size() == 1)
    {
			int iRow = 0;
			
			pTableStatistics->setText(iRow, 0, i18n("Number of connections"));
  		pTableStatistics->setText(iRow, 1, pqxxResult[0]["numbackends"].c_str());
			pTableStatistics->setText(iRow++, 2, i18n("Number of active connections to the database"));
			
			pTableStatistics->setText(iRow, 0, i18n("Transactions committed"));
  		pTableStatistics->setText(iRow, 1, pqxxResult[0]["xact_commit"].c_str());
			pTableStatistics->setText(iRow++, 2, i18n("Total number of committed transactions"));
			
			pTableStatistics->setText(iRow, 0, i18n("Transactions rolled back"));
  		pTableStatistics->setText(iRow, 1, pqxxResult[0]["xact_rollback"].c_str());
			pTableStatistics->setText(iRow++, 2, i18n("Total number of rolled back transactions"));
			
			pTableStatistics->setText(iRow, 0, i18n("Readed blocks"));
  		pTableStatistics->setText(iRow, 1, pqxxResult[0]["blks_read"].c_str());
			pTableStatistics->setText(iRow++, 2, i18n("Total number of readed blocks"));
			
			pTableStatistics->setText(iRow, 0, i18n("Hitted blocks"));
  		pTableStatistics->setText(iRow, 1, pqxxResult[0]["blks_hit"].c_str());
			pTableStatistics->setText(iRow++, 2, i18n("Total number of hitted blocks"));
			
			//--- Adjust DataTable columns size
  		for(int nCol = 0; nCol < 3; nCol++)
  		{
    		pTableStatistics->adjustColumn(nCol);
  		}
    }
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return;
  } 
}

void KPGDatabase::displayCasts(QWidget *widgetProperies)
{
	KPGDatabasePropPage *pPropertyPage = static_cast <KPGDatabasePropPage *> (widgetProperies);
	
	//----------------------------------------------------------------------------
	//Display casts list
	QTable *pTableCasts = pPropertyPage->m_pTableCasts;
			
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pFolderCasts->resultCasts(), pTableCasts);

  // field names
  QHeader* hHeader = pTableCasts->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  hHeader->setLabel(nCol++, i18n("Context"));
  hHeader->setLabel(nCol++, i18n("Source type"));
	hHeader->setLabel(nCol++, i18n("Target type"));
  hHeader->setLabel(nCol++, i18n("Function"));
	hHeader->setLabel(nCol++, i18n("Namespace"));

  for(int nCol = 0; nCol < nTableCols; nCol++)
  {
    pTableCasts->adjustColumn(nCol);
  }
}

void KPGDatabase::displayLanguages(QWidget *widgetProperies)
{
	KPGDatabasePropPage *pPropertyPage = static_cast <KPGDatabasePropPage *> (widgetProperies);
	
	//----------------------------------------------------------------------------
	//Display languages list
	QTable *pTableLanguages = pPropertyPage->m_pTableLanguages;

	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pFolderLanguages->resultLanguages(), pTableLanguages);

  // field names
  QHeader* hHeader = pTableLanguages->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  hHeader->setLabel(nCol++, i18n("Is trusted"));
	hHeader->setLabel(nCol++, i18n("ACL"));
	hHeader->setLabel(nCol++, i18n("Language function"));
	hHeader->setLabel(nCol++, i18n("Language validator"));

	for(int nCol = 0; nCol < nTableCols; nCol++)
  {
    pTableLanguages->adjustColumn(nCol);
  }
}

void KPGDatabase::displaySchemas(QWidget *widgetProperies)
{
	// Get pointer to server for version info
	KPGServer *pServer = static_cast <KPGServer *> (parent());
  
	KPGDatabasePropPage *pPropertyPage = static_cast <KPGDatabasePropPage *> (widgetProperies);
	
	//----------------------------------------------------------------------------
	//Display schemas list
	QTable *pTableSchemas = pPropertyPage->m_pTableSchemas;
	
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pqxxResultSchemas, pTableSchemas);

  // field names
  QHeader* hHeader = pTableSchemas->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("ACL"));
	hHeader->setLabel(nCol++, i18n("Can Create"));
	hHeader->setLabel(nCol++, i18n("Owner"));
	if(pServer->versionMajor() >= 8) hHeader->setLabel(nCol++, i18n("Tablespace"));

  for(int nCol = 0; nCol < nTableCols; nCol++)
  {
    pTableSchemas->adjustColumn(nCol);
  }
}

// Return encoder name for QTextCodec regards to DB encoding
const QString KPGDatabase::encoderName() const
{
	return m_pDbConnection->encoderName(m_strEncoding);
}

void KPGDatabase::fillComboBoxWithChildSchemas(KComboBox *m_pComboBox, bool bOnlyUserSchemas)
{
  QListViewItem * pItem = firstChild();
  while(pItem)
  {
    KPGTreeItem *pTreeItem = static_cast <KPGTreeItem *> (pItem);
    
    if(pTreeItem->type() == KPGTreeItem::nodeSchema)
    {
      KPGSchema *pSchema = static_cast <KPGSchema *> (pTreeItem);
      
      if(!bOnlyUserSchemas || !pSchema->isSystemObject())
        m_pComboBox->insertItem(* pTreeItem->pixmap(0), pTreeItem->text(0));
    }
       
    pItem = pItem->nextSibling();
  }
}

