/***************************************************************************
                          kpgdatabase.h  -  description
                             -------------------
    begin                : � led 6 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPGDATABASE_H
#define KPGDATABASE_H

#include "kpgobject.h"
#include "kpgconnection.h"

class KPGServer;
class KPGCastsFolder;
class KPGLanguagesFolder;
class KComboBox;

/**Item in ListView, that represent database
  *@author Lumir Vanek
  */

class KPGDatabase : public KPGObject
{
public: 
	KPGDatabase(KPGServer *, const QString, pqxx::oid);
  KPGDatabase(KPGServer *, KPGDatabase *, const QString, pqxx::oid);
	~KPGDatabase();

  virtual eNodeType type() const { return nodeDatabase; }
  virtual bool isSystemObject() const { return m_bIsSystemObject; }
  bool isTemplate() const { return m_bIsTemplate; }
  bool connectToDB();
	void disconnectFromDB();
	bool isConnected() const { return (m_pDbConnection != 0); }
  pqxx::result runQuery(const QString, KPGConnection::eTransType = KPGConnection::eTransNormal);
	void displayProperties(QWidget *);
  
  void setProperties(const QString, const QString, const QString, const QString, bool, const QString, bool, bool);
  void setProperties(const QString, const QString, pqxx::oid, const QString, const QString, bool, const QString, bool, bool);
  
  void displayStatistics(QWidget *);
	KPGConnection * connection() { return m_pDbConnection; }
	bool refresh();
  bool refreshItem();
	const QString connectionString();
	
	// Return encoder name for QTextCodec regards to DB encoding
	const QString encoderName() const;
  
  const QString & owner() const { return m_strOwner; }
  const QString & tablespace() const { return m_strTablespace; }
  
  // Folders
  KPGCastsFolder * getCastsFolder() const { return m_pFolderCasts; }
  KPGLanguagesFolder * getLanguagesFolder() const { return m_pFolderLanguages; }
  
  void fillComboBoxWithChildSchemas(KComboBox *, bool = false);
   
  
protected:
  virtual void activate();
	
  void displayCasts(QWidget *);
	void displayLanguages(QWidget *);
	void displaySchemas(QWidget *);

protected:
  KPGConnection *m_pDbConnection;
  QString m_strUser;
  QString m_strPassword;
  QString m_strHost;
	
	// Database properties
	QString m_strOwner;
	QString m_strACL;
	QString m_strPath;           // For PostgreSQL 7.x or older
  pqxx::oid m_oidTablespace;   // For PostgreSQL 8 or newer 
  QString m_strTablespace;     // For PostgreSQL 8 or newer
	QString	m_strEncoding;
	bool m_bCanCreate;
	QString m_strDefaultVariables;
  bool m_bAllowConnections;
	bool m_bIsTemplate;
  
  bool m_bIsSystemObject;
	
	// Folders
	KPGCastsFolder *m_pFolderCasts;
	KPGLanguagesFolder *m_pFolderLanguages;
	
protected:
  // results that will be displaied in properties window
  pqxx::result m_pqxxResultSchemas; 
};

#endif
