//
// C++ Implementation: kpgdomain
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgdomain.h"

// include files for KDE
#include <kiconloader.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <qtable.h>

#include "../kpglinklabel.h"
#include "kpgconnection.h"
#include "kpgdomainsfolder.h"
#include "kpgdomainconstraintsfolder.h"
#include "../PropertyPages/kpgdomainproppage.h"


KPGDomain::KPGDomain(KPGDomainsFolder *parent, const QString name, pqxx::oid _oid)
	: KPGObject(parent, name, _oid)
{
  setPixmap(0, *m_pIconDomain);
  m_pFolderConstraints = new KPGDomainConstraintsFolder(this);
}

KPGDomain::KPGDomain(KPGDomainsFolder *parent, KPGDomain *after, const QString name, pqxx::oid _oid)
	: KPGObject(parent, after, name, _oid)
{
  setPixmap(0, *m_pIconDomain);
  m_pFolderConstraints = new KPGDomainConstraintsFolder(this);
}

KPGDomain::~KPGDomain()
{
}

bool KPGDomain::refresh()
{
  KPGConnection *pDbConnection = connection();
  pqxx::work pqxxXaction(*pDbConnection, "refresh");
  
  if(!m_pFolderConstraints->refresh(pqxxXaction, m_oid))
    return false;
   
  return true;
}

void KPGDomain::setProperties(
	const QString strDescription,
	pqxx::oid oidBaseType,
	const QString strBaseType,
	int iTypLen,
	bool bNotNull,
	const QString strDefault,
	int iTypNDims,
	const QString strTypDelim,
	const QString strOwner
	)
{
	m_strDescription = strDescription;
	m_oidBaseType = oidBaseType;
	m_strBaseType = strBaseType;
	m_iTypLen = iTypLen;
	m_bNotNull = bNotNull;
	m_strDefault = strDefault; 
	m_iTypNDims = iTypNDims;
	m_strTypDelim = strTypDelim;
	m_strOwner = strOwner;
}

bool KPGDomain::refreshConstraints(work &pqxxXaction)
{
  return m_pFolderConstraints->refresh(pqxxXaction, m_oid);
}

void KPGDomain::displayProperties(QWidget *widgetProperies)
{
	// Display domain info
	KPGDomainPropPage *pPropertyPage = static_cast <KPGDomainPropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
  QHeader* hHeader = pTableProperties->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
  
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
		
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Domain name"));
	
	//--- Description
	pTableProperties->setText(iRow, 0, i18n("Description"));
  pTableProperties->setText(iRow, 1, m_strDescription);
	pTableProperties->setText(iRow++, 2, i18n("Description of the domain"));
	
	//--- Base type
	pTableProperties->setText(iRow, 0, i18n("Base type"));
  KPGLinkLabel *pLabel = new KPGLinkLabel( pTableProperties, "Language function" );
	pLabel->setValues(m_oidBaseType, m_strBaseType);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("The type that this domain is based on"));
	
	//--- Type length
	pTableProperties->setText(iRow, 0, i18n("Type length"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_iTypLen));
	pTableProperties->setText(iRow++, 2, i18n("For a fixed-size type, typlen is the number of bytes in the internal representation of the type. But for a variable-length type, typlen is negative. -1 indicates a \"varlena\" type (one that has a length word), -2 indicates a null-terminated C string."));
	
	//--- Not null
	pTableProperties->setText(iRow, 0, i18n("Not null"));
  pTableProperties->setText(iRow, 1, (m_bNotNull) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("This represents a not-null constraint. It is possible to change this column to enable or disable the constraint."));
	
	//--- Default value
	pTableProperties->setText(iRow, 0, i18n("Default value"));
  pTableProperties->setText(iRow, 1, m_strDefault);
	pTableProperties->setText(iRow++, 2, i18n("A human-readable representation of the default value"));
	
	//--- Number of dimensions
	pTableProperties->setText(iRow, 0, i18n("Number of dimensions"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_iTypNDims));
	pTableProperties->setText(iRow++, 2, i18n("Number of dimensions, if the domain is an array type; otherwise 0. (Presently, the number of dimensions of an array is not enforced, so any nonzero value effectively means \"it's an array\".) "));
	
	//--- Array delimiter
	pTableProperties->setText(iRow, 0, i18n("Array delimiter"));
  pTableProperties->setText(iRow, 1, m_strTypDelim);
	pTableProperties->setText(iRow++, 2, i18n("Character that separates two values of this type when parsing array input. Note that the delimiter is associated with the array element data type, not the array data type."));
			
	//--- Owner
	pTableProperties->setText(iRow, 0, i18n("Owner"));
  pTableProperties->setText(iRow, 1, m_strOwner);
	pTableProperties->setText(iRow++, 2, i18n("Owner of the domain"));
	
	for(int nCol = 0; nCol < 3; nCol++)
  {
    pTableProperties->adjustColumn(nCol);
  }
}


