//
// C++ Implementation: kpgforeignkeyconstr
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgforeignkeyconstr.h"

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <qtable.h>

#include "kpgtableconstraintsfolder.h"
#include "../PropertyPages/kpgforeignkeyconstrproppage.h"
#include "../kpglinklabel.h"

KPGForeignKeyConstr::KPGForeignKeyConstr(KPGTableConstraintsFolder *parent, const QString name, pqxx::oid _oid)
  : KPGTableConstraint(parent, name, _oid)
{
  setPixmap(0, *m_pIconForeignKeyConstr);
}

KPGForeignKeyConstr::KPGForeignKeyConstr(KPGTableConstraintsFolder *parent, KPGTableConstraint *after, const QString name, pqxx::oid _oid)
  : KPGTableConstraint(parent, after, name, _oid)
{
  setPixmap(0, *m_pIconForeignKeyConstr);
}

KPGForeignKeyConstr::~KPGForeignKeyConstr()
{
}

void KPGForeignKeyConstr::displayProperties(QWidget *widgetProperies)
{
	// Display server version info
	KPGForeignKeyConstrPropPage *pPropertyPage = static_cast <KPGForeignKeyConstrPropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
  QHeader* hHeader = pTableProperties->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
  	
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
	
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Constraint name (not necessarily unique!)"));
	
	//--- Description
	pTableProperties->setText(iRow, 0, i18n("Description"));
  pTableProperties->setText(iRow, 1, m_strDescription);
	pTableProperties->setText(iRow++, 2, i18n("Description of the constraint"));
	
	//--- Is Deferrable
	pTableProperties->setText(iRow, 0, i18n("Is Deferrable"));
  pTableProperties->setText(iRow, 1, (m_bIsDeferrable) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("Is the constraint deferrable ?"));
	
	//--- Is Deferred
	pTableProperties->setText(iRow, 0, i18n("Is Deferred"));
  pTableProperties->setText(iRow, 1, (m_bIsDeferred) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("Is the constraint deferred by default ?"));
			
	//--- Update action
	pTableProperties->setText(iRow, 0, i18n("Update action"));
  pTableProperties->setText(iRow, 1, m_strUpdType);
	pTableProperties->setText(iRow++, 2, i18n("Foreign key update action code"));
	
	//--- Delete action
	pTableProperties->setText(iRow, 0, i18n("Delete action"));
  pTableProperties->setText(iRow, 1, m_strDelType);
	pTableProperties->setText(iRow++, 2, i18n("Foreign key deletion action code"));
		
	//--- Constraint key
	pTableProperties->setText(iRow, 0, i18n("Constraint key"));
  pTableProperties->setText(iRow, 1, m_strConstrKey);
	pTableProperties->setText(iRow++, 2, i18n("List of columns which the constraint constrains."));
	
	//--- Namespace
	pTableProperties->setText(iRow, 0, i18n("Namespace"));
  pTableProperties->setText(iRow, 1, m_strNspName);
	pTableProperties->setText(iRow++, 2, i18n("Name of the namespace"));
	
	//--- Referenced table
	pTableProperties->setText(iRow, 0, i18n("Referenced table"));
  KPGLinkLabel *pLabel = new KPGLinkLabel( pTableProperties, "Referenced table" );
	pLabel->setValues(m_oidReferencedTable, m_strReferencedTable);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Referenced table name"));
	
	//--- Foreign key
	pTableProperties->setText(iRow, 0, i18n("Foreign key"));
  pTableProperties->setText(iRow, 1, m_strForeignKey);
	pTableProperties->setText(iRow++, 2, i18n("This is an array of the referenced columns."));
			
	for(int nCol = 0; nCol < 3; nCol++)
  {
    pTableProperties->adjustColumn(nCol);
  }
}
