//
// C++ Implementation: kpgfunction
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgfunction.h"

#include <qlineedit.h>
#include <qtable.h>

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <ktextedit.h>

#include "../kpglinklabel.h"
#include "kpgfunctionsfolder.h"
#include "../PropertyPages/kpgfunctionproppage.h"

KPGFunction::KPGFunction(KPGFunctionsFolder *parent, const QString name, pqxx::oid _oid)
 : KPGObject(parent, name, _oid)
{
	setPixmap(0, *m_pIconFunction);
}

KPGFunction::KPGFunction(KPGFunctionsFolder *parent, KPGFunction *after, const QString name, pqxx::oid _oid)
 : KPGObject(parent, after, name, _oid)
{
	setPixmap(0, *m_pIconFunction);
}


KPGFunction::~KPGFunction()
{
}

void KPGFunction::setProperties(
		const QString strDescription,
		bool bIsAgg,
		bool bIsSecdef,
		bool bIsStrict,
		bool bRetSet,
		const QString strVolatile,
		int iNArgs,
		const QString strBin,
		const QString strAcl,
		const QString strReturnTypName,
		pqxx::oid oidReturnType,
		const QString strLanName,
		const QString strFuncOwner
	)
{
	m_strDescription = strDescription;
	m_bIsAgg = bIsAgg;
	m_bIsSecdef = bIsSecdef;
	m_bIsStrict = bIsStrict;
	m_bRetSet = bRetSet;
	m_strVolatile = strVolatile;
	m_iNArgs = iNArgs;
	m_strBin = strBin;
	m_strAcl = strAcl;
	m_strReturnTypName = strReturnTypName;
	m_oidReturnType = oidReturnType;
	m_strLanName = strLanName;
	m_strFuncOwner = strFuncOwner;
	
	//--- Set icon for function type
	if(strLanName == "c")
		setPixmap(0, *m_pIconFunctionC);
	else
		if(strLanName == "sql")
			setPixmap(0, *m_pIconFunctionSql);
		else
			if(strLanName == "plpgsql")
				setPixmap(0, *m_pIconFunctionPl);
}

void KPGFunction::displayProperties(QWidget *widgetProperies)
{
	// Display server version info
	KPGFunctionPropPage *pPropertyPage = static_cast <KPGFunctionPropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
  QHeader* hHeader = pTableProperties->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
  	
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
	
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Function name"));
	
	//--- Description
	pTableProperties->setText(iRow, 0, i18n("Description"));
  pTableProperties->setText(iRow, 1, m_strDescription);
	pTableProperties->setText(iRow++, 2, i18n("Description of the function"));
		
	//--- Is Aggregate
	pTableProperties->setText(iRow, 0, i18n("Is Aggregate"));
  pTableProperties->setText(iRow, 1, (m_bIsAgg) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("True if function is an aggregate function)"));
	
	//--- Is Security definer
	pTableProperties->setText(iRow, 0, i18n("Is Security definer"));
  pTableProperties->setText(iRow, 1, (m_bIsSecdef) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("Function is a security definer (i.e., a \"setuid\" function"));
			
	//--- Is Strict
	pTableProperties->setText(iRow, 0, i18n("Is Strict"));
  pTableProperties->setText(iRow, 1, (m_bIsStrict) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("Function returns null if any call argument is null. In that case the function won't actually be called at all. Functions that are not \"strict\" must be prepared to handle null inputs."));
	
	//--- Return Set
	pTableProperties->setText(iRow, 0, i18n("Return Set"));
  pTableProperties->setText(iRow, 1, (m_bRetSet) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("Function returns a set (i.e., multiple values of the specified data type)"));
	
	//--- Volatile
	pTableProperties->setText(iRow, 0, i18n("Volatile"));
  pTableProperties->setText(iRow, 1, m_strVolatile);
	pTableProperties->setText(iRow++, 2, i18n("Tells whether the function's result depends only on its input arguments, or is affected by outside factors. It is i for \"immutable\" functions, which always deliver the same result for the same inputs. It is s for \"stable\" functions, whose results (for fixed inputs) do not change within a scan. It is v for \"volatile\" functions, whose results may change at any time. (Use v also for functions with side-effects, so that calls to them cannot get optimized away.) "));
	
	//--- Number of argument
	pTableProperties->setText(iRow, 0, i18n("Number of arguments"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_iNArgs));
	pTableProperties->setText(iRow++, 2, i18n("Number of arguments"));
	
	//--- Bin
	pTableProperties->setText(iRow, 0, i18n("Bin"));
  pTableProperties->setText(iRow, 1, m_strBin);
	pTableProperties->setText(iRow++, 2, i18n("Additional information about how to invoke the function. Again, the interpretation is language-specific."));
	
	//--- Access privileges
	pTableProperties->setText(iRow, 0, i18n("ACL"));
  pTableProperties->setText(iRow, 1, m_strAcl);
	pTableProperties->setText(iRow++, 2, i18n("Function access privileges"));
	
	//--- Type name
	pTableProperties->setText(iRow, 0, i18n("Type name"));
  KPGLinkLabel *pLabel = new KPGLinkLabel( pTableProperties, "Type name" );
	pLabel->setValues(m_oidReturnType, m_strReturnTypName);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Return data type name"));
	
	//--- Language name
	pTableProperties->setText(iRow, 0, i18n("Language name"));
  pTableProperties->setText(iRow, 1, m_strLanName);
	pTableProperties->setText(iRow++, 2, i18n("Name of the language (to be specified when creating a function)"));
	
	//--- Owner
	pTableProperties->setText(iRow, 0, i18n("Owner"));
  pTableProperties->setText(iRow, 1, m_strFuncOwner);
	pTableProperties->setText(iRow++, 2, i18n("Function owner"));
	
	//--- Namespace
	/*pTableProperties->setText(iRow, 0, i18n("Namespace"));
  pTableProperties->setText(iRow, 1, m_strNameSpace);
	pTableProperties->setText(iRow++, 2, i18n("Name of the namespace"));
	*/
	
	for(int nCol = 0; nCol < 3; nCol++)
  {
    pTableProperties->adjustColumn(nCol);
  }
	
	pPropertyPage->m_pLineEditArgTypes->setText(m_strArgTypes);
	pPropertyPage->m_pTextEditSource->setText(m_strSrc);
}


