//
// C++ Implementation: kpgindex
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgindex.h"

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <ktextedit.h>

#include <qtable.h>

#include "../kpglinklabel.h"
#include "kpgtableindexesfolder.h"
#include "../PropertyPages/kpgindexproppage.h"

KPGIndex::KPGIndex(KPGTableIndexesFolder *parent, const QString name, pqxx::oid _oid)
  : KPGTableIndex(parent, name, _oid)
{
  setPixmap(0, *m_pIconIndex);
}

KPGIndex::KPGIndex(KPGTableIndexesFolder *parent, KPGTableIndex *after, const QString name, pqxx::oid _oid)
  : KPGTableIndex(parent, after, name, _oid)
{
  setPixmap(0, *m_pIconIndex);
}

KPGIndex::~KPGIndex()
{
}

void KPGIndex::displayProperties(QWidget *widgetProperies)
{
	// Display server version info
	KPGIndexPropPage *pPropertyPage = static_cast <KPGIndexPropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
  QHeader* hHeader = pTableProperties->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
  	
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
	
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Name of the index"));
	
	//--- Description
	pTableProperties->setText(iRow, 0, i18n("Description"));
  pTableProperties->setText(iRow, 1, m_strDescription);
	pTableProperties->setText(iRow++, 2, i18n("Description of the conindex"));
	
	//--- Index key
	pTableProperties->setText(iRow, 0, i18n("Index key"));
  pTableProperties->setText(iRow, 1, m_strIndKey);
	pTableProperties->setText(iRow++, 2, i18n("This is an array of indnatts (up to INDEX_MAX_KEYS) values that indicate which table columns this index indexes. For example a value of 1 3 would mean that the first and the third table columns make up the index key. A zero in this array indicates that the corresponding index attribute is an expression over the table columns, rather than a simple column reference. "));
	
	//--- Is Unique
	pTableProperties->setText(iRow, 0, i18n("Is Unique"));
  pTableProperties->setText(iRow, 1, (m_bIsUnique) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("If true, this is a unique index."));
	
	//--- Is Primary
	pTableProperties->setText(iRow, 0, i18n("Is Primary"));
  pTableProperties->setText(iRow, 1, (m_bIsPrimary) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("If true, this index represents the primary key of the table. (indisunique should always be true when this is true.)"));
	
	//--- Is Clustered
	pTableProperties->setText(iRow, 0, i18n("Is Clustered"));
  pTableProperties->setText(iRow, 1, (m_bIsClustered) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("If true, the table was last clustered on this index."));
	
	//--- Tablespace
  pTableProperties->setText(iRow, 0, i18n("Tablespace"));
  KPGLinkLabel *pLabel = new KPGLinkLabel( pTableProperties, "Tablespace" );
  pLabel->setValues(m_oidTablespace, m_strTablespace);
  pTableProperties->setCellWidget(iRow, 1, pLabel);
  pTableProperties->setText(iRow++, 2, i18n("Tablespace name"));
  			
	for(int nCol = 0; nCol < 3; nCol++)
  {
    pTableProperties->adjustColumn(nCol);
  }
  
  pPropertyPage->m_pTextEditDefinition->setText(m_strIndexdef);
}

