//
// C++ Implementation: kpglanguage
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpglanguage.h"

// include files for KDE
#include <kiconloader.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <qtable.h>

#include "../kpglinklabel.h"
#include "kpglanguagesfolder.h"
#include "../PropertyPages/kpglanguageproppage.h"


KPGLanguage::KPGLanguage(KPGLanguagesFolder *parent, const QString name, pqxx::oid _oid)
  : KPGObject(parent, name, _oid)
{
  setPixmap(0, *m_pIconLanguage);
}

KPGLanguage::KPGLanguage(KPGLanguagesFolder *parent, KPGLanguage *after, const QString name, pqxx::oid _oid)
  : KPGObject(parent, after, name, _oid)
{
  setPixmap(0, *m_pIconLanguage);
}

KPGLanguage::~KPGLanguage()
{
}

void KPGLanguage::setProperties(
		bool bIsTrusted,	
		const QString strACL,
		pqxx::oid oidLanguageProc,
		const QString strLanguageProcName,
		pqxx::oid oidValidatorProc,
		const QString strValidatorProcName
	)
{
		m_bIsTrusted = bIsTrusted;	
		m_strACL = strACL;
		m_oidLanguageProc = oidLanguageProc;
		m_strLanguageProcName = strLanguageProcName;
		m_oidValidatorProc = oidValidatorProc;
		m_strValidatorProcName = strValidatorProcName;
}

void KPGLanguage::displayProperties(QWidget *widgetProperies)
{
	// Display language info
	KPGLanguagePropPage *pPropertyPage = static_cast <KPGLanguagePropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
  QHeader* hHeader = pTableProperties->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
	  	
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
		
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Language name"));
	
	//--- Is trusted
	pTableProperties->setText(iRow, 0, i18n("Is Trusted"));
  pTableProperties->setText(iRow, 1, (m_bIsTrusted) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n(" This is a trusted language. See under CREATE LANGUAGE what this means."));
	
	//--- ACL
	pTableProperties->setText(iRow, 0, i18n("ACL"));
  pTableProperties->setText(iRow, 1, m_strACL);
	pTableProperties->setText(iRow++, 2, i18n("Access privileges; see the descriptions of GRANT and REVOKE for details."));
	
	//--- Language Function
	pTableProperties->setText(iRow, 0, i18n("Language function"));
  KPGLinkLabel *pLabel = new KPGLinkLabel( pTableProperties, "Language function" );
	pLabel->setValues(m_oidLanguageProc, m_strLanguageProcName);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Language handler, which is a special function that is responsible for executing all functions that are written in the particular language.)"));
	
	//--- Language Validator
	pTableProperties->setText(iRow, 0, i18n("Language validator"));
  pLabel = new KPGLinkLabel( pTableProperties, "Language validator" );
	pLabel->setValues(m_oidValidatorProc, m_strValidatorProcName);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Language validator function that is responsible for checking the syntax and validity of new functions when they are created."));
		
	for(int nCol = 0; nCol < 3; nCol++)
  {
    pTableProperties->adjustColumn(nCol);
  }
}


