//
// C++ Implementation: kpglanguagesfolder
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpglanguagesfolder.h"

#include <qtable.h>

#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include "kpgconnection.h"
#include "kpglanguage.h"
#include "kpgdatabase.h"
#include "../PropertyPages/kpgpropertypage.h"
#include "../kpgsqldialog.h"
#include "../kpgsqldialog.h"

KPGLanguagesFolder::KPGLanguagesFolder(KPGDatabase *parent)
 : KPGItemsFolder(parent, i18n("Languages"))
{
	setPixmap(0, *m_pIconFolderYellow);
}


KPGLanguagesFolder::~KPGLanguagesFolder()
{
}

bool KPGLanguagesFolder::refresh(pqxx::work &pqxxXaction)
{
  // delete all child items 
  while (QListViewItem * pItem = firstChild())
    delete pItem;

	// obtain list of languages
  QString strQuery("SELECT lan.oid, lan.lanname, lanpltrusted, lanacl, hp.oid AS procoid, hp.proname as langproc, vp.oid AS validoid, vp.proname as langvalidator ");
	strQuery.append("FROM pg_language lan ");
	strQuery.append("JOIN pg_proc hp on hp.oid=lanplcallfoid ");
	strQuery.append("LEFT OUTER JOIN pg_proc vp on vp.oid=lanvalidator ");
	strQuery.append("WHERE lanispl IS TRUE ");
	strQuery.append("ORDER BY lanname");
			
  try
  {
    m_pqxxResultLanguages = pqxxXaction.Exec(strQuery.latin1());

		KPGLanguage *pgLanguage = 0;
		
    for (result::size_type i = 0; i != m_pqxxResultLanguages.size(); ++i)
    {
			pqxx::oid oid;
			m_pqxxResultLanguages[i][0].to(oid);
		
			if(pgLanguage == 0)
      	pgLanguage = new KPGLanguage(this, m_pqxxResultLanguages[i]["lanname"].c_str(), oid);
			else
				pgLanguage = new KPGLanguage(this, pgLanguage, m_pqxxResultLanguages[i]["lanname"].c_str(), oid);
				
			bool m_bIsTrusted;
			pqxx::oid oidLanguageProc = 0;
			pqxx::oid oidValidatorProc = 0;
			
			m_pqxxResultLanguages[i]["lanpltrusted"].to(m_bIsTrusted);
			m_pqxxResultLanguages[i]["procoid"].to(oidLanguageProc);
			m_pqxxResultLanguages[i]["validoid"].to(oidValidatorProc);
						
			pgLanguage->setProperties(
				m_bIsTrusted,
				m_pqxxResultLanguages[i]["lanacl"].c_str(),
				oidLanguageProc, 
				m_pqxxResultLanguages[i]["langproc"].c_str(), 
				oidValidatorProc,
				m_pqxxResultLanguages[i]["langvalidator"].c_str()
				); 
    }
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return false;
  } 
  
  return true;
}

void KPGLanguagesFolder::displayProperties(QWidget *widgetProperies)
{
	KPGPropertyPage *pPropertyPage = static_cast <KPGPropertyPage *> (widgetProperies);
  pPropertyPage->setLabelText(i18n("Languages &list:"));
	
	//----------------------------------------------------------------------------
	//Display languages list
	QTable *pTableLanguages = pPropertyPage->m_pTableItems;
  
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pqxxResultLanguages, pTableLanguages);

  // field names
  QHeader* hHeader = pTableLanguages->horizontalHeader();

  int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  hHeader->setLabel(nCol++, i18n("Is Trusted"));
	hHeader->setLabel(nCol++, i18n("ACL"));
	hHeader->setLabel(nCol++, i18n("Language Function"));
	hHeader->setLabel(nCol++, i18n("Language Validator"));

  for(int nCol = 0; nCol < nTableCols; nCol++)
  {
    pTableLanguages->adjustColumn(nCol);
  }
}


