//
// C++ Implementation: kpgoperatorclass
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgoperatorclass.h"

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <ktextedit.h>

#include <qtable.h>

#include "../kpglinklabel.h"
#include "kpgoperatorclassesfolder.h"
#include "../PropertyPages/kpgoperatorclassproppage.h"

KPGOperatorClass::KPGOperatorClass(KPGOperatorClassesFolder *parent, const QString name, pqxx::oid _oid)
  : KPGObject(parent, name, _oid)
{
  setPixmap(0, *m_pIconOperatorClass);
}

KPGOperatorClass::KPGOperatorClass(KPGOperatorClassesFolder *parent, KPGOperatorClass *after, const QString name, pqxx::oid _oid)
  : KPGObject(parent, after, name, _oid)
{
  setPixmap(0, *m_pIconOperatorClass);
}


KPGOperatorClass::~KPGOperatorClass()
{
}


void KPGOperatorClass::setProperties(
	bool bIsDefault,
	pqxx::oid oidDataType,
	const QString strDataType,
	pqxx::oid oidKeyType,
	const QString strKeyType,
	const QString strAccessMethodName,
	const QString strOwner
	)
{
	m_bIsDefault = bIsDefault;
	m_oidDataType = oidDataType;
	m_strDataType = strDataType;
	m_oidKeyType = oidKeyType;
	m_strKeyType = strKeyType;
	m_strAccessMethodName = strAccessMethodName;
	m_strOwner = strOwner;
}

void KPGOperatorClass::displayProperties(QWidget *widgetProperies)
{
	// Display server version info
	KPGOperatorClassPropPage *pPropertyPage = static_cast <KPGOperatorClassPropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
  QHeader* hHeader = pTableProperties->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
  	
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
	
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Operator Class name"));
	
	//--- Is Default
	pTableProperties->setText(iRow, 0, i18n("Is default"));
  pTableProperties->setText(iRow, 1, (m_bIsDefault) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("True if this operator class is the default for data type"));
	
	//--- Data type
	pTableProperties->setText(iRow, 0, i18n("Data type"));
  KPGLinkLabel *pLabel = new KPGLinkLabel( pTableProperties, "Data type" );
	pLabel->setValues(m_oidDataType, m_strDataType);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Data type that the operator class indexes"));
	
	//--- Key Type
	pTableProperties->setText(iRow, 0, i18n("Key type"));
  pLabel = new KPGLinkLabel( pTableProperties, "Key type" );
	pLabel->setValues(m_oidKeyType, m_strKeyType);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Type of data stored in index, or noyhing if same as Data Type"));
	
	//--- Access method
	pTableProperties->setText(iRow, 0, i18n("Access method"));
  pTableProperties->setText(iRow, 1, m_strAccessMethodName);
	pTableProperties->setText(iRow++, 2, i18n("Index access method operator class is for"));
	
	//--- Owner
	pTableProperties->setText(iRow, 0, i18n("Owner"));
  pTableProperties->setText(iRow, 1, m_strOwner);
	pTableProperties->setText(iRow++, 2, i18n("Operator owner"));
	
	for(int nCol = 0; nCol < 3; nCol++)
  {
    pTableProperties->adjustColumn(nCol);
  }
}
