//
// C++ Implementation: kpgoperatorclasesfolder
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgoperatorclassesfolder.h"

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <qtable.h>

#include "kpgconnection.h"
#include "kpgschema.h"
#include "../PropertyPages/kpgpropertypage.h"
#include "kpgoperatorclass.h"
#include "../kpgsqldialog.h"

KPGOperatorClassesFolder::KPGOperatorClassesFolder(KPGSchema *parent, pqxx::oid oidNameSpace)
 : KPGItemsFolder(parent, i18n("Operator Classes"))
{
	setPixmap(0, *m_pIconFolderViolet);
	m_oidNameSpace = oidNameSpace;
}

KPGOperatorClassesFolder::~KPGOperatorClassesFolder()
{
}

bool KPGOperatorClassesFolder::refresh(pqxx::work &pqxxXaction)
{
  // delete all child items 
  while (QListViewItem * pItem = firstChild())
    delete pItem;

	// obtain list of operator classes
  QString strQuery("SELECT op.oid, op.opcname, op.opcdefault, opcintype, it.typname as intypename, opckeytype, dt.typname as keytypename, amname, pg_get_userbyid(op.opcowner) as opowner ");
		
	strQuery.append("FROM pg_opclass op ");
	strQuery.append("JOIN pg_am am ON am.oid=opcamid ");
	strQuery.append("JOIN pg_type it ON it.oid=opcintype ");
	strQuery.append("LEFT OUTER JOIN pg_type dt ON dt.oid=opckeytype ");
	strQuery.append("WHERE opcnamespace = " + QString("%1").arg(m_oidNameSpace) + " ");
	strQuery.append("ORDER BY op.opcname");
			
  try
  {
    m_pqxxResultOperatorClasses = pqxxXaction.Exec(strQuery.latin1());

		KPGOperatorClass *pgOperatorClass = 0;
		
    for (result::size_type i = 0; i != m_pqxxResultOperatorClasses.size(); ++i)
    {
			pqxx::oid oid;
			m_pqxxResultOperatorClasses[i][0].to(oid);
		
			if(pgOperatorClass == 0)
      	pgOperatorClass = new KPGOperatorClass(this, m_pqxxResultOperatorClasses[i]["opcname"].c_str(), oid);
			else
				pgOperatorClass = new KPGOperatorClass(this, pgOperatorClass, m_pqxxResultOperatorClasses[i]["opcname"].c_str(), oid);
			
			bool m_bIsDefault;
			pqxx::oid oidDataType = 0;
			pqxx::oid m_oidKeyType = 0;
						
			m_pqxxResultOperatorClasses[i]["opcdefault"].to(m_bIsDefault);
			m_pqxxResultOperatorClasses[i]["opcintype"].to(oidDataType);
			m_pqxxResultOperatorClasses[i]["opckeytype"].to(m_oidKeyType);
			
					
			pgOperatorClass->setProperties(
				m_bIsDefault,
				oidDataType,
				m_pqxxResultOperatorClasses[i]["intypename"].c_str(),
				m_oidKeyType,
				m_pqxxResultOperatorClasses[i]["keytypename"].c_str(),
				m_pqxxResultOperatorClasses[i]["amname"].c_str(),
				m_pqxxResultOperatorClasses[i]["opowner"].c_str()
				);
    }
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return false;
  } 
  
  return true;
}
void KPGOperatorClassesFolder::displayProperties(QWidget *widgetProperies)
{
	KPGPropertyPage *pPropertyPage = static_cast <KPGPropertyPage *> (widgetProperies);
  pPropertyPage->setLabelText(i18n("Operator classes &list:"));
	
	//----------------------------------------------------------------------------
	//Display operator classes list
		
  QTable *pTableOperatorClasses = pPropertyPage->m_pTableItems;
  
  // fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pqxxResultOperatorClasses, pTableOperatorClasses);

  // field names
  QHeader* hHeader = pTableOperatorClasses->horizontalHeader();

  int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  hHeader->setLabel(nCol++, i18n("Is Defalt"));
	hHeader->setLabel(nCol++, i18n("Data type"));
  hHeader->setLabel(nCol++, i18n("Key type"));
	hHeader->setLabel(nCol++, i18n("Access method"));
	hHeader->setLabel(nCol++, i18n("Owner"));

  for(int nCol = 0; nCol < nTableCols; nCol++)
  {
    pTableOperatorClasses->adjustColumn(nCol);
  }
}



