//
// C++ Implementation: kpgrule
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgrule.h"

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <ktextedit.h>

#include <qtable.h>

#include "kpgrulesfolder.h"
#include "../PropertyPages/kpgruleproppage.h"

KPGRule::KPGRule(KPGRulesFolder *parent, const QString name, pqxx::oid _oid)
 : KPGObject(parent, name, _oid)
{
	setPixmap(0, *m_pIconRule);
}

KPGRule::KPGRule(KPGRulesFolder *parent, KPGRule *after, const QString name, pqxx::oid _oid)
 : KPGObject(parent, after, name, _oid)
{
	setPixmap(0, *m_pIconRule);
}

KPGRule::~KPGRule()
{
}

void KPGRule::setProperties(
		const QString strDescription,
		bool bIsInstead,
		const QString strEvType,
		const QString strEvQual,
		const QString strNameSpace
	)
{
	m_strDescription = strDescription;
	m_bIsInstead = bIsInstead;
	m_strEvType = strEvType;
	m_strEvQual = strEvQual;
	m_strNameSpace = strNameSpace;
}
void KPGRule::displayProperties(QWidget *widgetProperies)
{
	// Display server version info
	KPGRulePropPage *pPropertyPage = static_cast <KPGRulePropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
  QHeader* hHeader = pTableProperties->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
  	
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
	
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Name of the rule"));
	
	//--- Description
	pTableProperties->setText(iRow, 0, i18n("Description"));
  pTableProperties->setText(iRow, 1, m_strDescription);
	pTableProperties->setText(iRow++, 2, i18n("Description of the rule"));
	
	//--- Is Instead
	pTableProperties->setText(iRow, 0, i18n("Is Instead"));
  pTableProperties->setText(iRow, 1, (m_bIsInstead) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("True if the rule is an INSTEAD rule"));
	
	//--- Event type
	pTableProperties->setText(iRow, 0, i18n("Event type"));
  pTableProperties->setText(iRow, 1, m_strEvType);
	pTableProperties->setText(iRow++, 2, i18n("Event type that the rule is for: 1 = SELECT, 2 = UPDATE, 3 = INSERT, 4 = DELETE "));
	
	//--- Qualifying Condition
	pTableProperties->setText(iRow, 0, i18n("Qualifying Condition"));
  pTableProperties->setText(iRow, 1, m_strEvQual);
	pTableProperties->setText(iRow++, 2, i18n("Expression tree (in the form of a nodeToString() representation) for the rule's qualifying condition"));
	
	//--- Namespace
	pTableProperties->setText(iRow, 0, i18n("Namespace"));
  pTableProperties->setText(iRow, 1, m_strNameSpace);
	pTableProperties->setText(iRow++, 2, i18n("Name of the namespace"));
	
	for(int nCol = 0; nCol < 3; nCol++)
  {
    pTableProperties->adjustColumn(nCol);
  }
	
	pPropertyPage->m_pTextEditDefinition->setText(m_strDefinition);
}


