//
// C++ Implementation: kpgsequence
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgsequence.h"

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <qtable.h>

#include "kpgsequencesfolder.h"
#include "../PropertyPages/kpgsequenceproppage.h"

KPGSequence::KPGSequence(KPGSequencesFolder *parent, const QString name, pqxx::oid _oid)
 : KPGObject(parent, name, _oid)
{
	setPixmap(0, *m_pIconSequence);
	
	m_llLastValue = 0;
	m_llMinValue = 0; 
	m_llMaxValue = 0; 
	m_llCacheValue = 0; 
	m_bIsCycled = false; 
	m_iIncrementBy = 1;
}

KPGSequence::KPGSequence(KPGSequencesFolder *parent, KPGSequence *after, const QString name, pqxx::oid _oid)
 : KPGObject(parent, after, name, _oid)
{
	setPixmap(0, *m_pIconSequence);
	
	m_llLastValue = 0;
	m_llMinValue = 0; 
	m_llMaxValue = 0; 
	m_llCacheValue = 0; 
	m_bIsCycled = false; 
	m_iIncrementBy = 1;
}


KPGSequence::~KPGSequence()
{
}

void KPGSequence::setProperties(
	const QString strOwner, 
	const QString strACL,
	const QString strDescription
	)
{
	m_strOwner = strOwner;
	m_strACL = strACL;
	m_strDescription = strDescription;
}

void KPGSequence::setSeqValues(long long llLastValue, long long llMinValue, long long llMaxValue, long long llCacheValue, bool bIsCycled, int iIncrementBy)
{

	m_llLastValue = llLastValue;
	m_llMinValue = llMinValue; 
	m_llMaxValue = llMaxValue; 
	m_llCacheValue = llCacheValue; 
	m_bIsCycled = bIsCycled; 
	m_iIncrementBy = iIncrementBy;
}


void KPGSequence::displayProperties(QWidget *widgetProperies)
{
	KPGSequencePropPage *pPropertyPage = static_cast <KPGSequencePropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
  QHeader* hHeader = pTableProperties->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
  	
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
	
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Name of the sequence"));
	
	//--- Description
	pTableProperties->setText(iRow, 0, i18n("Description"));
  pTableProperties->setText(iRow, 1, m_strDescription);
	pTableProperties->setText(iRow++, 2, i18n("Description of the sequence"));
	
	//--- Owner
	pTableProperties->setText(iRow, 0, i18n("Owner"));
  pTableProperties->setText(iRow, 1, m_strOwner);
	pTableProperties->setText(iRow++, 2, i18n("Owner of the sequence"));
	
	//--- ACL
	pTableProperties->setText(iRow, 0, i18n("ACL"));
  pTableProperties->setText(iRow, 1, m_strACL);
	pTableProperties->setText(iRow++, 2, i18n("Access privileges; see the descriptions of GRANT and REVOKE for details."));
	
	//--- Last value
	pTableProperties->setText(iRow, 0, i18n("Last value"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_llLastValue));
	pTableProperties->setText(iRow++, 2, i18n("Last counter value"));
	
	//--- Minimum value
	pTableProperties->setText(iRow, 0, i18n("Minimum value"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_llMinValue));
	pTableProperties->setText(iRow++, 2, i18n("Minimum counter value"));
	
	//--- Maximum value
	pTableProperties->setText(iRow, 0, i18n("Maximum value"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_llMaxValue));
	pTableProperties->setText(iRow++, 2, i18n("Maximum counter value"));
	
	//--- Cache value
	pTableProperties->setText(iRow, 0, i18n("Cache value"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_llCacheValue));
	pTableProperties->setText(iRow++, 2, i18n("Cache counter value"));
	
	//--- Is Cycled
	pTableProperties->setText(iRow, 0, i18n("Is Cycled"));
  pTableProperties->setText(iRow, 1, (m_bIsCycled) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("True, if counter value is cycled, when reaching max. value"));
	
	//--- Increment By
	pTableProperties->setText(iRow, 0, i18n("Increment By"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_iIncrementBy));
	pTableProperties->setText(iRow++, 2, i18n("Increment value for sequence"));
		
				
	for(int nCol = 0; nCol < 3; nCol++)
  {
    pTableProperties->adjustColumn(nCol);
  }
}

