/***************************************************************************
                          kpgsequencesfolder  -  description
                             -------------------
    begin                : Ne no 1 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "kpgsequencesfolder.h"

// include files for Qt
#include <qtable.h>

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include "kpgconnection.h"
#include "kpgserver.h"
#include "kpgschema.h"
#include "kpgsequence.h"
#include "../PropertyPages/kpgpropertypage.h"
#include "../kpgsqldialog.h"

KPGSequencesFolder::KPGSequencesFolder(KPGSchema *parent, pqxx::oid oidNameSpace)
 : KPGItemsFolder(parent, i18n("Sequences"))
{
	setPixmap(0, *m_pIconFolderViolet);
	m_oidNameSpace = oidNameSpace;
}


KPGSequencesFolder::~KPGSequencesFolder()
{
}

bool KPGSequencesFolder::refresh(pqxx::work &pqxxXaction)
{
  // delete all child items (databases)
  while (QListViewItem * pItem = firstChild())
    delete pItem;
  
	KPGSchema *pSchema = static_cast <KPGSchema *> (parent());	
  //                                              schema -> database -> server    
  KPGServer *pServer = static_cast <KPGServer *> (pSchema->parent()->parent());
  if(pServer->type() != nodeServer)
  {
    kdError() << "KPGSequencesFolder::refresh cannot find parent server." << endl;
    return "";
  }

	// obtain list of sequences
  QString strQuery("SELECT cl.oid, relname, description, pg_get_userbyid(relowner) AS seqowner, relacl ");
	strQuery.append("FROM pg_class cl ");
	strQuery.append("LEFT OUTER JOIN pg_description des ON des.objoid=cl.oid ");
	strQuery.append("WHERE relkind = 'S' AND relnamespace = " + QString("%1").arg(m_oidNameSpace) + " ");
	strQuery.append("ORDER BY relname");
			
  try
  {
    m_pqxxResultSequences = pqxxXaction.Exec(strQuery.latin1());

		KPGSequence *pgSequence = 0;
		
    for (result::size_type i = 0; i != m_pqxxResultSequences.size(); ++i)
    {
			pqxx::oid oid;
			m_pqxxResultSequences[i][0].to(oid);
		
      if(pgSequence == 0)
				pgSequence = new KPGSequence(this, m_pqxxResultSequences[i]["relname"].c_str(), oid);
			else
				pgSequence = new KPGSequence(this, pgSequence, m_pqxxResultSequences[i]["relname"].c_str(), oid);
						
			pgSequence->setProperties(
				m_pqxxResultSequences[i]["seqowner"].c_str(), 
				m_pqxxResultSequences[i]["relacl"].c_str(),
				m_pqxxResultSequences[i]["description"].c_str()
				);
			
      
      if((pServer->versionMajor() >= 8) || ((pServer->versionMajor() == 7) && (pServer->versionMiddle() >= 4)))
      {          	
			 //-- Get seq. values
			 strQuery = QString("SELECT last_value, min_value, max_value, cache_value, is_cycled, increment_by FROM  %1.%2;").arg(pSchema->text(0)).arg(m_pqxxResultSequences[i]["relname"].c_str());
			 pqxx::result pqxxResultSeqValues = pqxxXaction.exec(strQuery.latin1());
		    
			 if (pqxxResultSeqValues.size() == 1)
          { 
				    QString strLastValue(pqxxResultSeqValues[0]["last_value"].c_str());
				    QString strMinValue(pqxxResultSeqValues[0]["min_value"].c_str()); 
				    QString strMaxValue(pqxxResultSeqValues[0]["max_value"].c_str()); 
				    QString strCacheValue(pqxxResultSeqValues[0]["cache_value"].c_str()); 
							 
			   long long llLastValue = strLastValue.toLongLong();
				    long long llMinValue = strMinValue.toLongLong(); 
				    long long llMaxValue = strMaxValue.toLongLong(); 
				    long long llCacheValue = strCacheValue.toLongLong(); 
				    bool bIsCycled; 
				    int iIncrementBy;
    
				    /*pqxxResultSeqValues[0]["last_value"].to(iLastValue);
				    pqxxResultSeqValues[0]["min_value"].to(iMinValue);
				    pqxxResultSeqValues[0]["max_value"].to(iMaxValue);
				    pqxxResultSeqValues[0]["cache_value"].to(iCacheValue);*/
				    pqxxResultSeqValues[0]["is_cycled"].to(bIsCycled);
				    pqxxResultSeqValues[0]["increment_by"].to(iIncrementBy);
	 
				    pgSequence->setSeqValues(llLastValue, llMinValue, llMaxValue, llCacheValue, bIsCycled, iIncrementBy);
	   		 }	
      }	
    }
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return false;
  } 
  
  return true;
}
void KPGSequencesFolder::displayProperties(QWidget *widgetProperies)
{
	KPGPropertyPage *pPropertyPage = static_cast <KPGPropertyPage *> (widgetProperies);
  pPropertyPage->setLabelText(i18n("Sequences &list:"));
	
	//----------------------------------------------------------------------------
	//Display sequences list
  QTable *pTableSequences = pPropertyPage->m_pTableItems;
  
  // fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pqxxResultSequences, pTableSequences);

  // field names
  QHeader* hHeader = pTableSequences->horizontalHeader();

  int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Owner"));
  hHeader->setLabel(nCol++, i18n("ACL"));

  for(int nCol = 0; nCol < nTableCols; nCol++)
  {
    pTableSequences->adjustColumn(nCol);
  }
}

