//
// C++ Implementation: kpgtableconstraintsfolder
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgtableconstraintsfolder.h"

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <qtable.h>

#include "kpgconnection.h"
#include "../PropertyPages/kpgpropertypage.h"
#include "kpgprimarykeyconstr.h"
#include "kpgforeignkeyconstr.h"
#include "kpgcheckconstr.h"
#include "kpguniqueconstr.h"
#include "../kpgsqldialog.h"


KPGTableConstraintsFolder::KPGTableConstraintsFolder(KPGTable *parent)
 : KPGTableItemsFolder(parent, i18n("Constraints"))
{
	setPixmap(0, *m_pIconFolderGreen);
}

KPGTableConstraintsFolder::KPGTableConstraintsFolder(KPGTable *parent, KPGTableItemsFolder *after)
 : KPGTableItemsFolder(parent, after, i18n("Constraints"))
{
	setPixmap(0, *m_pIconFolderGreen);
}

KPGTableConstraintsFolder::~KPGTableConstraintsFolder()
{
}

bool KPGTableConstraintsFolder::refresh(work &pqxxXaction, pqxx::oid oidTable)
{
  // delete all child items
  while (QListViewItem * pItem = firstChild())
    delete pItem;

  QString strQuery("SELECT c.oid, conname, CASE contype WHEN 'p' THEN 'Primary Key' WHEN 'f' THEN 'Foreign Key' WHEN 'u' THEN 'Unique' WHEN 'c' THEN 'Check' END AS contypedesc, description, condeferrable, condeferred, nspname "); 
	
	strQuery.append("FROM pg_constraint c ");

	strQuery.append("JOIN pg_class cl ON cl.oid=conrelid ");
	strQuery.append("JOIN pg_namespace nl ON nl.oid=relnamespace ");
	strQuery.append("LEFT OUTER JOIN pg_description des ON des.objoid=c.oid ");
	
	strQuery.append("WHERE conrelid = ");
	strQuery.append(QString("%1").arg(oidTable));
	strQuery.append(" ORDER BY conname;");
		
  try
  {
	  m_pqxxResultConstraints = pqxxXaction.exec(strQuery.latin1());
		
		KPGTableConstraint * pPGTableConstraint = 0;
		
		for (result::size_type i = 0; i != m_pqxxResultConstraints.size(); ++i)
    {
		  pqxx::oid oid;
			m_pqxxResultConstraints[i][0].to(oid);
			
			QString strType(m_pqxxResultConstraints[i]["contypedesc"].c_str());
						
			
			if(strType == "Primary Key")
			{
				if(pPGTableConstraint == 0)
					pPGTableConstraint = new KPGPrimaryKeyConstr(this, m_pqxxResultConstraints[i]["conname"].c_str(), oid);
				else
					pPGTableConstraint = new KPGPrimaryKeyConstr(this, pPGTableConstraint, m_pqxxResultConstraints[i]["conname"].c_str(), oid);
			}
			
			if(strType == "Foreign Key")
			{
				
				if(pPGTableConstraint == 0)
					pPGTableConstraint = new KPGForeignKeyConstr(this, m_pqxxResultConstraints[i]["conname"].c_str(), oid);
				else
					pPGTableConstraint = new KPGForeignKeyConstr(this, pPGTableConstraint, m_pqxxResultConstraints[i]["conname"].c_str(), oid);
				
				KPGForeignKeyConstr *pForeignKeyConstr = static_cast <KPGForeignKeyConstr *> (pPGTableConstraint);	
				
				//-- Get foreign key properties
        strQuery = QString("SELECT confupdtype, confdeltype, conkey, confkey, conrelid, confrelid, refcl.relname AS reftable FROM pg_constraint c LEFT OUTER JOIN pg_class refcl ON refcl.oid=confrelid WHERE c.oid=%1;").arg(oid);
        pqxx::result pqxxResultFk = pqxxXaction.exec(strQuery.latin1());
                        
        if(pqxxResultFk.size() == 1)
        { 
					pqxx::oid oidReferencedTable;
					pqxxResultFk[0]["confrelid"].to(oidReferencedTable);
						
					pForeignKeyConstr->setUpdType(pqxxResultFk[0]["confupdtype"].c_str());
					pForeignKeyConstr->setDelType(pqxxResultFk[0]["confdeltype"].c_str());
					pForeignKeyConstr->setConstrKey(pqxxResultFk[0]["conkey"].c_str());	
					pForeignKeyConstr->setReferencedTable(oidReferencedTable, pqxxResultFk[0]["reftable"].c_str());
					pForeignKeyConstr->setForeignKey(pqxxResultFk[0]["confkey"].c_str());
        } 
			}
			
			if(strType == "Unique")
			{
				if(pPGTableConstraint == 0)
					pPGTableConstraint = new KPGUniqueConstr(this, m_pqxxResultConstraints[i]["conname"].c_str(), oid);
				else
					pPGTableConstraint = new KPGUniqueConstr(this, pPGTableConstraint, m_pqxxResultConstraints[i]["conname"].c_str(), oid);			
			}
			
			if(strType == "Check")
			{
				if(pPGTableConstraint == 0)
					pPGTableConstraint = new KPGCheckConstr(this, m_pqxxResultConstraints[i]["conname"].c_str(), oid);
				else
					pPGTableConstraint = new KPGCheckConstr(this, pPGTableConstraint, m_pqxxResultConstraints[i]["conname"].c_str(), oid);
				
				//-- Get check source
        QString strQuery(QString("SELECT consrc FROM pg_constraint c WHERE c.oid=%1;").arg(oid));
        pqxx::result pqxxResultExpression = pqxxXaction.exec(strQuery.latin1());
                        
        if(pqxxResultExpression.size() == 1)
        { 
        	(static_cast <KPGCheckConstr *> (pPGTableConstraint))->setExpression(pqxxResultExpression[0][0].c_str());
        }    
			}
			
			bool bIsDeferrable;
			bool bIsDeferred;
			
			m_pqxxResultConstraints[i]["condeferrable"].to(bIsDeferrable);
			m_pqxxResultConstraints[i]["condeferred"].to(bIsDeferred);
						
			pPGTableConstraint->setProperties(
				m_pqxxResultConstraints[i]["description"].c_str(), 
				bIsDeferrable,
				bIsDeferred,
				m_pqxxResultConstraints[i]["nspname"].c_str()
			);
		}
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return false;
  } 
  
  return true;
}

void KPGTableConstraintsFolder::displayProperties(QWidget *widgetProperies)
{
	KPGPropertyPage *pPropertyPage = static_cast <KPGPropertyPage *> (widgetProperies);
  pPropertyPage->setLabelText(i18n("Constraints &list:"));
	
	//----------------------------------------------------------------------------
	//Display constraints list
	QTable *pTableConstraints = pPropertyPage->m_pTableItems;
		
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pqxxResultConstraints, pTableConstraints);

	// field names
  QHeader* hHeader = pTableConstraints->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
	hHeader->setLabel(nCol++, i18n("Type"));
  hHeader->setLabel(nCol++, i18n("Description"));
  hHeader->setLabel(nCol++, i18n("Id Deferrable"));
  hHeader->setLabel(nCol++, i18n("Is Deferred"));
  hHeader->setLabel(nCol++, i18n("Namespace"));
  

  for(int nCol = 0; nCol < nTableCols; nCol++)
  {
    pTableConstraints->adjustColumn(nCol);
  }
}


