//
// C++ Implementation: kpgtableindexesfolder
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgtableindexesfolder.h"

// include files for Qt
#include <qtable.h>

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include "kpgconnection.h"
#include "kpgserver.h"
#include "../PropertyPages/kpgpropertypage.h"
#include "kpgprimarykey.h"
#include "kpgindex.h"
#include "../kpgsqldialog.h"

KPGTableIndexesFolder::KPGTableIndexesFolder(KPGTable *parent)
 : KPGTableItemsFolder(parent, i18n("Indexes"))
{
	setPixmap(0, *m_pIconFolderRed);
}

KPGTableIndexesFolder::KPGTableIndexesFolder(KPGTable *parent, KPGTableItemsFolder *after)
 : KPGTableItemsFolder(parent, after, i18n("Indexes"))
{
	setPixmap(0, *m_pIconFolderRed);
}

KPGTableIndexesFolder::~KPGTableIndexesFolder()
{
}

bool KPGTableIndexesFolder::refresh(work &pqxxXaction, pqxx::oid oidTable)
{
  // delete all child items (databases)
  while (QListViewItem * pItem = firstChild())
    delete pItem;
    
  // Get pointer to server for version info       table   -> tblfold-> schema ->database->server 
  KPGServer *pServer = static_cast <KPGServer *> (parent()->parent()->parent()->parent()->parent());

  QString strQuery("SELECT cls.oid, cls.relname as idxname, CASE contype WHEN 'p' THEN 'Primary Key' ELSE 'Index' END AS idxtypedesc, CASE contype WHEN 'p' THEN despkey.description ELSE desidx.description END AS description, indkey, indisunique, indisprimary, indisclustered, pg_get_indexdef(cls.oid) AS indexdef ");
  
  if(pServer->versionMajor() >= 8)
    strQuery.append(", cls.reltablespace, ts.spcname ");
	
	strQuery.append("FROM pg_index idx JOIN pg_class cls ON cls.oid=indexrelid JOIN pg_am am ON am.oid=cls.relam ");

	strQuery.append("LEFT JOIN pg_depend dep ON (dep.classid = cls.tableoid AND dep.objid = cls.oid AND dep.refobjsubid = '0') ");
	strQuery.append("LEFT OUTER JOIN pg_constraint con ON (con.tableoid = dep.refclassid AND con.oid = dep.refobjid)   ");
	strQuery.append("LEFT OUTER JOIN pg_description desidx ON desidx.objoid=cls.oid ");
	strQuery.append("LEFT OUTER JOIN pg_description despkey ON (despkey.objoid=con.oid AND despkey.objsubid = 0) ");
  
  if(pServer->versionMajor() >= 8)
    strQuery.append("LEFT JOIN pg_tablespace ts ON cls.reltablespace=ts.oid ");

	strQuery.append("WHERE indrelid =  ");
	strQuery.append(QString("%1").arg(oidTable));
	strQuery.append(" ORDER BY cls.relname;");
		
  try
  {
		m_pqxxResultIndexes = pqxxXaction.exec(strQuery.latin1());
		
		KPGTableIndex *pPGTableIndex = 0;
		
		for (result::size_type i = 0; i != m_pqxxResultIndexes.size(); ++i)
    {
		  pqxx::oid oid;
			m_pqxxResultIndexes[i][0].to(oid);
			
			QString strType(m_pqxxResultIndexes[i]["idxtypedesc"].c_str());
			
						
			if(strType == "Primary Key")
			{
				if(pPGTableIndex == 0)
					pPGTableIndex = new KPGPrimaryKey(this, m_pqxxResultIndexes[i]["idxname"].c_str(), oid);
				else
					pPGTableIndex = new KPGPrimaryKey(this, pPGTableIndex, m_pqxxResultIndexes[i]["idxname"].c_str(), oid);
			}
			
			if(strType == "Index")
			{
				if(pPGTableIndex == 0)
					pPGTableIndex = new KPGIndex(this, m_pqxxResultIndexes[i]["idxname"].c_str(), oid);
				else
					pPGTableIndex = new KPGIndex(this, pPGTableIndex, m_pqxxResultIndexes[i]["idxname"].c_str(), oid);
			}
			
			bool bIsUnique;
			bool bIsPrimary;
			bool bIsClustered;
						
			m_pqxxResultIndexes[i]["indisunique"].to(bIsUnique);
			m_pqxxResultIndexes[i]["indisprimary"].to(bIsPrimary);
			m_pqxxResultIndexes[i]["indisclustered"].to(bIsClustered);
			
      if(pServer->versionMajor() <= 7)
      {
        pPGTableIndex->setProperties(
				  m_pqxxResultIndexes[i]["description"].c_str(), 
				  m_pqxxResultIndexes[i]["indkey"].c_str(),
				  bIsUnique,
				  bIsPrimary,
				  bIsClustered,
          m_pqxxResultIndexes[i]["indexdef"].c_str()
			 );
      }
      else
      {
        pqxx::oid oidTablespace;
        m_pqxxResultIndexes[i]["reltablespace"].to(oidTablespace);
        
        pPGTableIndex->setProperties(
          m_pqxxResultIndexes[i]["description"].c_str(), 
          m_pqxxResultIndexes[i]["indkey"].c_str(),
          bIsUnique,
          bIsPrimary,
          bIsClustered,
          oidTablespace,
          m_pqxxResultIndexes[i]["spcname"].c_str(),
          m_pqxxResultIndexes[i]["indexdef"].c_str()
       );
      }
		}
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return false;
  } 
  
  return true;
}

void KPGTableIndexesFolder::displayProperties(QWidget *widgetProperies)
{
	// Get pointer to server for version info       table   -> tblfold-> schema ->database->server 
  KPGServer *pServer = static_cast <KPGServer *> (parent()->parent()->parent()->parent()->parent());
  
  KPGPropertyPage *pPropertyPage = static_cast <KPGPropertyPage *> (widgetProperies);
  pPropertyPage->setLabelText(i18n("Endexes &list:"));
	
	//----------------------------------------------------------------------------
	//Display indexes list
	QTable *pTableIndexes = pPropertyPage->m_pTableItems;
		
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pqxxResultIndexes, pTableIndexes);

	// field names
  QHeader* hHeader = pTableIndexes->horizontalHeader();

	int nCol = 0;
  hHeader->setLabel(nCol++, i18n("Name"));
	hHeader->setLabel(nCol++, i18n("Type"));
  hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Index key"));
	hHeader->setLabel(nCol++, i18n("Is Unique"));
	hHeader->setLabel(nCol++, i18n("Is Primary"));
	hHeader->setLabel(nCol++, i18n("Is Clustered"));
	hHeader->setLabel(nCol++, i18n("Procedure name"));
	hHeader->setLabel(nCol++, i18n("Procedure namespace"));
	hHeader->setLabel(nCol++, i18n("Access method"));
  if(pServer->versionMajor() >= 8) hHeader->setLabel(nCol++, i18n("Tablespace"));
	
  for(int nCol = 0; nCol < nTableCols; nCol++)
  {
    pTableIndexes->adjustColumn(nCol);
  }
}



