//
// C++ Implementation: %{MODULE}
//
// Description:
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgtablespacesfolder.h"

#include <qtable.h>

#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include "kpgconnection.h"
#include "kpgserver.h"
#include "kpgtablespace.h"
#include "../PropertyPages/kpgpropertypage.h"
#include "../kpgsqldialog.h"


KPGTablespacesFolder::KPGTablespacesFolder(KPGServer *parent)
 : KPGItemsFolder(parent, i18n("Tablespaces"))
{
}


KPGTablespacesFolder::~KPGTablespacesFolder()
{
}

bool KPGTablespacesFolder::refresh(pqxx::work &pqxxXaction)
{
  // delete all child items (databases)
  while (QListViewItem * pItem = firstChild())
    delete pItem;

	// obtain list of tablespaces
  QString strQuery("SELECT ts.oid, ts.spcname, sh.usename AS owner, ts.spclocation, ts.spcacl ");
	strQuery.append("FROM pg_tablespace ts ");
	strQuery.append("LEFT JOIN pg_shadow sh ON ts.spcowner=sh.usesysid  ");
		
  try
  {
    m_pqxxResultTablespaces = pqxxXaction.Exec(strQuery.latin1());

		KPGTablespace *pTablespace = 0;
		
    for (result::size_type i = 0; i != m_pqxxResultTablespaces.size(); ++i)
    {
			pqxx::oid oid;
			m_pqxxResultTablespaces[i][0].to(oid);
		
			if(pTablespace == 0)
      	pTablespace = new KPGTablespace(this, m_pqxxResultTablespaces[i]["spcname"].c_str(), oid);
			else
				pTablespace = new KPGTablespace(this, pTablespace, m_pqxxResultTablespaces[i]["spcname"].c_str(), oid);
					
			pTablespace->setProperties(
				m_pqxxResultTablespaces[i]["owner"].c_str(), 
				m_pqxxResultTablespaces[i]["spclocation"].c_str(),
				m_pqxxResultTablespaces[i]["spcacl"].c_str()
				);
    }
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return false;
  } 
  
  return true;
}

void KPGTablespacesFolder::displayProperties(QWidget *widgetProperies)
{
	KPGPropertyPage *pPropertyPage = static_cast <KPGPropertyPage *> (widgetProperies);
  pPropertyPage->setLabelText(i18n("Tablespaces &list:"));
	
	//----------------------------------------------------------------------------
	//Display tebles list
		
	QTable *pTableTablespaces = pPropertyPage->m_pTableItems;
  
  // fill properties table with result data
  int nTableCols = fillPropertiesTable(m_pqxxResultTablespaces, pTableTablespaces);

  // field names
  QHeader* hHeader = pTableTablespaces->horizontalHeader();

  int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  hHeader->setLabel(nCol++, i18n("Owner"));
	hHeader->setLabel(nCol++, i18n("Location"));
  hHeader->setLabel(nCol++, i18n("ACL"));
	
  for(int nCol = 0; nCol < nTableCols; nCol++)
  {
    pTableTablespaces->adjustColumn(nCol);
  }
}

