/***************************************************************************
                          kpgtreeitem.h  -  description
                             -------------------
    begin                : Ne led 11 2004
    copyright            : (C) 2004 by Dominik Sacher
    email                : dsacher@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPGTREEITEM_H
#define KPGTREEITEM_H

#include <qlistview.h>

// include files for libpgxx 
#include "pqxx/transaction.h" 
#include "pqxx/result.h" 

class KPGConnection;
class QTable;

/**
Storage class for OID and their object name.

@author Lumir Vanek
*/

class KPGOidName
{
public:
	KPGOidName() {;} 
    
	KPGOidName(pqxx::oid _oid, const QString & strName)
	{
			m_oid = _oid;
			m_strName = strName;
	}
	
	KPGOidName(const KPGOidName &src)
	{
			m_oid = src.oid();
			m_strName = src.name();
	}
		
					
	~KPGOidName() {;}
	
	void operator = (const KPGOidName &src)
	{
			m_oid = src.oid();
			m_strName = src.name();
	}
	
	// Return PostgreSQL row identifier
	const pqxx::oid oid() const { return m_oid; }
		
	// Return name
	const QString & name() const { return m_strName; }
	
protected:

	QString m_strName;

	// PostgreSQL row identifier
	pqxx::oid m_oid;		
};

/** Abstract base class for all tree items. Mother of KPGObject and KPGItemsFolder
  *@author Lumir Vanek
  */


class KPGTreeItem : public QListViewItem
{
public: 
	KPGTreeItem(QListView *);
  KPGTreeItem(KPGTreeItem *, const QString);
	KPGTreeItem(QListView *, KPGTreeItem *);
  KPGTreeItem(KPGTreeItem *, KPGTreeItem *, const QString);
	~KPGTreeItem();

  
  enum eNodeType { nodeUnselected = 0, nodeServer, nodeDatabase, nodeSchema, nodeTablespace, nodeTable, nodeView, nodeFunction, nodeSequence, nodeDomainConstr,
									 nodeTableColumn, nodePrimaryKeyConstr, nodeForeignKeyConstr, nodeUniqueConstr, nodeCheckConstr,
									 nodePrimaryKey, nodeIndex, nodeRule, nodeTrigger, nodeOperator, nodeOperatorClass, nodeAggregate, nodeUser, nodeGroup, nodeType, nodeLanguage, nodeDomain, nodeConversion, nodeCast,
	 								 nodeFolderTablespaces, nodeFolderTables, nodeFolderViews, nodeFolderSequences, nodeFolderFunctions,
									 nodeFolderTableColumns, nodeFolderTableIndexes, nodeFolderTableConstraints, nodeFolderDomainConstraints,
									 nodeFolderRules, nodeFolderTriggers,
									 nodeFolderOperators, nodeFolderOperatorClasses, nodeFolderAggregates, nodeFolderUsers, nodeFolderGroups, nodeFolderTypes, nodeFolderLanguages, nodeFolderDomains, nodeFolderConversions, nodeFolderCasts
								 };

  virtual eNodeType type() const = 0; // return tree node type
	virtual bool hasOid() const { return false; }
  virtual void displayProperties(QWidget *) = 0; // display node properties in right windows
	virtual KPGConnection * connection();
	virtual bool refresh() = 0;
  virtual bool smartRefresh() { return refresh(); }
	virtual KPGTreeItem * searchObject(const QString &, pqxx::oid); // search object with given path and OID
	const QString path();
	
  QString makeConnectionString(const QString, const QString, const QString, const QString, const QString);
  QString makeConnectionString(const QString, const QString, const QString, const QString);

	// Return child item with given name
	KPGTreeItem * getChildByName(const QString &);

  // Return nearest parent item with given type
  KPGTreeItem * getParentByType(const eNodeType);
  
protected:
	int fillPropertiesTable(pqxx::result, QTable *);
	
public:
	static QPixmap *m_pIconServer;
	static QPixmap *m_pIconTablespaceSys;
	static QPixmap *m_pIconTablespaceUsr;
	static QPixmap *m_pIconDatabase;
	static QPixmap *m_pIconDatabaseSys;
	static QPixmap *m_pIconDatabaseUsr;
	static QPixmap *m_pIconSchema;
	static QPixmap *m_pIconSchemaSys;
	static QPixmap *m_pIconSchemaUsr;
	static QPixmap *m_pIconTable;
	static QPixmap *m_pIconView;
	static QPixmap *m_pIconSequence;
	static QPixmap *m_pIconOperator;
	static QPixmap *m_pIconOperatorClass;
	static QPixmap *m_pIconAggregate;
	static QPixmap *m_pIconFunction;
	static QPixmap *m_pIconFunctionC;
	static QPixmap *m_pIconFunctionSql;
	static QPixmap *m_pIconFunctionPl;
	static QPixmap *m_pIconUser;
	static QPixmap *m_pIconUserSuper;
	static QPixmap *m_pIconUserCanCreateDb;
	static QPixmap *m_pIconGroup;
	static QPixmap *m_pIconType;
	static QPixmap *m_pIconTypeBase;
	static QPixmap *m_pIconTypeComposite;
	static QPixmap *m_pIconTypeStandalone;
	static QPixmap *m_pIconLanguage;
	static QPixmap *m_pIconDomain;
	static QPixmap *m_pIconConversion;
	static QPixmap *m_pIconCast;
		
	static QPixmap *m_pIconColumnOther;
	static QPixmap *m_pIconColumnBool; 
	static QPixmap *m_pIconColumnGeo;   
	static QPixmap *m_pIconColumnNet; 
	static QPixmap *m_pIconColumnOid;  
	static QPixmap *m_pIconColumnXid;
	static QPixmap *m_pIconColumnTid;
	static QPixmap *m_pIconColumnCid;
	static QPixmap *m_pIconColumnDate; 
	static QPixmap *m_pIconColumnChar;  
	static QPixmap *m_pIconColumnNum;
	static QPixmap *m_pIconColumnTime;
	static QPixmap *m_pIconColumnBit;
	static QPixmap *m_pIconColumnBin;
		
	static QPixmap *m_pIconPrimaryKeyConstr;
	static QPixmap *m_pIconForeignKeyConstr;
	static QPixmap *m_pIconUniqueConstr;
	static QPixmap *m_pIconCheckConstr;
	static QPixmap *m_pIconPrimaryKey;
	static QPixmap *m_pIconIndex;
	static QPixmap *m_pIconRule;
	static QPixmap *m_pIconTrigger;
	
	static QPixmap *m_pIconFolderBlue;
	static QPixmap *m_pIconFolderRed;
	static QPixmap *m_pIconFolderCyan;
	static QPixmap *m_pIconFolderGreen;
	static QPixmap *m_pIconFolderYellow;
	static QPixmap *m_pIconFolderViolet;
	static QPixmap *m_pIconFolderGrey;
};

#endif
