//
// C++ Implementation: kpgtrigger
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgtrigger.h"

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <qtable.h>

#include "../kpglinklabel.h"
#include "kpgtriggersfolder.h"
#include "../PropertyPages/kpgtriggerproppage.h"

KPGTrigger::KPGTrigger(KPGTriggersFolder *parent, const QString name, pqxx::oid _oid)
 : KPGObject(parent, name, _oid)
{
	setPixmap(0, *m_pIconTrigger);
}

KPGTrigger::KPGTrigger(KPGTriggersFolder *parent, KPGTrigger *after, const QString name, pqxx::oid _oid)
 : KPGObject(parent, after, name, _oid)
{
	setPixmap(0, *m_pIconTrigger);
}

KPGTrigger::~KPGTrigger()
{
}

void KPGTrigger::setProperties(
		const QString strDescription,
		bool bIsEnabled,
	  bool bDeferrable,
	  bool bInitDeferred,
	  int iNArgs,
		const QString strArgs,
	  const QString strNameSpace,
		pqxx::oid oidProcedure,
		const QString strProcName
	)
{
	m_strDescription = strDescription;
	m_bIsEnabled = bIsEnabled;
	m_bDeferrable = bDeferrable;
	m_bInitDeferred = bInitDeferred;
	m_iNArgs = iNArgs;
	m_strArgs = strArgs;
	m_strNameSpace = strNameSpace;
	m_oidProcedure = oidProcedure;
	m_strProcName = strProcName;
}

void KPGTrigger::displayProperties(QWidget *widgetProperies)
{
	// Display server version info
	KPGTriggerPropPage *pPropertyPage = static_cast <KPGTriggerPropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
  QHeader* hHeader = pTableProperties->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
  	
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
	
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Trigger name (must be unique among triggers of same table)"));
	
	//--- Description
	pTableProperties->setText(iRow, 0, i18n("Description"));
  pTableProperties->setText(iRow, 1, m_strDescription);
	pTableProperties->setText(iRow++, 2, i18n("Description of the trigger"));
		
	//--- Is Enabled
	pTableProperties->setText(iRow, 0, i18n("Is Enabled"));
  pTableProperties->setText(iRow, 1, (m_bIsEnabled) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("True if trigger is enabled (not presently checked everywhere it should be, so disabling a trigger by setting this false does not work reliably)"));
	
	//--- Is Deferrable
	pTableProperties->setText(iRow, 0, i18n("Is Deferrable"));
  pTableProperties->setText(iRow, 1, (m_bDeferrable) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("True if deferrable"));
	
	//--- Is Is initially Deferred
	pTableProperties->setText(iRow, 0, i18n("Is initially Deferred"));
  pTableProperties->setText(iRow, 1, (m_bInitDeferred) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("True if initially deferred"));
	
	//--- Number of argument
	pTableProperties->setText(iRow, 0, i18n("Number of arguments"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_iNArgs));
	pTableProperties->setText(iRow++, 2, i18n("Number of argument strings passed to trigger function"));
	
	//--- Arguments
	pTableProperties->setText(iRow, 0, i18n("Arguments"));
  pTableProperties->setText(iRow, 1, m_strArgs);
	pTableProperties->setText(iRow++, 2, i18n("Argument strings to pass to trigger, each null-terminated"));
	
	//--- Namespace
	pTableProperties->setText(iRow, 0, i18n("Namespace"));
  pTableProperties->setText(iRow, 1, m_strNameSpace);
	pTableProperties->setText(iRow++, 2, i18n("Name of the namespace"));
	
	//--- Procedure name
	pTableProperties->setText(iRow, 0, i18n("Procedure name"));
  KPGLinkLabel *pLabel = new KPGLinkLabel( pTableProperties, "Procedure name" );
	pLabel->setValues(m_oidProcedure, m_strProcName);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Name of the trigger procedure, to be called"));
	
	for(int nCol = 0; nCol < 3; nCol++)
  {
    pTableProperties->adjustColumn(nCol);
  }
}


