//
// C++ Implementation: kpgtriggersfolder
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgtriggersfolder.h"

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <qtable.h>

#include "kpgconnection.h"
#include "../PropertyPages/kpgpropertypage.h"
#include "kpgtrigger.h"
#include "../kpgsqldialog.h"

KPGTriggersFolder::KPGTriggersFolder(KPGTable *parent)
 : KPGTableItemsFolder(parent, i18n("Triggers"))
{
	setPixmap(0, *m_pIconFolderViolet);
}

KPGTriggersFolder::KPGTriggersFolder(KPGTable *parent, KPGTableItemsFolder *after)
 : KPGTableItemsFolder(parent, after, i18n("Triggers"))
{
	setPixmap(0, *m_pIconFolderViolet);
}

KPGTriggersFolder::~KPGTriggersFolder()
{
}

bool KPGTriggersFolder::refresh(work &pqxxXaction, pqxx::oid oidTable)
{
  // delete all child items (databases)
  while (QListViewItem * pItem = firstChild())
    delete pItem;

  QString strQuery("SELECT t.oid, t.tgname, des.description, t.tgenabled, t.tgdeferrable, t.tginitdeferred, tgnargs, tgargs, nspname, tgfoid,  pr.proname ");
	
	strQuery.append("FROM pg_trigger t ");
	strQuery.append("JOIN pg_class cl ON cl.oid=tgrelid ");
	strQuery.append("JOIN pg_namespace na ON na.oid=relnamespace ");
  strQuery.append("LEFT OUTER JOIN pg_description des ON des.objoid=t.oid ");
	strQuery.append("LEFT OUTER JOIN pg_proc pr ON tgfoid=pr.oid ");
  strQuery.append("WHERE NOT tgisconstraint ");
  strQuery.append(" AND tgrelid = ");
	strQuery.append(QString("%1").arg(oidTable));
  strQuery.append(" ORDER BY tgname");
		
  try
  {
	  m_pqxxResultTriggers = pqxxXaction.exec(strQuery.latin1());
		
		KPGTrigger *pgTrigger = 0;
		
		for (result::size_type i = 0; i != m_pqxxResultTriggers.size(); ++i)
    {
			pqxx::oid oid;
			m_pqxxResultTriggers[i][0].to(oid);
						
			if(pgTrigger == 0)
				pgTrigger = new KPGTrigger(this, m_pqxxResultTriggers[i]["tgname"].c_str(), oid);
			else
				pgTrigger = new KPGTrigger(this, pgTrigger, m_pqxxResultTriggers[i]["tgname"].c_str(), oid);
		
			bool bIsEnabled;
			bool bDeferrable;
			bool bInitDeferred;
			int iNArgs;
			pqxx::oid oidProcedure = 0;
			
			m_pqxxResultTriggers[i]["tgenabled"].to(bIsEnabled);
	  	m_pqxxResultTriggers[i]["tgdeferrable"].to(bDeferrable);
	  	m_pqxxResultTriggers[i]["tginitdeferred"].to(bInitDeferred);
	  	m_pqxxResultTriggers[i]["tgnargs"].to(iNArgs);
			m_pqxxResultTriggers[i]["tgfoid"].to(oidProcedure);
				
			pgTrigger->setProperties(
				m_pqxxResultTriggers[i]["description"].c_str(),
				bIsEnabled,
				bDeferrable,
				bInitDeferred,
				iNArgs,
				m_pqxxResultTriggers[i]["tgargs"].c_str(),
				m_pqxxResultTriggers[i]["nspname"].c_str(),
				oidProcedure,
				m_pqxxResultTriggers[i]["proname"].c_str()
				);
		}
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return false;
  } 
  
  return true;
}

void KPGTriggersFolder::displayProperties(QWidget *widgetProperies)
{
	KPGPropertyPage *pPropertyPage = static_cast <KPGPropertyPage *> (widgetProperies);
  pPropertyPage->setLabelText(i18n("Triggers &list:"));
	
	//----------------------------------------------------------------------------
	//Display triggers list
	QTable *pTableTriggers = pPropertyPage->m_pTableItems;
			
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pqxxResultTriggers, pTableTriggers);

	// field names
  QHeader* hHeader = pTableTriggers->horizontalHeader();

	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
	hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Is Enabled"));
  hHeader->setLabel(nCol++, i18n("Is Deferrable"));
  hHeader->setLabel(nCol++, i18n("Is initially Deferred"));
  hHeader->setLabel(nCol++, i18n("Number of arguments"));
  hHeader->setLabel(nCol++, i18n("Arguments"));
	hHeader->setLabel(nCol++, i18n("Namespace"));
	hHeader->setLabel(nCol++, i18n("Procedure"));

  for(int nCol = 0; nCol < nTableCols; nCol++)
  {
    pTableTriggers->adjustColumn(nCol);
  }
}

