//
// C++ Implementation: kpgtype
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgtype.h"

// include files for KDE
#include <kiconloader.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <ktabwidget.h>

#include <qtable.h>

#include "../kpglinklabel.h"
#include "kpgtypesfolder.h"
#include "../PropertyPages/kpgtypeproppage.h"
#include "../kpgsqldialog.h"

KPGType::KPGType(KPGTypesFolder *parent, const QString name, pqxx::oid _oid)
  : KPGObject(parent, name, _oid)
{
  setPixmap(0, *m_pIconType);
}

KPGType::KPGType(KPGTypesFolder *parent, KPGType *after, const QString name, pqxx::oid _oid)
  : KPGObject(parent, after, name, _oid)
{
  setPixmap(0, *m_pIconType);
}

KPGType::~KPGType()
{
}

void KPGType::setProperties(
	const QString strDescription,
	const QString strKind,
	int iTypLen,
	bool bByVal,
	bool bIsDefined,
	const QString strTypDelim,
	const QString strTypAlign,
	const QString strTypStorage,
	pqxx::oid oidTypElem,
	const QString strElement,
	pqxx::oid oidInputFunc,
	const QString strInputFunc,
	pqxx::oid oidOutputFunc,
	const QString strOutputFunc,
	pqxx::oid oidTable,
	const QString strTableName,
	const QString strOwner
	)
{
		m_strDescription = strDescription;
		m_strKind = strKind;
		m_iTypLen = iTypLen; 
		m_bByVal = bByVal; 
		m_bIsDefined = bIsDefined; 
		m_strTypDelim = strTypDelim;
		m_strTypAlign = strTypAlign;
		m_strTypStorage = strTypStorage;
		m_oidTypElem = oidTypElem;
		m_strElement = strElement;
		m_oidInputFunc = oidInputFunc;
		m_strInputFunc = strInputFunc;
		m_oidOutputFunc = oidOutputFunc;
		m_strOutputFunc = strOutputFunc;
		m_oidTable = oidTable;
		m_strTableName = strTableName;
		m_strOwner = strOwner;
		
		//--- Set icon for type
		if(m_strKind == "composite")
		{
			if(m_oidTable == 0)
				setPixmap(0, *m_pIconTypeStandalone);
			else
			  setPixmap(0, *m_pIconTypeComposite);
		}
		
		if(m_strKind == "base")
		{
			setPixmap(0, *m_pIconTypeBase);
		}
}

void KPGType::displayProperties(QWidget *widgetProperies)
{
	// Display data type info
	KPGTypePropPage *pPropertyPage = static_cast <KPGTypePropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
  QHeader* hHeader = pTableProperties->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
  
	//--- OID
	pTableProperties->setText(iRow, 0, i18n("OID"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_oid));
	pTableProperties->setText(iRow++, 2, i18n("PostgreSQL row identifier"));
		
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("Type name"));
	
	//--- Description
	pTableProperties->setText(iRow, 0, i18n("Description"));
  pTableProperties->setText(iRow, 1, m_strDescription);
	pTableProperties->setText(iRow++, 2, i18n("Description of the type"));
	
	//--- Type Name
	pTableProperties->setText(iRow, 0, i18n("Kind"));
  pTableProperties->setText(iRow, 1, m_strKind);
	pTableProperties->setText(iRow++, 2, i18n("Kind of this type"));
	
	//--- Type length
	pTableProperties->setText(iRow, 0, i18n("Type length"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_iTypLen));
	pTableProperties->setText(iRow++, 2, i18n("For a fixed-size type, typlen is the number of bytes in the internal representation of the type. But for a variable-length type, typlen is negative. -1 indicates a \"varlena\" type (one that has a length word), -2 indicates a null-terminated C string."));
	
	//--- By Value
	pTableProperties->setText(iRow, 0, i18n("By value"));
  pTableProperties->setText(iRow, 1, (m_bByVal) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("Determines whether internal routines pass a value of this type by value or by reference. It had better be false if typlen is not 1, 2, or 4 (or 8 on machines where Date is 8 bytes). Variable-length types are always passed by reference. Note that typbyval can be false even if the length would allow pass-by-value; this is currently true for type float4, for example."));
	
	//--- Is Defined
	pTableProperties->setText(iRow, 0, i18n("Is Defined"));
  pTableProperties->setText(iRow, 1, (m_bIsDefined) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("True if the type is defined, false if this is a placeholder entry for a not-yet-defined type. When typisdefined is false, nothing except the type name, namespace, and OID can be relied on."));
	
	//--- Array delimiter
	pTableProperties->setText(iRow, 0, i18n("Array delimiter"));
  pTableProperties->setText(iRow, 1, m_strTypDelim);
	pTableProperties->setText(iRow++, 2, i18n("Character that separates two values of this type when parsing array input. Note that the delimiter is associated with the array element data type, not the array data type."));
	
	//--- Align
	pTableProperties->setText(iRow, 0, i18n("Align"));
  pTableProperties->setText(iRow, 1, m_strTypAlign);
	pTableProperties->setText(iRow++, 2, i18n("Alignment required when storing a value of this type. It applies to storage on disk as well as most representations of the value inside PostgreSQL. When multiple values are stored consecutively, such as in the representation of a complete row on disk, padding is inserted before a datum of this type so that it begins on the specified boundary."));
	
	//--- Storage
	pTableProperties->setText(iRow, 0, i18n("Storage"));
  pTableProperties->setText(iRow, 1, m_strTypStorage);
	pTableProperties->setText(iRow++, 2, i18n("Storage tells for varlena types (those with typlen = -1) if the type is prepared for toasting and what the default strategy for attributes of this type should be."));
	
	//--- Element
	pTableProperties->setText(iRow, 0, i18n("Element"));
  KPGLinkLabel *pLabel = new KPGLinkLabel( pTableProperties, "Element" );
	pLabel->setValues(m_oidTypElem, m_strElement);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Element type name"));
	
	//--- Input function
	pTableProperties->setText(iRow, 0, i18n("Input function"));
  pLabel = new KPGLinkLabel( pTableProperties, "Input function" );
	pLabel->setValues(m_oidInputFunc, m_strInputFunc);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Input conversion function"));
		
	//--- Output function
	pTableProperties->setText(iRow, 0, i18n("Output function"));
  pLabel = new KPGLinkLabel( pTableProperties, "Output function" );
	pLabel->setValues(m_oidOutputFunc, m_strOutputFunc);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("Output conversion function "));
		
	//--- Table OID
	pTableProperties->setText(iRow, 0, i18n("Corresponding object"));
  pLabel = new KPGLinkLabel( pTableProperties, "Corresponding object" );
	pLabel->setValues(m_oidTable, m_strTableName);
	pTableProperties->setCellWidget(iRow, 1, pLabel);
	pTableProperties->setText(iRow++, 2, i18n("If this is a composite type, then this points to the the corresponding table, view or sequence."));
		
	//--- Owner
	pTableProperties->setText(iRow, 0, i18n("Owner"));
  pTableProperties->setText(iRow, 1, m_strOwner);
	pTableProperties->setText(iRow++, 2, i18n("Owner of the type"));
	
	for(int nCol = 0; nCol < 3; nCol++)
  {
    pTableProperties->adjustColumn(nCol);
  }
	
	//--- Attributes
	displayAttributes(widgetProperies);
}

bool KPGType::refresh(pqxx::work &pqxxXaction)
{
  // delete all child items (databases)
  while (QListViewItem * pItem = firstChild())
    delete pItem;

  QString strQuery("SELECT attname, t.typname, a.attlen, attnum, attndims, atttypmod, nspname ");
		
  strQuery.append("FROM pg_attribute a ");
  strQuery.append("JOIN pg_type t ON t.oid=atttypid ");
  strQuery.append("JOIN pg_namespace nsp ON t.typnamespace=nsp.oid ");
  strQuery.append("JOIN pg_class c ON a.attrelid=c.oid ");
  strQuery.append(" WHERE c.reltype = ");
	strQuery.append(QString("%1").arg(m_oid));
  strQuery.append(" ORDER BY attnum");
		
  try
  {
	  m_pqxxResultAttributes = pqxxXaction.exec(strQuery.latin1());
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return false;
  } 
  
  return true;
}

void KPGType::displayAttributes(QWidget *widgetProperies)
{
	KPGTypePropPage *pPropertyPage = static_cast <KPGTypePropPage *> (widgetProperies);
	
	if((m_strKind == "base") || (m_strKind == "pseudo"))
	{
		// base and pseudo types don't have properties
		pPropertyPage->m_pTabWidget->setTabEnabled(pPropertyPage->tabAttributes, false);
	  return;
	}
	else
	{
		pPropertyPage->m_pTabWidget->setTabEnabled(pPropertyPage->tabAttributes, true);
	}
	
	//----------------------------------------------------------------------------
	//Display attributes list
	QTable *pTableAttributes = pPropertyPage->m_pTableAttributes;
			
	// Fill output area with query result
  int nRows = m_pqxxResultAttributes.size();
  int nCols = m_pqxxResultAttributes.columns();

  pTableAttributes->setNumRows(nRows);
  pTableAttributes->setNumCols(nCols);

	//--- Header
	// field names
  QHeader* hHeader = pTableAttributes->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Type"));
	hHeader->setLabel(2, i18n("Type length"));
  hHeader->setLabel(3, i18n("Attribute number"));
  hHeader->setLabel(4, i18n("Dimensions"));
  hHeader->setLabel(5, i18n("Type mode"));
  hHeader->setLabel(6, i18n("Namespace"));
	
  // data
  for(int nRow = 0; nRow < nRows; nRow++)
  {
    for(int nCol = 0; nCol < nCols; nCol++)
    {
      QString strValue(m_pqxxResultAttributes[nRow][nCol].c_str());
      pTableAttributes->setText(nRow, nCol, strValue);
    }
  }

  for(int nCol = 0; nCol < nCols; nCol++)
  {
    pTableAttributes->adjustColumn(nCol);
  }
}

