//
// C++ Implementation: kpguser
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpguser.h"

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <ktextedit.h>

#include <qtable.h>
#include <qdatetime.h>

#include "kpgusersfolder.h"
#include "../PropertyPages/kpguserproppage.h"

KPGUser::KPGUser(KPGUsersFolder *parent, const QString name)
  : KPGTreeItem(parent, name)
{
  setPixmap(0, *m_pIconUser);
}

KPGUser::KPGUser(KPGUsersFolder *parent, KPGUser *after, const QString name)
  : KPGTreeItem(parent, after, name)
{
  setPixmap(0, *m_pIconUser);
}

KPGUser::~KPGUser()
{
}

const QDate KPGUser::validUntil() const
{
	QDate dateValidUntil;
	
	if(m_strValUntil.length() > 9)
	{
		QString strYear(m_strValUntil.left(4));
		QString strMonth(m_strValUntil.mid(5, 2));
		QString strDay(m_strValUntil.mid(8, 2));
		
		QDate d(strYear.toInt(), strMonth.toInt(), strDay.toInt());
		dateValidUntil = d;
	}
	
	return dateValidUntil;
}

void KPGUser::setProperties(
		int iSysId, 
		bool bCreateDb, 
		bool bIsSuper, 
		bool bUCatUpd, 
		const QString strValUntil
	)
{
	m_iSysId = iSysId; 
	m_bCreateDb = bCreateDb; 
	m_bIsSuper = bIsSuper; 
	m_bUCatUpd = bUCatUpd; 
	m_strValUntil = strValUntil;
	
	if(bIsSuper)
		setPixmap(0, *m_pIconUserSuper);
	else
		{
			if(bCreateDb)
				setPixmap(0, *m_pIconUserCanCreateDb);
			else
				setPixmap(0, *m_pIconUser);
		}
}

void KPGUser::displayProperties(QWidget *widgetProperies)
{
	// Display server version info
	KPGUserPropPage *pPropertyPage = static_cast <KPGUserPropPage *> (widgetProperies);
	
	//---------------------------------------------------------------------------
	// Display properties
	QTable *pTableProperties = pPropertyPage->m_pTableProperties;
	
	//--- Header
	// field names
  QHeader* hHeader = pTableProperties->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
	hHeader->setLabel(1, i18n("Value"));
	hHeader->setLabel(2, i18n("Description"));
	
	int iRow = 0;
  	
	//--- Name
	pTableProperties->setText(iRow, 0, i18n("Name"));
  pTableProperties->setText(iRow, 1, text(0));
	pTableProperties->setText(iRow++, 2, i18n("User name"));
	
	//--- System ID
	pTableProperties->setText(iRow, 0, i18n("System ID"));
  pTableProperties->setText(iRow, 1, QString("%1").arg(m_iSysId));
	pTableProperties->setText(iRow++, 2, i18n("User id (arbitrary number used to reference this user)"));
	
	
	//--- Create DB
	pTableProperties->setText(iRow, 0, i18n("May Create DB"));
  pTableProperties->setText(iRow, 1, (m_bCreateDb) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("User may create databases"));
	
	//--- Is superuser
	pTableProperties->setText(iRow, 0, i18n("Is Superuser"));
  pTableProperties->setText(iRow, 1, (m_bIsSuper) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("User is a superuser"));
			
	//--- May update catalog
	pTableProperties->setText(iRow, 0, i18n("May update catalog"));
  pTableProperties->setText(iRow, 1, (m_bUCatUpd) ? i18n("True") : i18n("False"));
	pTableProperties->setText(iRow++, 2, i18n("User may update system catalogs. (Even a superuser may not do this unless this column is true.)"));
	
	//--- Valid until
	pTableProperties->setText(iRow, 0, i18n("Valid until"));
  pTableProperties->setText(iRow, 1, m_strValUntil);
	pTableProperties->setText(iRow++, 2, i18n("Account expiry time (only used for password authentication)"));
	
	
	for(int nCol = 0; nCol < 3; nCol++)
  {
    pTableProperties->adjustColumn(nCol);
  }
}

