//
// C++ Implementation: kpgusersfolder
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgusersfolder.h"

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <qtable.h>

#include "kpgconnection.h"
#include "kpgserver.h"
#include "../PropertyPages/kpgpropertypage.h"
#include "kpguser.h"
#include "../kpgsqldialog.h"

KPGUsersFolder::KPGUsersFolder(KPGServer *parent)
 : KPGItemsFolder(parent, i18n("Users"))
{
	setPixmap(0, *m_pIconFolderGrey);
}


KPGUsersFolder::~KPGUsersFolder()
{
}

bool KPGUsersFolder::refresh(pqxx::work &pqxxXaction)
{
  // delete all child items (databases)
  while (QListViewItem * pItem = firstChild())
    delete pItem;

	// obtain list of users
  QString strQuery("SELECT usename, usesysid, usecreatedb, usesuper, usecatupd, valuntil FROM ");
	strQuery.append("pg_user u "); // or pg_shadow
	strQuery.append("ORDER BY usename");
			
  try
  {
    m_pqxxResultUsers = pqxxXaction.Exec(strQuery.latin1());

		KPGUser *pgUser = 0;
		
    for (result::size_type i = 0; i != m_pqxxResultUsers.size(); ++i)
    {
			if(pgUser == 0)
				pgUser = new KPGUser(this, m_pqxxResultUsers[i]["usename"].c_str());
			else
				pgUser = new KPGUser(this, pgUser, m_pqxxResultUsers[i]["usename"].c_str());
			
			int iSysId; 
			bool bCreateDb; 
			bool bIsSuper; 
			bool bUCatUpd;
			
			m_pqxxResultUsers[i]["usesysid"].to(iSysId);
			m_pqxxResultUsers[i]["usecreatedb"].to(bCreateDb);
			m_pqxxResultUsers[i]["usesuper"].to(bIsSuper);
			m_pqxxResultUsers[i]["usecatupd"].to(bUCatUpd);
		
			pgUser->setProperties(
				iSysId, 
				bCreateDb, 
				bIsSuper, 
				bUCatUpd, 
				m_pqxxResultUsers[i]["valuntil"].c_str()
			);
    }
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return false;
  } 
  
  return true;
}
void KPGUsersFolder::displayProperties(QWidget *widgetProperies)
{
	KPGPropertyPage *pPropertyPage = static_cast <KPGPropertyPage *> (widgetProperies);
  pPropertyPage->setLabelText(i18n("Users &list:"));
	
	//----------------------------------------------------------------------------
	//Display views list
		
	// Fill output area with query result
  int nRows = m_pqxxResultUsers.size();
  int nCols = m_pqxxResultUsers.columns();

  QTable *pTableUsers = pPropertyPage->m_pTableItems;
  
  pTableUsers->setNumRows(nRows);
  pTableUsers->setNumCols(nCols);

  // field names
  QHeader* hHeader = pTableUsers->horizontalHeader();

  int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  hHeader->setLabel(nCol++, i18n("System ID"));
	hHeader->setLabel(nCol++, i18n("May Create DB"));
  hHeader->setLabel(nCol++, i18n("Is Superuser"));
	hHeader->setLabel(nCol++, i18n("May update catalog"));
	hHeader->setLabel(nCol++, i18n("Valid until"));
	
  // data
  for(int nRow = 0; nRow < nRows; nRow++)
  {
    for(int nCol = 0; nCol < nCols; nCol++)
    {
      QString strValue(m_pqxxResultUsers[nRow][nCol].c_str());
      pTableUsers->setText(nRow, nCol, strValue);
    }
  }

  for(int nCol = 0; nCol < nCols; nCol++)
  {
    pTableUsers->adjustColumn(nCol);
  }
}


