//
// C++ Implementation: kpgcreateviewwidget1
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgcreateviewwidget1.h"

// include files for Qt
#include <qpushbutton.h>

// include files for KDE
#include <kcombobox.h>
#include <klineedit.h>
#include <klistbox.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <klocale.h>

// application specific includes
#include "../DbObjects/kpgtreeitem.h"
#include "../DbObjects/kpgdatabase.h"
#include "../DbObjects/kpgschema.h"
#include "../DbObjects/kpgtablesfolder.h"


KPGCreateViewWidget1::KPGCreateViewWidget1(QWidget *parent, const char *name, KPGDatabase *pDatabase, QPtrList<QListViewItem> & listOfSchemas)
 : KPGCreateViewWidget1Base(parent, name)
{
  m_pDatabase = pDatabase;
	m_pLineEditViewName->setFocus();
	
	m_bIsDirty1 = false;
	m_bIsDirty2 = false;
	m_bIsDirty3 = false;
   
  int iCurrentItem = -1;
  int i = 0;  
  for(QListViewItem *pSchema = listOfSchemas.first(); pSchema; pSchema = listOfSchemas.next())
    {
      m_pComboBoxSchema->insertItem(* pSchema->pixmap(0), pSchema->text(0));
      
      if(pSchema->text(0) == "public")
        iCurrentItem = i;
        
      ++i;
    }
    
  if(iCurrentItem >= 0)
    m_pComboBoxSchema->setCurrentItem(iCurrentItem);
}


KPGCreateViewWidget1::~KPGCreateViewWidget1()
{
}

void KPGCreateViewWidget1::slotViewNameChanged(const QString& strText)
{
	emit sigEnableNextButton((! strText.isEmpty()) && (m_pListBoxViewTables->count() > 0));
}

void KPGCreateViewWidget1::slotSchemaActivated(const QString & strSchema)
{
	refreshAvailTablesList(strSchema);
}

void KPGCreateViewWidget1::refreshAvailTablesList(const QString & strNamespace)
{
	m_pListBoxAvailTables->clear();
	m_pListBoxViewTables->clear();
	
	// list of available tables
  KPGTreeItem *pItem = m_pDatabase->getChildByName(strNamespace);
  if(!pItem)
  {
    kdDebug() << "KPGCreateViewWizard::showPage: no item found: " << strNamespace << endl;
    return;
  }
        
  KPGSchema *pSchema = static_cast <KPGSchema *> (pItem);
  pSchema->getTablesFolder()->fillListBoxWithChildItems(m_pListBoxAvailTables); 
}

void KPGCreateViewWidget1::slotAvailableTablesSelectChanged(QListBoxItem* pSelectedItem)
{
	m_pPushButtonTableAdd->setEnabled(pSelectedItem != 0);
}

void KPGCreateViewWidget1::slotViewTablesSelectChanged(QListBoxItem* pSelectedItem)
{
	m_pPushButtonTableDel->setEnabled(pSelectedItem != 0);
}

void KPGCreateViewWidget1::slotTableAdd()
{
	QListBoxItem *pSelectedItem = m_pListBoxAvailTables->selectedItem();
	if(!pSelectedItem) return;
	
	// Zabraneni duplicite
	QString strTableName(m_pComboBoxSchema->currentText() + ".");
	
	if(m_pLineEditAlias->text().isEmpty())
		strTableName.append(pSelectedItem->text());
	else
		strTableName.append(pSelectedItem->text() + " " + m_pLineEditAlias->text());
		
	for(unsigned int i = 0; i < m_pListBoxViewTables->count(); i++)
	{
		QListBoxItem * pItem = m_pListBoxViewTables->item(i);
		
		// Avoid table name duplicity
		if(pItem->text() == strTableName)
		{
			KMessageBox::sorry(this, i18n("Table name or alias already exists !"));
			return;	
		}
		
		// Avoid Alias duplicity
		if(m_pLineEditAlias->text().isEmpty() == false)
		{
			QString strAlias(pItem->text());
			// strip alias, if any
			int iPos = strAlias.find(' ');
			if(iPos > 0)
				strAlias = strAlias.mid(iPos + 1);
				
			if(m_pLineEditAlias->text() == strAlias)
			{
				KMessageBox::sorry(this, i18n("Table with this alias already exists !"));
				return;	
			}
		}
	}
	
	// Move column from available list to list 		
	m_pListBoxViewTables->insertItem(* pSelectedItem->pixmap(), strTableName);
		
	// set default view name, if empty
	if(m_pLineEditViewName->text().isEmpty())
		m_pLineEditViewName->setText("vw_" + pSelectedItem->text());
	
	// clear alias
	m_pLineEditAlias->setText("");
	
	emit sigEnableNextButton((! m_pLineEditViewName->text().isEmpty()) && (m_pListBoxViewTables->count() > 0));
	
	m_bIsDirty1 = true;
	m_bIsDirty2 = true;
	m_bIsDirty3 = true;
}

void KPGCreateViewWidget1::slotTableDelete()
{
	QListBoxItem *pSelectedItem = m_pListBoxViewTables->selectedItem();
	if(!pSelectedItem) return;

	m_pListBoxViewTables->removeItem(m_pListBoxViewTables->currentItem());
	
	m_pPushButtonTableDel->setEnabled(false);
	
	emit sigEnableNextButton((! m_pLineEditViewName->text().isEmpty()) && (m_pListBoxViewTables->count() > 0));
	
	m_bIsDirty1 = true;
	m_bIsDirty2 = true;
	m_bIsDirty3 = true;
}

// Return part of SQL stetement for CREATE VIEW
const QString KPGCreateViewWidget1::getSQL() const
{
	QString strSQL("CREATE VIEW ");
	
	strSQL.append(m_pComboBoxSchema->currentText());
	strSQL.append(".");
	strSQL.append(m_pLineEditViewName->text());
	
	return strSQL;
}

#include "kpgcreateviewwidget1.moc"
