//
// C++ Implementation: kpgcreateviewwidget3
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgcreateviewwidget3.h"

// include files for Qt
#include <qpushbutton.h>
#include <qheader.h>

// include files for KDE
#include <kcombobox.h>
#include <klineedit.h>
#include <klistbox.h>
#include <klistview.h>
#include <kmessagebox.h>
#include <kdebug.h>

// application specific includes
#include "../DbObjects/kpgdatabase.h"
#include "../DbObjects/kpgschema.h"
#include "../DbObjects/kpgtablesfolder.h"
#include "../DbObjects/kpgtable.h"
#include "../DbObjects/kpgtablecolumnsfolder.h"

KPGCreateViewWidget3::KPGCreateViewWidget3(QWidget *parent, const char *name, KPGDatabase *pDatabase)
 : KPGCreateViewWidget3Base(parent, name)
{
	m_pDatabase = pDatabase;
  m_bCheckBoxShowSysColumns = false;
	m_eJoinType = joinInner;
	
	m_pListViewJoins->setSortColumn(-1);
}


KPGCreateViewWidget3::~KPGCreateViewWidget3()
{
}

void KPGCreateViewWidget3::slotCheckBoxShowSysColumnsToggled(bool bCheckBoxShowSysColumns)
{
	m_bCheckBoxShowSysColumns = bCheckBoxShowSysColumns;
	refreshAvailColumnsList(m_pComboBoxLeftTables->currentText(), false);
	refreshAvailColumnsList(m_pComboBoxRightTables->currentText(), true);
}

void KPGCreateViewWidget3::refreshAvailColumnsList(const QString & strTableNamespaceNameAlias, bool bRight)
{
	// divide to namespace and name, strip alias, if any
  QString strTableName(strTableNamespaceNameAlias);
  int iPos = strTableName.find(' ');
  if(iPos > 0)
    strTableName.truncate(iPos);
  
  QString strNamespace;
  iPos = strTableName.find('.');
  if(iPos > 0)
  {
    strNamespace = strTableName;
    strNamespace.truncate(iPos);
    strTableName = strTableName.mid(iPos + 1);
  }
		
	KListBox *pListBoxColumns;
	if(bRight)
		pListBoxColumns = m_pListBoxRightColumns;
	else	
		pListBoxColumns = m_pListBoxLeftColumns;
		
	pListBoxColumns->clear();
	m_pPushButtonJoinAdd->setEnabled(false);
	
  // Fill list of columns to listbox
  KPGTreeItem *pItem = m_pDatabase->getChildByName(strNamespace);
  if(!pItem)
  {
    kdDebug() << "KPGCreateViewWidget3::refreshAvailColumnsList: no item found: " << strNamespace << endl;
    return;
  }
        
  KPGSchema *pSchema = static_cast <KPGSchema *> (pItem);
  KPGTablesFolder *pTablesFolder = pSchema->getTablesFolder();
  
  // Lookup pointer to table
  pItem = pTablesFolder->getChildByName(strTableName);
  if(!pItem)
  {
    kdDebug() << "KPGCreateViewWidget3::refreshAvailColumnsList: no item found: " << strNamespace << endl;
    return;
  }
  
  KPGTable *pTable = static_cast <KPGTable *> (pItem);
  pTable->getTableColumnsFolder()->fillListBoxWithChildItems(pListBoxColumns, m_bCheckBoxShowSysColumns);
}

void KPGCreateViewWidget3::slotComboBoxLeftTableActivated(const QString & strTableName)
{
	refreshAvailColumnsList(strTableName, false);
}

void KPGCreateViewWidget3::slotComboBoxRightTableActivated(const QString & strTableName)
{
	refreshAvailColumnsList(strTableName, true);
}

void KPGCreateViewWidget3::slotViewColumnsLeftSelectChanged(QListBoxItem* pSelectedItem)
{
	m_pPushButtonJoinAdd->setEnabled((pSelectedItem != 0) && (m_pListBoxRightColumns->selectedItem() != 0));
}

void KPGCreateViewWidget3::slotViewColumnsRightSelectChanged(QListBoxItem* pSelectedItem)
{
	m_pPushButtonJoinAdd->setEnabled((pSelectedItem != 0) && (m_pListBoxLeftColumns->selectedItem() != 0));
}

void KPGCreateViewWidget3::slotRadioButtonInnerJoinClicked()
{
	m_eJoinType = joinInner;
}

void KPGCreateViewWidget3::slotRadioButtonLeftJoinClicked()
{
	m_eJoinType = joinLeft;
}

void KPGCreateViewWidget3::slotRadioButtonRigthtJoinClicked()
{
	m_eJoinType = joinRight;
}	
		
void KPGCreateViewWidget3::slotRadioButtonFullJoinClicked()
{
	m_eJoinType = joinFull;
}		

void KPGCreateViewWidget3::slotJoinsSelectionChanged(QListViewItem* pItem)
{
  m_pPushButtonJoinDel->setEnabled(pItem != 0);
}

void KPGCreateViewWidget3::slotJoinsCurrentChanged(QListViewItem* pItem)
{
  m_pPushButtonJoinDel->setEnabled(pItem != 0);
}

void KPGCreateViewWidget3::slotJoinAdd()
{
	QListBoxItem *pLeftSelectedItem = m_pListBoxLeftColumns->selectedItem();
	if(!pLeftSelectedItem) return;

	QListBoxItem *pRightSelectedItem = m_pListBoxRightColumns->selectedItem();
	if(!pRightSelectedItem) return;
	
	// Left table or alias
	QString strLeftTableName(m_pComboBoxLeftTables->currentText());
	QString strLeftTableAlias;
	
	// strip alias, if any
	int iPos = strLeftTableName.find(' ');
	if(iPos > 0)
		{
			strLeftTableAlias = strLeftTableName.mid(iPos + 1);
		  strLeftTableName.truncate(iPos);
		}
	
	// Right table or alias
	QString strRightTableName(m_pComboBoxRightTables->currentText());
	QString strRightTableAlias;
	
	// strip alias, if any
	iPos = strRightTableName.find(' ');
	if(iPos > 0)
		{
			strRightTableAlias = strRightTableName.mid(iPos + 1);
		  strRightTableName.truncate(iPos);
		}
	
	// set left column name for view
	QString strLeftColumn;
	
	if(strLeftTableAlias.length() > 0)
	{
		strLeftColumn = strLeftTableAlias;
		strLeftColumn.append(".");
		strLeftColumn.append(pLeftSelectedItem->text());
	}
	else
	{
		strLeftColumn = strLeftTableName;
		strLeftColumn.append(".");
		strLeftColumn.append(pLeftSelectedItem->text());
	}
	
	// set right column name for view
	QString strRightColumn;
	
	if(strRightTableAlias.length() > 0)
	{
		strRightColumn = strRightTableAlias;
		strRightColumn.append(".");
		strRightColumn.append(pRightSelectedItem->text());
	}
	else
	{
		strRightColumn = strRightTableName;
		strRightColumn.append(".");
		strRightColumn.append(pRightSelectedItem->text());
	}
	
	// Build SQL string
	QString strSQL;
	
	switch(m_eJoinType)
	{
		case joinInner:
			strSQL.append("INNER JOIN ");
			break;
		 
		case  joinLeft:
			strSQL.append("LEFT OUTER JOIN ");
			break; 
			
		case joinRight:
			strSQL.append("RIGHT OUTER JOIN ");
			break; 
			
		case joinFull:
			strSQL.append("FULL OUTER JOIN ");
			break;
	
	}
	
	strSQL.append(m_pComboBoxRightTables->currentText());
	strSQL.append(" ON (");
	strSQL.append(strLeftColumn);
	strSQL.append(" = ");
	strSQL.append(strRightColumn);
	strSQL.append(")");
	
	// Add JOIN to list
	QListViewItem *pLastItem = 0;
	
	QListViewItem * pItem = m_pListViewJoins->firstChild();
	while(pItem)
	{
		pLastItem = pItem;
		pItem = pItem->nextSibling();
	}
	
	QListViewItem *pNewItem;
		
	if(pLastItem == 0) 
		pNewItem = new QListViewItem(m_pListViewJoins, strSQL);
	else
		pNewItem = new QListViewItem(m_pListViewJoins, pLastItem, strSQL);
		
	// Prevent left table from changing
	m_pComboBoxLeftTables->setEnabled(false);
	
}

void KPGCreateViewWidget3::slotJoinDelete()
{
	QListViewItem *pSelectedItem = m_pListViewJoins->selectedItem();
	if(pSelectedItem)
			delete pSelectedItem;
	
	m_pPushButtonJoinDel->setEnabled(false);
	
	m_pComboBoxLeftTables->setEnabled(m_pListViewJoins->childCount() == 0);
}
		
// Return part of SQL statement for CREATE VIEW
const QString KPGCreateViewWidget3::getSQL() const
{
	QString strSQL(" ");
	
	QListViewItem * pItem = m_pListViewJoins->firstChild();
	if(pItem)
	{
		strSQL.append(m_pComboBoxLeftTables->currentText());
	
		while(pItem)
		{
			strSQL.append("\n");
			strSQL.append(pItem->text(0));
							
			pItem = pItem->nextSibling();
		}
	}
	else
	{
		for(int i = 0; i < m_pComboBoxLeftTables->count(); i++)
		{
			if(i > 0)
				strSQL.append(", ");
				
			strSQL.append(m_pComboBoxLeftTables->text(i));
		}
	}
	
	
	return strSQL;
}


#include "kpgcreateviewwidget3.moc"
