/***************************************************************************
                          kpgsqlquerresultwindow.h  -  description
                             -------------------
    begin                : So led 17 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPGSQLQUERRESULTWINDOW_H
#define KPGSQLQUERRESULTWINDOW_H

#include "kpgsqlqueryresultwindowbase.h"

class KTextEdit;
class KPGSyntaxHighlighter;

/**Window that allow edit SQL query and display result
  *@author Lumir Vanek
  */

class KPGSqlQueryResultWindow : public KPGSqlQueryResultWindowBase
{
   Q_OBJECT
public: 
	KPGSqlQueryResultWindow(QWidget *parent=0, const char *name=0);
	~KPGSqlQueryResultWindow();

  KTextEdit * editQuery() const { return m_pTextEditQuery; }
  QTable *tableResult() const { return m_pTableResult; }

	// Is SQL query changed after last open/save ?
	bool queryTextChanged() const { return m_bQueryTextChanged; }
	
	// Signalize, if result contain at least one row
	bool resultEmpty() const;
				
protected slots:
    
		// Changes syntax highlighting colors.
		void slotSqlEditorSettingsChanged();
  
		void slotServerChanged(const QString &);

		/** Called, when user change text in SQL input area. Mark document as modyfied. */
   	void slotTextEditQueryTextChanged();
	
		/** Called, when user copu action availability is changed in SQL input area */
		void slotTextEditCopyAvailable(bool);

		/** Called, when undo action availability is changed in SQL input area */
		void slotTextEditUndoAvailable(bool);

		/** Called, when redo action availability is changed in SQL input area */
		void slotTextEditRedoAvailable(bool);

signals:

		// Fired, when info abour databases list is needed
		void sigRequestDatabasesList(KPGSqlQueryResultWindow *, const QString&);
		
		// Fired when availability of text for cut/copy is changed
		void sigCopyAvailable(KPGSqlQueryResultWindow *, bool);

		// Fired when availability of undo is changed
		void sigUndoAvailable(KPGSqlQueryResultWindow *, bool);
		
		// Fired when availability of redo is changed
		void sigRedoAvailable(KPGSqlQueryResultWindow *, bool);
				
		// Fired when user change text in query edit widget
		void sigQueryTextChanged(KPGSqlQueryResultWindow *);
		
protected:

  /** Syntax highlighter for m_pMultiLineEditQuery */
  KPGSyntaxHighlighter *m_pSyntaxHighlighter;

  bool m_bQueryTextChanged;  
};

#endif
