/***************************************************************************
                          kpgsyntaxhighlighter.cpp  -  description
                             -------------------
    begin                : � led 13 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kpgsyntaxhighlighter.h"

#include <iostream>

#include <qcolor.h>
#include <qregexp.h>
#include <qstring.h>
#include <qstringlist.h>

#include <ktextedit.h>

// SQL language keywords list
#define NUM_KEYWORDS 494
const char *arStrKeyWords[NUM_KEYWORDS] = {
"ABORT",
"ABS",
"ABSOLUTE",
"ACCESS",
"ACTION",
"ADA",
"ADD",
"ADMIN",
"AFTER",
"AGGREGATE",
"ALIAS",
"ALL",
"ALLOCATE",
"ALSO",
"ALTER",
"ANALYSE",
"ANALYZE",
"ANY",
"ARE",
"ARRAY",
"AS",
"ASC",
"ASENSITIVE",
"ASSERTION",
"ASSIGNMENT",
"ASYMMETRIC",
"AT",
"ATOMIC",
"AUTHORIZATION",
"AVG",
"BACKWARD",
"BEFORE",
"BEGIN",
"BOTH",
"BREADTH",
"BY",
"C",
"CACHE",
"CALL",
"CALLED",
"CARDINALITY",
"CASCADE",
"CASCADED",
"CASE",
"CAST",
"CATALOG",
"CATALOG_NAME",
"CHAIN",
"CHARACTERISTICS",
"CHECK",
"CHECKED",
"CHECKPOINT",
"CLASS",
"CLASS_ORIGIN",
"CLOSE",
"CLUSTER",
"COALESCE",
"COBOL",
"COLLATE",
"COLLATION",
"COLLATION_CATALOG",
"COLLATION_NAME",
"COLLATION_SCHEMA",
"COLUMN",
"COLUMN_NAME",
"COMMAND_FUNCTION",
"COMMAND_FUNCTION_CODE",
"COMMENT",
"COMMIT",
"COMMITTED",
"COMPLETION",
"CONDITION_NUMBER",
"CONNECT",
"CONNECTION",
"CONNECTION_NAME",
"CONSTRAINT",
"CONSTRAINTS",
"CONSTRAINT_CATALOG",
"CONSTRAINT_NAME",
"CONSTRAINT_SCHEMA",
"CONSTRUCTOR",
"CONTAINS",
"CONTINUE",
"CONVERSION",
"CONVERT",
"COPY",
"CORRESPONDING",
"COUNT",
"CREATE",
"CREATEDB",
"CREATEUSER",
"CROSS",
"CUBE",
"CURRENT",
"CURRENT_DATE",
"CURRENT_PATH",
"CURRENT_ROLE",
"CURRENT_TIME",
"CURRENT_TIMESTAMP",
"CURRENT_USER",
"CURSOR",
"CURSOR_NAME",
"CYCLE",
"DATA",
"DATABASE",
"DATETIME_INTERVAL_CODE",
"DATETIME_INTERVAL_PRECISION",
"DAY",
"DEALLOCATE",
"DECLARE",
"DEFAULT",
"DEFAULTS",
"DEFERRABLE",
"DEFERRED",
"DEFINED",
"DEFINER",
"DELETE",
"DELIMITER",
"DELIMITERS",
"DEPTH",
"DEREF",
"DESC",
"DESCRIBE",
"DESCRIPTOR",
"DESTROY",
"DESTRUCTOR",
"DETERMINISTIC",
"DIAGNOSTICS",
"DICTIONARY",
"DISCONNECT",
"DISPATCH",
"DISTINCT",
"DO",
"DOMAIN",
"DROP",
"DYNAMIC",
"DYNAMIC_FUNCTION",
"DYNAMIC_FUNCTION_CODE",
"EACH",
"ELSE",
"ENCODING",
"ENCRYPTED",
"EXIT",
"END",
"END-EXEC",
"EQUALS",
"ESCAPE",
"EVERY",
"EXCEPT",
"EXCEPTION",
"EXCLUDING",
"EXCLUSIVE",
"EXEC",
"EXECUTE",
"EXISTING",
"EXISTS",
"EXPLAIN",
"EXTERNAL",
"EXTRACT",
"FALSE",
"FETCH",
"FINAL",
"FIRST",
"FOR",
"FORCE",
"FOREIGN",
"FORTRAN",
"FORWARD",
"FOUND",
"FREE",
"FREEZE",
"FROM",
"FULL",
"FUNCTION",
"G",
"GENERAL",
"GENERATED",
"GET",
"GLOBAL",
"GO",
"GOTO",
"GRANT",
"GRANTED",
"GROUP",
"GROUPING",
"HANDLER,",
"HAVING",
"HIERARCHY",
"HOLD",
"HOST",
"HOUR",
"IDENTITY",
"IGNORE",
"IMMEDIATE",
"IMMUTABLE",
"IMPLEMENTATION",
"IMPLICIT",
"INCLUDING",
"INCREMENT",
"INDEX",
"INDICATOR",
"INFIX",
"INHERITS",
"INITIALIZE",
"INITIALLY",
"INNER",
"INOUT",
"INPUT",
"INSENSITIVE",
"INSERT",
"INSTANCE",
"INSTANTIABLE",
"INSTEAD",
"INTERSECT",
"INTERVAL",
"INTO",
"INVOKER",
"IF",
"IS",
"ISNULL",
"ISOLATION",
"ITERATE",
"JOIN",
"K",
"KEY",
"KEY_MEMBER",
"KEY_TYPE",
"LANCOMPILER",
"LANGUAGE",
"LARGE",
"LAST",
"LATERAL",
"LEADING",
"LEFT",
"LENGTH",
"LESS",
"LEVEL",
"LIMIT",
"LISTEN",
"LOAD",
"LOCAL",
"LOCALTIME",
"LOCALTIMESTAMP",
"LOCATION",
"LOCATOR",
"LOCK",
"LOOP",
"LOWER",
"M",
"MAP",
"MATCH",
"MAX",
"MAXVALUE",
"MESSAGE_LENGTH",
"MESSAGE_OCTET_LENGTH",
"MESSAGE_TEXT",
"METHOD",
"MIN",
"MINUTE",
"MINVALUE",
"MOD",
"MODE",
"MODIFIES",
"MODIFY",
"MODULE",
"MONTH",
"MORE",
"MOVE",
"MUMPS",
"NAME",
"NAMES",
"NATIONAL",
"NATURAL",
"NEW",
"NEXT",
"NO",
"NOCREATEDB",
"NOCREATEUSER",
"NONE",
"NOTHING",
"NOTIFY",
"NOTNULL",
"NULL",
"NULLABLE",
"NULLIF",
"NUMBER",
"OBJECT",
"OCTET_LENGTH",
"OF",
"OFF",
"OFFSET",
"OIDS",
"OLD",
"ON",
"ONLY",
"OPEN",
"OPERATION",
"OPERATOR",
"OPTION",
"OPTIONS",
"ORDER",
"ORDINALITY",
"OUT",
"OUTER",
"OUTPUT",
"OVERLAPS",
"OVERLAY",
"OVERRIDING",
"OWNER",
"PAD",
"PARAMETER",
"PARAMETERS",
"PARAMETER_MODE",
"PARAMETER_NAME",
"PARAMETER_ORDINAL_POSITION",
"PARAMETER_SPECIFIC_CATALOG",
"PARAMETER_SPECIFIC_NAME",
"PARAMETER_SPECIFIC_SCHEMA",
"PARTIAL",
"PASCAL",
"PASSWORD",
"PENDANT",
"PLACING",
"PLI",
"POSITION",
"POSTFIX",
"PRECISION",
"PREFIX",
"PREORDER",
"PREPARE",
"PRESERVE",
"PRIMARY",
"PRIOR",
"PRIVILEGES",
"PROCEDURAL",
"PROCEDURE",
"PUBLIC",
"READ",
"READS",
"RECHECK",
"RECURSIVE",
"REF",
"REFERENCES",
"REFERENCING",
"REINDEX",
"RELATIVE",
"RENAME",
"REPEATABLE",
"REPLACE",
"RESET",
"RESTART",
"RESTRICT",
"RESULT",
"RETURN",
"RETURNED_LENGTH",
"RETURNED_OCTET_LENGTH",
"RETURNED_SQLSTATE",
"RETURNS",
"REVOKE",
"RIGHT",
"ROLE",
"ROLLBACK",
"ROLLUP",
"ROUTINE",
"ROUTINE_CATALOG",
"ROUTINE_NAME",
"ROUTINE_SCHEMA",
"ROW",
"ROWS",
"ROW_COUNT",
"RULE",
"SAVEPOINT",
"SCALE",
"SCHEMA",
"SCHEMA_NAME",
"SCOPE",
"SCROLL",
"SEARCH",
"SECOND",
"SECTION",
"SECURITY",
"SELECT",
"SELF",
"SENSITIVE",
"SEQUENCE",
"SERIALIZABLE",
"SERVER_NAME",
"SESSION",
"SESSION_USER",
"SET",
"SETOF",
"SETS",
"SHARE",
"SHOW",
"SIMILAR",
"SIMPLE",
"SIZE",
"SOME",
"SOURCE",
"SPACE",
"SPECIFIC",
"SPECIFICTYPE",
"SPECIFIC_NAME",
"SQL",
"SQLCODE",
"SQLERROR",
"SQLEXCEPTION",
"SQLSTATE",
"SQLWARNING",
"STABLE",
"START",
"STATE",
"STATEMENT",
"STATIC",
"STATISTICS",
"STDIN",
"STDOUT",
"STORAGE",
"STRICT",
"STRUCTURE",
"STYLE",
"SUBCLASS_ORIGIN",
"SUBLIST",
"SUBSTRING",
"SUM",
"SYMMETRIC",
"SYSID",
"SYSTEM",
"SYSTEM_USER",
"TABLE",
"TABLE_NAME",
"TEMP",
"TEMPLATE",
"TEMPORARY",
"TERMINATE",
"THAN",
"THEN",
"TO",
"TOAST",
"TRAILING",
"TRANSACTION",
"TRANSACTIONS_COMMITTED",
"TRANSACTIONS_ROLLED_BACK",
"TRANSACTION_ACTIVE",
"TRANSFORM",
"TRANSFORMS",
"TRANSLATE",
"TRANSLATION",
"TREAT",
"TRIGGER",
"TRIGGER_CATALOG",
"TRIGGER_NAME",
"TRIGGER_SCHEMA",
"TRIM",
"TRUE",
"TRUNCATE",
"TRUSTED",
"TYPE",
"UNCOMMITTED",
"UNDER",
"UNENCRYPTED",
"UNION",
"UNIQUE",
"UNKNOWN",
"UNLISTEN",
"UNNAMED",
"UNNEST",
"UNTIL",
"UPDATE",
"UPPER",
"USAGE",
"USER",
"USER_DEFINED_TYPE_CATALOG",
"USER_DEFINED_TYPE_NAME",
"USER_DEFINED_TYPE_SCHEMA",
"USING",
"VACUUM",
"VALID",
"VALIDATOR",
"VALUE",
"VALUES",
"VARIABLE",
"VERBOSE",
"VERSION",
"VIEW",
"VOLATILE",
"WHEN",
"WHENEVER",
"WHERE",
"WITH",
"WITHOUT",
"WORK",
"WRITE",
"YEAR"
};

// SQL language datatypes list
#define NUM_DATATYPES 43
const char *arStrDataTypes[NUM_DATATYPES] = {
"boolean",
"bool",
"bit",
"varying",
"character",
"char",
"varchar",
"text",
"smallint",
"int2",
"integer",
"int",
"int4",
"bigint",
"int8",
"real",
"float4",
"double",
"precision",
"float8",
"float",
"numeric",
"decimal",
"money",
"serial",
"date",
"time",
"with",
"zone",
"timestamp",
"interval",
"box",
"line",
"lseg",
"circle",
"path",
"point",
"polygon",
"cidr",
"inet",
"macadr",
"oid",
"xid" };

// SQL language operators list
#define NUM_OPERATORS 35
const char *arStrOperators[NUM_OPERATORS] = {
"+",
"-",
"/",
"*",
"%",
"^",
"|/",
"||/",
"!",
"!!",
"@",
"<",
">",
"<=",
">=",
"~",
"!~",
"~*",
"!~*",
"AND",
"OR",
"NOT",
"=",
"!=",
"<>",
"&",
"|",
"#",
"<<",
">>",
"BETWEEN",
"IN",
"LIKE",
"ILIKE",
"OVERLAPS" };

/*
Functions
nextval
curval

*/

KPGSyntaxHighlighter::KPGSyntaxHighlighter(QTextEdit *textEdit)
  : QSyntaxHighlighter(textEdit)
{
  m_clrDefaultText.setRgb(0, 0, 0);
  m_clrKeyWord.setRgb(128, 0, 0);
  m_clrDataType.setRgb(0, 255, 255);
  m_clrOperator.setRgb(0, 255, 0);
  m_clrQuotedString.setRgb(0, 0, 128);
  m_clrNumber.setRgb(255, 255, 0);
  m_clrComment.setRgb(128, 128, 128);
}

KPGSyntaxHighlighter::~KPGSyntaxHighlighter()
{
}


int KPGSyntaxHighlighter::highlightParagraph(const QString& text, int endStateOfLastPara)
{
    //first I format the given line to default so any remaining highlighting is removed (Qt does not do it by itself)
    setFormat(0 , text.length(), QColor(0, 0, 0));

    int pos;
		unsigned int i = 0;
	
		if(endStateOfLastPara == 1)
		{
			for(i = 0; i < text.length(); i++)
    	{
				QRegExp patternComment("[^*]*[^/]*\\*/"); // search end of comment
        pos=patternComment.search(text, i);

        if(pos >= 0) // end comment found ?
        {
          int l = patternComment.matchedLength();
              
          setFormat(0, l, m_clrComment);
          i += l; // skip comment
					break;
        }
				else
				{
					setFormat(0, text.length(), m_clrComment);
					return 1; // return 1 to signify "in comment"
				}
			}
		}
		
		
    for(; i < text.length(); i++)
    {
      if((text[i] == '-') && (i+1 < text.length()) && (text[i+1] == '-'))
      {
        // simple comment found
        setFormat(i, text.length(), m_clrComment );
        break; // skip rest of line
      }
			
			if((text[i] == '/') && (i+1 < text.length()) && (text[i+1] == '*'))
			{
				QRegExp patternComment("/\\*[^*]*[^/]*\\*/"); // search comment
        pos=patternComment.search(text, i);

        if((unsigned int) pos == i) // closed comment found ?
        {
          int l = patternComment.matchedLength();
              
          setFormat(pos, l, m_clrComment);
          i += l; // skip comment
        }
				else
				{
					setFormat(i, text.length() - i, m_clrComment);
					return 1; // return 1 to signify "in comment"
				}
			}
      
      if(text[i] != ' ')
      {
        // try find keyword
        for(int j = 0; j < NUM_KEYWORDS; j++)
        {
          if(QChar(text[i]).upper() != QChar(arStrKeyWords[j][0]))
            continue; // test first character for speed up  
            
          if(QChar(text[i]).upper() > QChar(arStrKeyWords[j][0]))
            break; // test first character for speed up
          
          // For spped-up first try dumb test    
          // first letter match, test whole keyword
          bool bFound = true;
          int p = 0;
          for(unsigned int o = i; o < text.length(); o++)
          {
            if(((QChar(text[o]).upper() >= 'A') && (QChar(text[o]).upper() <= 'Z')) || (QChar(text[o]).upper() == '_'))
            {
              if(QChar(text[o]).upper() != QChar(arStrKeyWords[j][p++]))
              {
                bFound = false;
                break;
              }
            }
            else
              break;
          }

          if(bFound) // If dumb test success, try smart but slow test
          {
            QString strPattern("\\b");
	          strPattern.append(arStrKeyWords[j]);
            strPattern.append("\\b");
            QRegExp patternKeyWord(strPattern, false); // search keyword
            pos=patternKeyWord.search(text, i);
            if((unsigned int) pos == i)
            {
              int l = patternKeyWord.matchedLength();
              setFormat(i, l, m_clrKeyWord );
              i += l;
              break;
            }
          }
        }

        // try find datatype
        for(int j = 0; j < NUM_DATATYPES; j++)
        {
          if(QChar(text[i]).lower() != QChar(arStrDataTypes[j][0]).lower())
            continue; // test first character for speed up

	  			QString strPattern("\\b");  
          strPattern.append(arStrDataTypes[j]);
          strPattern.append("\\b");
          QRegExp patternDataType(strPattern, false); // search datatype
          pos=patternDataType.search(text, i);
          if((unsigned int) pos == i)
          {
            int l = patternDataType.matchedLength();
            setFormat(i, l, m_clrDataType );
            i += l;
            break;
          }
        }

        // try find operators
        for(int j = 0; j < NUM_OPERATORS; j++)
        {
          if(QChar(text[i]).lower() != QChar(arStrOperators[j][0]).lower())
            continue; // test first character for speed up

					QString strPattern("\\b");  
          strPattern.append(arStrOperators[j]);
          strPattern.append("\\b");
          QRegExp patternOperators(strPattern, false); // search datatype
          pos=patternOperators.search(text, i);
          if((unsigned int) pos == i)
          {
            int l = patternOperators.matchedLength();
            if(l > 0)
            {
              setFormat(i, l, m_clrOperator );
              i += l;
            }
            break;
          }
        }

        // try find quoted string
        if((text[i] == '\'') || (text[i] == '\"'))
        {
          QRegExp patternQuotedString("\"[^<\"]*\"|'[^<']*'"); // search quoted string
          pos=patternQuotedString.search(text, i);

          if((unsigned int) pos == i) 
            {
              int l = patternQuotedString.matchedLength();

              setFormat(i+1, l - 2, m_clrQuotedString);
              
              i += l;
            }
        }

        // try find numbers
       if((text[i] >= '0') && (text[i] <= '9'))
        {
          
					QString strPattern("\\b\\d+\\b");
	  		
					QRegExp patternNumber(strPattern, false); // search numbers
        	pos=patternNumber.search(text, i);
        	if((unsigned int) pos == i)
        	{
          	int l = patternNumber.matchedLength();
          	setFormat(i, l, m_clrNumber );
          	i += l;
        	}
				} 
      }
    }

    return 0;
}

