/***************************************************************************
                          kpogre.h  -  description
                             -------------------
    begin                : Fre Sep  7 16:06:25 CEST 2001
    copyright            : (C) 2002 by Dominik Sacher
    email                : dsacher@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPOGRE_H
#define KPOGRE_H
 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// include files for Qt

// include files for KDE 
#include <kapp.h>
#include <kmainwindow.h>
#include <kaccel.h>
#include <kaction.h>
#include <kurl.h>

#include "DbObjects/kpgtreeitem.h"
#include "kpogreview.h"

// forward declaration of the KPoGre classes
class KPoGreDoc;
class KPGConfiguration;
class KPGBookmarksButton;
class KPGSearchDialog;
class KPoGreComboAction;

/**
  * The base class for KPoGre application windows. It sets up the main
  * window and reads the config file as well as providing a menubar, toolbar
  * and statusbar. An instance of KPoGreView creates your center view, which is connected
  * to the window's Doc object.
  * KPoGreApp reimplements the methods that KMainWindow provides for main window handling and supports
  * full session management as well as using KActions.
  * @see KMainWindow
  * @see KApplication
  * @see KConfig
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
  * @version KDevelop version 1.2 code generation
  */
class KPoGreApp : public KMainWindow
{
  Q_OBJECT

  friend class KPoGreView;

  public:
    /** construtor of KPoGreApp, calls all init functions to create the application.
     */
    KPoGreApp(QWidget* parent=0, const char* name=0);
    ~KPoGreApp();
    /** opens a file specified by commandline option
     */
    void openDocumentFile(const KURL& url=0);
    /** returns a pointer to the current document connected to the KTMainWindow instance and is used by
     * the View class to access the document object's methods
     */	
    KPoGreDoc *getDocument() const;

    static KPGConfiguration * configuration();

  protected:
    /** save general Options like all bar positions and status as well as the geometry and the recent file list to the configuration
     * file
     */ 	
    void saveOptions();
    /** read general Options again and initialize all variables like the recent file list
     */
    void readOptions();
    /** initializes the KActions of the application */
		
    void initActions();
    /** sets up the statusbar for the main window by initialzing a statuslabel.
     */
    void initStatusBar();
    /** initializes the document object of the main window that is connected to the view in initView().
     * @see initView();
     */
    void initDocument();
		
    /** creates the centerwidget of the KTMainWindow instance and sets it as the view
     */
    void initView();
    /** queryClose is called by KTMainWindow on each closeEvent of a window. Against the
     * default implementation (only returns true), this calles saveModified() on the document object to ask if the document shall
     * be saved if Modified; on cancel the closeEvent is rejected.
     * @see KTMainWindow#queryClose
     * @see KTMainWindow#closeEvent
     */
    virtual bool queryClose();
    /** queryExit is called by KTMainWindow when the last window of the application is going to be closed during the closeEvent().
     * Against the default implementation that just returns true, this calls saveOptions() to save the settings of the last window's	
     * properties.
     * @see KTMainWindow#queryExit
     * @see KTMainWindow#closeEvent
     */
    virtual bool queryExit();
    /** saves the window properties for each open window during session end to the session config file, including saving the currently
     * opened file by a temporary filename provided by KApplication.
     * @see KTMainWindow#saveProperties
     */
    //virtual void saveProperties(KConfig *_cfg);
    /** reads the session config file and restores the application's state including the last opened files and documents by reading the
     * temporary files saved by saveProperties()
     * @see KTMainWindow#readProperties
     */
    //virtual void readProperties(KConfig *_cfg);

  public slots:
    /** open a new application window by creating a new instance of KPoGreApp */
    void slotFileNewWindow();
		
    /** clears the document in the actual view to reuse it as the new document */
    void slotFileNew();
		
    /** open a file and load it into the document*/
    void slotFileOpen();
		
    /** opens a file from the recent files menu */
    void slotFileOpenRecent(const KURL& url);
		
    /** save a document */
    void slotFileSave();
		
    /** save a document by a new filename*/
    void slotFileSaveAs();
		
    /** asks for saving if the file is modified, then closes the actual file and window*/
    void slotFileClose();
		
    /** print the actual file */
    void slotFilePrint();
		
    /** closes all open windows by calling close() on each memberList item until the list is empty, then quits the application.
     * If queryClose() returns false because the user canceled the saveModified() dialog, the closing breaks.
     */
    void slotFileQuit();
    
		 // Undo last edit action in SQL query edit-widget
		 void slotEditUndo();
		 
		 // Redo last edit action in SQL query edit-widget
		 void slotEditRedo();
				
		/** put the marked text/object into the clipboard and remove
     *	it from the document
     */
    void slotEditCut();
		
    /** put the marked text/object into the clipboard
     */
    void slotEditCopy();
		
    /** paste the clipboard into the document
     */
    void slotEditPaste();
    /** toggles the toolbar
     */
		 
		/** Displays search dialog and finds string in this tree view */
		void slotEditFind();

    /** Finds next occurence of string in this tree view */
		void slotEditFindNext();

		/** Deselects currently selected item */
		void slotEditDeselect();
		
		void slotViewToolBar();
		
    /** toggles the statusbar
     */
    void slotViewStatusBar();
    /** changes the statusbar contents for the standard label permanently, used to indicate current actions.
     * @param text the text that is displayed in the statusbar
     */
    void slotStatusMsg(const QString &text);

    /** Connect to Database Server */
		void slotDatabaseConnect();
	
		/** Disconnect Database */
		void slotDatabaseDisconnect();
		
		// Send query to DB
    void slotDatabaseRunQuery();

		// Run Wizard
		void slotDatabaseWizard();
		
		/** Clear Input Area */
		void slotEditClearInputArea();
		
		/** Dialog for keys binding configuration */
		void slotConfigureKeys();
	
		/** Configure toolbars dialog */
		void slotConfigToolbars();
	
		/** Configure application */
		void slotConfigure();
		
		/** Save query result to file */
		void slotExportResult();
	
		/** Called, when document is modified. Enable Save button. */
		void slotDocumentModified();
	
		// Called, when user change selection in tree view
		void slotSelectionChanged( const KPGTreeItem::eNodeType, KPGTreeItem *, unsigned int, unsigned int);
	
		// Called, when user change SQL input area text
		void slotEnableRunQueryAction(bool);
		
		// Enables/disables File Open, Save and Export action
		void slotEnableFileActions(bool, bool, bool);
			
		// Enables/disables Edit actions
		void slotEnableEditActions(bool, bool, bool, bool, bool, bool);
		
		// Called when user press Bookmarks Button
		void slotBookmarksButtonPressed();
		
		// Called when user choose bookmark
		void slotOpenUrl( const QString& );
		
		// Display popup menu
		void slotContextMenuRequested(const QString &, const QPoint &);
			
		// Refresh selected item
		void slotRefresh();
		
		void slotPathSelected( const QString & szPath );
			
		void slotPathClear();
		
		/** @short Selects the selected item's parent item. */
		void slotViewNodeUp() { view->viewNodeUp(); }
		
		/** @short Selects the previously selected item. */
		void slotViewNodeBack() { view->viewNodeBack(); }
		
		/** @short Selects the next selected item. */
		void slotViewNodeForward() { view->viewNodeForward(); }
		
		//--- Database related menu actions
		void slotCreateDatabase();
		void slotDropDatabase();
    void slotAnalyseDatabase();
    void slotReindexDatabase();
    void slotSetOwnerDatabase();
    void slotDisconnectDatabase();
		
    //--- Schema related menu actions
		void slotCreateSchema();
	  void slotDropSchema();
		void slotModifySchema();
    void slotSetOwnerSchema();
		
    //--- Table related menu actions
    void slotCreateTable();
	  void slotDropTable();
    void slotRenameTable();
    void slotAnalyseTable();
    void slotVacuumTable();
    void slotReindexTable();
    void slotSetClusterOnTable();
    void slotSetWithoutClusterTable();
    void slotClusterTable();
    void slotCopyFromTable();
    void slotCopyToTable();
    void slotSetOwnerTable();
    void slotSetTablespaceTable();
    void slotSetWithoutOIDsTable();
    
		//--- View related menu actions
    void slotCreateView();
		void slotDropView();
		void slotModifyView();
		    
    //--- Tablespaces related menu actions
    void slotCreateTablespace();
    void slotDropTablespace();
    void slotRenameTablespace();
    void slotSetOwnerTablespace();
    
    //--- User related menu actions
    void slotCreateUser();
		void slotDropUser();
		void slotModifyUser();
				
		//--- Group related menu actions
		void slotCreateGroup();
		void slotDropGroup();
		void slotModifyGroup();
        
    //--- Language related menu actions
    void slotCreateLanguage();
    void slotDropLanguage();
    void slotModifyLanguage();
    
    //--- Domain related menu actions 
    void slotCreateDomain();
    void slotDropDomain();
    void slotSetDomainNull();
    void slotSetDomainDefault();
    void slotDropDomainDefault();
    void slotSetOwnerDomain();
    
    //--- Domain constraint related menu actions
    void slotCreateDomainConstraint();
    void slotDropDomainConstraint();
    
    //--- Function related menu actions
    void slotCreateFunction();
    void slotDropFunction();
    void slotRenameFunction();
    void slotModifyFunction();
    
    //--- Sequence related menu actions
    void slotCreateSequence();
    void slotDropSequence();
    void slotModifySequence();
        
    //--- Column related menu actions
    void slotCreateColumn();
    void slotDropColumn();
    void slotRenameColumn();
    void slotSetColumnNull();
    void slotSetColumnDefault();
    void slotDropColumnDefault();
    void slotChangeColumnType();
    void slotSetColumnStorage();
    void slotSetColumnStatistisc();
    
    //--- Constraint related menu actions
    void slotCreateConstraint();
    void slotDropConstraint();
    
    //--- Index related menu actions
    void slotCreateIndex();
    void slotDropIndex();
    void slotRenameIndex();
    void slotReindexIndex();
    void slotSetOwnerIndex();
    void slotSetTablespaceIndex();
        
    //--- Rule related menu actions
    void slotCreateRule();
    void slotDropRule();
    
    //--- Trigger related menu actions
    void slotCreateTrigger();
    void slotDropTrigger();
    void slotRenameTrigger();
    
    //--- Type related menu actions
    void slotCreateBaseType();
    void slotCreateCompositeType();
    void slotDropType();
    void slotSetOwnerType();
    
private:
    /** the configuration object of the application */
    KConfig *config;
    /** view is the main widget which represents your working area. The View
     * class should handle all events of the view widget.  It is kept empty so
     * you can create your view according to your application's needs by
     * changing the view class.
     */
    KPoGreView *view;
    /** doc represents your actual document and is created only once. It keeps
     * information such as filename and does the serialization of your files.
     */
    KPoGreDoc *doc;

    // KAction pointers to enable/disable actions
    
    KAction *m_pActFileExportResult;
    KAction* m_pActFileNewWindow;
    KAction* m_pActFileNew;
    KAction* m_pActFileOpen;
    KRecentFilesAction* m_pActFileOpenRecent;
    KAction* m_pActFileSave;
    KAction* m_pActFileSaveAs;
    KAction* m_pActFileClose;
    KAction* m_pActFilePrint;
    KAction* m_pActFileQuit;
    
		KAction* m_pActEditUndo;
		KAction* m_pActEditRedo;
		KAction* m_pActEditCut;
    KAction* m_pActEditCopy;
    KAction* m_pActEditPaste;
		KAction* m_pActEditFind;
		KAction* m_pActEditFindNext;
		KAction* m_pActEditClearInputArea;
		KAction* m_pActEditDeselect;
		
		
		KAction * m_pActViewNodeUp;
		KAction * m_pActViewNodeBack;
		KAction * m_pActViewNodeForward;
    KToggleAction* m_pActViewToolBar;
    KToggleAction* m_pActViewStatusBar;
    
    KAction* m_pActServerRegister;
    KAction* m_pActDatabaseConnect;
    KAction* m_pActDatabaseDisconnect;
    KAction* m_pActDatabaseRunQuery;
		KAction* m_pActDatabaseWizard;
		
		KAction* m_pActFolderRefresh;

		KPoGreComboAction * m_pActPathCombo;
		
    /** our configuration - the same for all parts */
		static KPGConfiguration * s_pKPGConfig;

    KPGBookmarksButton *m_pBookmarksButton;
    
		/** the search dialog, used for find action */
		KPGSearchDialog * m_pDlgSearch; 
};
 
#endif // KPOGRE_H
