/***************************************************************************
                          kpogredoc.cpp  -  description
                             -------------------
    begin                : � �n  6 16:08:39 CEST 2002
    copyright            : (C) 2002 by Dominik Sacher
    email                : dsacher@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qdir.h>
#include <qwidget.h>

// include files for KDE
#include <klocale.h>
#include <kmessagebox.h>
#include <kio/job.h>
#include <kio/netaccess.h>

// application specific includes
#include "kpogredoc.h"
#include "kpogre.h"
#include "kpogreview.h"

QPtrList<KPoGreView> *KPoGreDoc::pViewList = 0L;

KPoGreDoc::KPoGreDoc(QWidget *parent, const char *name) : QObject(parent, name)
{
  if(!pViewList)
  {
    pViewList = new QList<KPoGreView>();
  }

  pViewList->setAutoDelete(true);
}

KPoGreDoc::~KPoGreDoc()
{
}

void KPoGreDoc::addView(KPoGreView *view)
{
  pViewList->append(view);
}

void KPoGreDoc::removeView(KPoGreView *view)
{
  pViewList->remove(view);
}
void KPoGreDoc::setURL(const KURL &url)
{
  doc_url=url;
}

const KURL& KPoGreDoc::URL() const
{
  return doc_url;
}

void KPoGreDoc::slotUpdateAllViews(KPoGreView *sender)
{
  KPoGreView *w;
  if(pViewList)
  {
    for(w=pViewList->first(); w!=0; w=pViewList->next())
    {
      if(w!=sender)
        w->repaint();
    }
  }

}

bool KPoGreDoc::saveModified()
{
  bool completed=true;

  if(modified)
  {
    KPoGreApp *win=(KPoGreApp *) parent();
    int want_save = KMessageBox::warningYesNoCancel(win,
                                         i18n("The current file has been modified.\n"
                                              "Do you want to save it?"),
                                         i18n("Warning"));
    switch(want_save)
    {
      case KMessageBox::Yes:
           if (doc_url.fileName() == i18n("Untitled"))
           {
             win->slotFileSaveAs();
           }
           else
           {
             saveDocument(URL());
       	   };

       	   deleteContents();
           completed=true;
           break;

      case KMessageBox::No:
           setModified(false);
           deleteContents();
           completed=true;
           break;

      case KMessageBox::Cancel:
           completed=false;
           break;

      default:
           completed=false;
           break;
    }
  }

  return completed;
}

void KPoGreDoc::closeDocument()
{
  deleteContents();
}

bool KPoGreDoc::newDocument()
{
  /////////////////////////////////////////////////
  // TODO: Add your document initialization code here
  /////////////////////////////////////////////////
  modified=false;
  doc_url.setFileName(i18n("Untitled"));

  return true;
}

bool KPoGreDoc::openDocument(const KURL& url, const char */*format*/ /*=0*/)
{
  QString tmpfile;
  KIO::NetAccess::download( url, tmpfile, pViewList->first());

  QFile sqlfile(tmpfile);
  if( !sqlfile.open(IO_ReadOnly) )
    { QString error =  i18n("Error opening ");
      error.append(tmpfile);
	
      KMessageBox::error(0, error, i18n("Error"));
     	KIO::NetAccess::removeTempFile( tmpfile );
     	return false;
    }
  else
   { QTextStream inStream( &sqlfile );
     while( !inStream.atEnd() )
       { pViewList->first()->setSQLText( inStream.read() );
   	   }
      sqlfile.close();
   }

  KIO::NetAccess::removeTempFile( tmpfile );
  modified=false;
  return true;
}

bool KPoGreDoc::saveDocument(const KURL& url, const char */*format*/ /*=0*/)
{
  if(url.path().isEmpty())
		return true;

	QFile file(url.path());
	if(file.open(IO_WriteOnly) == true)
		{ QString strSQL = pViewList->first()->getSQLText();

 			file.writeBlock(strSQL, strSQL.length());
			file.flush();
			file.close();
		}
	else
		{ KMessageBox::error(0,
												 i18n("Can't create file %1").arg(url.path()),
												 i18n("Write error !"));
		}

  modified=false;
	doc_url = url;
  return true;
}

void KPoGreDoc::deleteContents()
{
  /////////////////////////////////////////////////
  // TODO: Add implementation to delete the document contents
  /////////////////////////////////////////////////

}
