/***************************************************************************
                          kpogreview.h  -  description
                             -------------------
    begin                : Fre Sep  7 16:06:25 CEST 2001
    copyright            : (C) 2002 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPOGREVIEW_H
#define KPOGREVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// include files for libpgxx 
#include "pqxx/util.h" 

// include files for Qt
#include <qwidget.h>
#include <qlistview.h>
#include <qtextview.h>
#include <qmultilineedit.h>
#include <qpushbutton.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qtable.h>
#include <qpopupmenu.h>
#include <qkeycode.h>
#include <qsplitter.h>
#include <qlayout.h>
#include <qlistview.h>
#include <qvaluelist.h>
#include <qdialog.h>
#include <qlineedit.h>
#include <qplatinumstyle.h>
#include <qpixmap.h>
#include <qcolor.h>
#include <qfont.h>
#include <qmessagebox.h>
#include <qtabwidget.h>
#include <qcombobox.h>
#include <qvaluestack.h>

//includes files for kde
#include <kmessagebox.h>
#include <klocale.h>

#include "DbObjects/kpgtreeitem.h"

class KPoGreDoc;
class KPGSqlQueryResultWindow;
class KPGSchemaPropPage;
class KPGServer;
class KPGDatabase;
class KPGSchema;
class KPGTable;

/**
Storage class for OID and their object path. Used for navigating through OIDs.

@author Lumir Vanek
*/

class KPGOidLink
{
public:
		KPGOidLink() {;} 
    KPGOidLink(pqxx::oid _oid, const QString & strPath)
		{
				m_oid = _oid;
				m_strPath = strPath;
		}
					
	~KPGOidLink() {;}

	// Return PostgreSQL row identifier
	const pqxx::oid oid() const { return m_oid; }
		
	// Return path
	const QString & path() const { return m_strPath; }
	
protected:

	QString m_strPath;

	// PostgreSQL row identifier
	pqxx::oid m_oid;		
};



/** The KPoGreView class provides the view widget for the KPoGreApp instance.	
 * The View instance inherits QWidget as a base class and represents the view object of a KTMainWindow. As KPoGreView is part of the
 * docuement-view model, it needs a reference to the document object connected with it by the KPoGreApp class to manipulate and display
 * the document structure provided by the KPoGreDoc class.
 * 	
 * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
 * @version KDevelop version 0.4 code generation
 */
class KPoGreView : public QWidget
{
  Q_OBJECT
  public:
    /** Constructor for the main view */
    KPoGreView(QWidget *parent = 0, const char *name=0);
    /** Destructor for the main view */
    ~KPoGreView();
		
/////////////////////////////////////////////////////////////////////
//                     PROTECTED ATTRIBUTES
/////////////////////////////////////////////////////////////////////
protected:

  // Right tab for properties QTable and 10 SQL query windows
  QTabWidget*  m_pTabPropertiesAndTerminals;
  
  // Display properties for acually selected item in left tree
  QWidget*	m_pWidgetProperties;
	
	// Pointer to array of KPGSqlQueryResultWindow * pointers to query/result windows
  KPGSqlQueryResultWindow** m_ppTerminals;
	
	// Vertical splitter betwwen m_pViewTree and properties window
	QSplitter*	m_pSplitterleft;
	
	// Left tree view that displays DB objects tree	
	QListView*  m_pViewTree;

	// Two stask for storing navigation history through OID links
	QValueStack<KPGOidLink> m_stackBackNodes;
	QValueStack<KPGOidLink> m_stackForwardNodes;	
  
  // Change cursor look
  void setWaitCursor();
  void setNormalCursor();
	
/////////////////////////////////////////////////////////////////////
//                     PUBLIC FUNCTIONS
/////////////////////////////////////////////////////////////////////
public:

    /** returns a pointer to the document connected to the view instance. Mind that this method requires a KPoGreApp instance as a parent
    widget to get to the window document pointer by calling the KPoGreApp::getDocument() method.
    @see KPoGreApp#getDocument
    */
  KPoGreDoc *getDocument() const;
		
  /** contains the implementation for printing functionality */
  void print(QPrinter *pPrinter);
	
  // Set the given sql-string from an sqlfile on inputArea
  void setSQLText(const QString sqlText);

	// Return SQL test from input area
	const QString getSQLText() const;

  // Return tree view
  QListView * getTreeView() const { return m_pViewTree; }

  // Return the currently selected result table (e.g. for saving the content to a file
	QTable* getCurrentlySelectedTableResult();
 
	
/////////////////////////////////////////////////////////////////////
//                     PROTECTED FUNCTIONS
/////////////////////////////////////////////////////////////////////	
protected:

	// Select given tree item
	void selectItem(KPGTreeItem * const);
	
	// Select item defined bu OID and path
	KPGTreeItem* findItem(const KPGOidLink & oidLink);
	
	// Walk through listView items
	KPGTreeItem * nextTreeItem(const KPGTreeItem *);
		
	// Create Properties tab, with regards to selected type in tree
  void createPropertiesTab(const KPGTreeItem::eNodeType);
	
	// Send SQL query to server
	void runQuery(const QString &);
	
	// Display SQL result
  void displayResult(pqxx::result, QTable*, const QString &);
	
	// Find parent database and schemas list for item
	KPGDatabase * lookUpParentDatabaseAndListOfSchemas(KPGTreeItem *, QPtrList<QListViewItem> &);
	
	// Refesh items child folders with given name
	void refreshItemsFolder(KPGTreeItem *, const QString &);
	
public:	
/////////////////////////////////////////////////////////////////////
//                     ACTION FUNCTIONS
/////////////////////////////////////////////////////////////////////
	
	// Called when user choose bookmark
  void openUrl( const QString& );
	
	// Connect to PostgreSQL server
	void databaseConnect();
	
	// Disconnect from selected server
	void databaseDisconnect();
	
	// Send text from selected Inputarea to server
	void databaseRunQuery();
	
	// Run Wizard
	void databaseWizard();
	
	// Called when user launch Find or Find Next menu item 
	void editFind(const QString &, bool, bool);
	
	// Clear inputarea in SQL query widget
	void editClearInputArea();
	
	// Deselects currently selected item in trewe viw
	void editDeselect();
	
	// Undo last edit action in SQL query edit-widget
	void editUndo();
		 
	// Redo last edit action in SQL query edit-widget
	void editRedo();
	
	// Cut text from SQL query widget
	void editCut();
	
	// Copy text from SQL query widget
	void editCopy();
	
	// Paste text to SQL query widget
	void editPaste();
	
	// Navigate to parent item
	void viewNodeUp();
	
	// Navigate back
	void viewNodeBack();
	
	// Navigate forward
	void viewNodeForward();	
		
	// Refresh current item
	void refresh();
	
	// Select item in tree view by given path 
	bool selectItemByPath(const QString &);
	
	//--- Database related menu actions
  void createDatabase();
	void dropDatabase();
  void analyseDatabase();
  void reindexDatabase();
  void setOwnerDatabase();
  void disconnectDatabase();
	
	//--- Schema related menu actions
  void createSchema();
	void dropSchema();
	void modifySchema();
  void setOwnerSchema();
	
	//--- Table related menu actions
  void createTable();
	void dropTable();
  void renameTable();
  void analyseTable();
  void vacuumTable();
  void reindexTable();
  void setClusterOnTable();
  void setWithoutClusterTable();
  void clusterTable();
  void copyFromTable();
  void copyToTable();
  void setOwnerTable();
  void setTablespaceTable();
  void setWithoutOIDsTable();
  
	//--- View related menu actions
	void createView();
	void dropView();
	void modifyView();
	  
  //--- Tablespaces related menu actions
  void createTablespace();
  void dropTablespace();
  void renameTablespace();
  void setOwnerTablespace();
  
  //--- User related menu actions
	void createUser();
	void dropUser();
	void modifyUser();
				
	//--- Group related menu actions
	void createGroup();
	void dropGroup();
	void modifyGroup();
    
  //--- Language related menu actions
  void createLanguage();
  void dropLanguage();
  void modifyLanguage();
  
  //--- Domain related menu actions
  void createDomain();
  void dropDomain();
  void setDomainNull();
  void setDomainDefault();
  void dropDomainDefault();
  void setOwnerDomain();
  
  //--- Domain constraint related menu actions
  void createDomainConstraint();
  void dropDomainConstraint();
    
  //--- Function related menu actions
  void createFunction();
  void dropFunction();
  void renameFunction();
  void modifyFunction();
    
  //--- Sequence related menu actions
  void createSequence();
  void dropSequence();
  void modifySequence();
  
  //--- Column related menu actions
  void createColumn();
  void dropColumn();
  void renameColumn();
  void setColumnNull();
  void setColumnDefault();
  void dropColumnDefault();
  void changeColumnType();
  void setColumnStorage();
  void setColumnStatistisc();
  
  //--- Constraint related menu actions
  void createConstraint();
  void dropConstraint();
  
	//--- Index related menu actions
  void createIndex();
  void dropIndex();
  void renameIndex();
  void reindexIndex();
  void setOwnerIndex();
  void setTablespaceIndex();
  
  //--- Rule related menu actions 
  void createRule();
  void dropRule();
    
  //--- Trigger related menu actions
  void createTrigger();
  void dropTrigger();
  void renameTrigger();
  
  //--- Type related menu actions
  void createBaseType();
  void createCompositeType();
  void dropType();
  void setOwnerType();
  	
/////////////////////////////////////////////////////////////////////
//                           OTHER SLOTS
/////////////////////////////////////////////////////////////////////
	
protected slots:		
	
	
	// Show popup for item in tree rightpressed
	void slotTreeViewRightButtonPressed(QListViewItem*, const QPoint&, int);
	
	// Called when user change main tab
	void slotTabWidgetCurrentChanged( QWidget *);
	
	// Called when user change main tab inside schema properties window
  void slotSchemaPropTabChanged(KPGSchemaPropPage *, QWidget *);
  
  // Checks, which type of item was selected
	void slotSelectionChanged();
	
	// Caled when user change selected server in SQL query / result window
	void slotRequestDatabasesList(KPGSqlQueryResultWindow *, const QString&);
	
	// Called from property pages when user double-clicks on KPGLinkLabel
	void slotSearchObject(const pqxx::oid);
	
	// Called when user change SQL query in editor
	void slotQueryTextChanged(KPGSqlQueryResultWindow *);
	
	// Called when availability of copy action is changed in SQL query editor
	void slotCopyAvailable(KPGSqlQueryResultWindow *, bool);
	
	// Called when availability of undo action is changed in SQL query editor
	void slotUndoAvailable(KPGSqlQueryResultWindow *, bool);
	
	// Called when availability of redo action is changed in SQL query editor
	void slotRedoAvailable(KPGSqlQueryResultWindow *, bool);

	
/////////////////////////////////////////////////////////////////////
//                       WIZARDS FUNCTIONS
/////////////////////////////////////////////////////////////////////		
	
  bool createTablespaceWizard(KPGServer *);
  
	bool createDatabaseWizard(KPGServer *);
	
	bool createSchemaWizard(KPGDatabase *);

	bool createGroupWizard(KPGServer *);
	
	bool createUserWizard(KPGServer *);
		
	bool createTableWizard(KPGDatabase *, QPtrList<QListViewItem> &);

  bool createBaseTypeWizard(KPGDatabase *, QPtrList<QListViewItem> &);
  
  bool createCompositeTypeWizard(KPGDatabase *, QPtrList<QListViewItem> &);
  
	bool createViewWizard(KPGDatabase *, QPtrList<QListViewItem> &);
  
  bool createDomainWizard(KPGDatabase *, QPtrList<QListViewItem> &);
  
  bool createFunctionWizard(KPGDatabase *, QPtrList<QListViewItem> &);
  
	bool createSequenceWizard(KPGDatabase *, QPtrList<QListViewItem> &);
	
  bool createLanguageWizard(KPGDatabase *);
  
  bool createColumnWizard(KPGDatabase *, KPGSchema *, KPGTable *);
  
  bool createConstraintWizard(KPGDatabase *, KPGSchema *, KPGTable *, int = 0);

	bool createIndexWizard(KPGDatabase *, KPGSchema *, KPGTable *);
  
  bool createRuleWizard(KPGDatabase *, KPGSchema *, KPGTable *);
  
  bool createTriggerWizard(KPGDatabase *, KPGSchema *, KPGTable *);
		
	// Run simple 2-steps for dropping DB object
	bool dropObjectWizard(KPGTreeItem *, KPGTreeItem::eNodeType, const char *, KPGConnection *,	const QString &, bool = true);
	
	// Run simple 2-steps for dropping Table child object
	bool dropTableObjectWizard(KPGTreeItem *, const char *, KPGConnection *, const QString &, const QString &, bool = true);

/////////////////////////////////////////////////////////////////////
//                           SIGNALS
/////////////////////////////////////////////////////////////////////
signals:

	/** emitted, when an tree item is selected */
	void sigSelectionChanged( const KPGTreeItem::eNodeType, KPGTreeItem *, unsigned int, unsigned int);

	// Request for enabling Run Query action
  void sigEnableRunQueryAction(bool);
	
	// Request for enabling File Open, Save and Export action
	void sigEnableFileActions(bool, bool, bool);
	
	// Request for enabling Edit actions
	void sigEnableEditActions(bool, bool, bool, bool, bool, bool);
	
	/** Emitted, when a context menu is requested */
	void sigContextMenuRequested(const QString & szMenuName, const QPoint & pos );
};

#endif // KPOGREVIEW_H
